/*
All material copyright ESRI, All Rights Reserved, unless otherwise specified.
See https://js.arcgis.com/4.31/esri/copyright.txt for details.
*/
define(["exports","../../../../../../core/libs/gl-matrix-2/factories/vec3f64","../../../../../../geometry/support/Ellipsoid","../../../../environment/Clouds","../../../../environment/CloudsParameters","../../../../environment/weather","../shading/MainLighting.glsl","../../shaderModules/BooleanPassUniform","../../shaderModules/Float3PassUniform","../../shaderModules/FloatPassUniform","../../shaderModules/interfaces","../../shaderModules/Matrix4PassUniform","../../shaderModules/TextureCubePassUniform"],(function(o,a,e,t,r,i,n,l,s,d,c,u,m){"use strict";function h(o){const e=o.fragment;e.constants.add("radiusCloudsSquared","float",C).code.add(c.glsl`vec3 intersectWithCloudLayer(vec3 dir, vec3 cameraPosition, vec3 spherePos) {
float B = 2.0 * dot(cameraPosition, dir);
float C = dot(cameraPosition, cameraPosition) - radiusCloudsSquared;
float det = B * B - 4.0 * C;
float pointIntDist = max(0.0, 0.5 *(-B + sqrt(det)));
return (cameraPosition + dir * pointIntDist) - spherePos;
}`),e.uniforms.add(new d.FloatPassUniform("radiusCurvatureCorrection",((o,a)=>a.clouds.parallax.radiusCurvatureCorrection))).code.add(c.glsl`vec3 correctForPlanetCurvature(vec3 dir) {
dir.z = dir.z * (1.0 - radiusCurvatureCorrection) + radiusCurvatureCorrection;
return dir;
}`),e.code.add(c.glsl`vec3 rotateDirectionToAnchorPoint(mat4 rotMat, vec3 inVec) {
return (rotMat * vec4(inVec, 0.0)).xyz;
}`),n.addMainLightDirection(e),n.addMainLightIntensity(e);const i=a.fromValues(.28,.175,.035);e.constants.add("RIM_COLOR","vec3",i);const h=.3,g=140,f=.2,v=10,P=.3;e.code.add(c.glsl`
    vec3 calculateCloudColor(vec3 cameraPosition, vec3 worldSpaceRay, vec4 clouds) {
      float upDotLight = dot(cameraPosition, mainLightDirection);
      float dirDotLight = max(dot(worldSpaceRay, mainLightDirection), 0.0);
      float sunsetTransition = clamp(pow(max(upDotLight, 0.0), ${c.glsl.float(h)}), 0.0, 1.0);

      // Base color of the clouds that depends on lighting of the sun and sky
      vec3 ambientLight = calculateAmbientIrradiance(cameraPosition,  0.0);
      vec3 combinedLight = clamp((mainLightIntensity + ambientLight )/PI, vec3(0.0), vec3(1.0));
      vec3 baseCloudColor = pow(combinedLight * pow(clouds.xyz, vec3(GAMMA)), vec3(INV_GAMMA));

      // Rim light around the edge of the clouds simulating scattering of the direct lun light
      float scatteringMod = max(clouds.a < 0.5 ? clouds.a / 0.5 : - clouds.a / 0.5 + 2.0, 0.0);
      float rimLightIntensity = 0.5 + 0.5 * pow(max(upDotLight, 0.0), 0.35);
      vec3 directSunScattering = RIM_COLOR * rimLightIntensity * (pow(dirDotLight, ${c.glsl.float(g)})) * scatteringMod;

      // Brighten the clouds around the sun at the sunsets
      float additionalLight = ${c.glsl.float(f)} * pow(dirDotLight, ${c.glsl.float(v)}) * (1. - pow(sunsetTransition, ${c.glsl.float(P)})) ;

      return vec3(baseCloudColor * (1.0 + additionalLight) + directSunScattering);
    }
  `),e.uniforms.add(new l.BooleanPassUniform("readChannelsRG",((o,a)=>a.clouds.readChannels===t.CloudsTextureChannels.RG)),new m.TextureCubePassUniform("cubeMap",((o,a)=>a.clouds.data?.cubeMap?.colorTexture??null))).code.add(c.glsl`vec4 sampleCloud(vec3 rayDir, bool readOtherChannel) {
vec4 s = texture(cubeMap, rayDir);
bool readRG = readChannelsRG ^^ readOtherChannel;
s = readRG ? vec4(vec3(s.r), s.g) : vec4(vec3(s.b), s.a);
return length(s) == 0.0 ? vec4(s.rgb, 1.0) : s;
}`),e.uniforms.add(new s.Float3PassUniform("anchorPoint",((o,a)=>a.clouds.parallax.anchorPoint)),new s.Float3PassUniform("anchorPointNew",((o,a)=>a.clouds.parallaxNew.anchorPoint)),new u.Matrix4PassUniform("rotationClouds",((o,a)=>a.clouds.parallax.transform)),new u.Matrix4PassUniform("rotationCloudsNew",((o,a)=>a.clouds.parallaxNew.transform)),new d.FloatPassUniform("cloudsOpacity",((o,a)=>a.clouds.opacity)),new d.FloatPassUniform("fadeFactor",((o,a)=>a.clouds.fadeFactor)),new l.BooleanPassUniform("crossFade",((o,a)=>a.clouds.fadeState===r.FadeState.CROSS_FADE))).code.add(c.glsl`vec4 renderClouds(vec3 worldRay, vec3 cameraPosition) {
vec3 intersectionPoint = intersectWithCloudLayer(worldRay, cameraPosition, anchorPoint);
vec3 worldRayRotated = rotateDirectionToAnchorPoint(rotationClouds, normalize(intersectionPoint));
vec3 worldRayRotatedCorrected = correctForPlanetCurvature(worldRayRotated);
vec4 cloudData = sampleCloud(worldRayRotatedCorrected, crossFade);
vec3 cameraPositionN = normalize(cameraPosition);
vec4 cloudColor = vec4(calculateCloudColor(cameraPositionN, worldRay, cloudData), cloudData.a);
if(crossFade) {
intersectionPoint = intersectWithCloudLayer(worldRay, cameraPosition, anchorPointNew);
worldRayRotated = rotateDirectionToAnchorPoint(rotationCloudsNew, normalize(intersectionPoint));
worldRayRotatedCorrected = correctForPlanetCurvature(worldRayRotated);
cloudData = sampleCloud(worldRayRotatedCorrected, false);
vec4 cloudColorNew = vec4(calculateCloudColor(cameraPositionN, worldRay, cloudData), cloudData.a);
cloudColor = mix(cloudColor, cloudColorNew, fadeFactor);
}
float totalTransmittance = length(cloudColor.rgb) == 0.0 ?
1.0 :
clamp(cloudColor.a * cloudsOpacity + (1.0 - cloudsOpacity), 0.0 , 1.0);
return vec4(cloudColor.rgb, totalTransmittance);
}`)}const C=(e.earth.radius+i.cloudsHeight)**2;o.CloudsParallaxShading=h,Object.defineProperty(o,Symbol.toStringTag,{value:"Module"})}));
