/*
All material copyright ESRI, All Rights Reserved, unless otherwise specified.
See https://js.arcgis.com/4.31/esri/copyright.txt for details.
*/
define(["exports"],(function(e){"use strict";const i={"applicationProperties_schema.json":{title:"applicationProperties",type:"object",description:"Configuration of application and UI elements.",properties:{viewing:{type:"object",$ref:"#/definitions/viewing_schema.json"}},additionalProperties:!1},"baseMap_schema.json":{title:"baseMap",type:"object",description:"The basemap provides geographic context to the map and scene.",properties:{baseMapLayers:{type:"array",description:"An array of baseMapLayer objects defining the basemaps used in the web scene.",items:{$ref:"#/definitions/baseMapLayer_schema.json"},uniqueItems:!0},elevationLayers:{type:"array",description:'<em>Deprecated, use `ground.layers` instead <a href="#elevationLayers"><sup>1</sup></a>.</em> An array of elevationLayer objects defining the basemaps used in the web scene.',items:{$ref:"#/definitions/elevationLayers_schema.json"},uniqueItems:!0},id:{type:"string",description:"A unique identifying string for the basemap."},title:{type:"string",description:"Required string title for the basemap that can be used in a table of contents."}},required:["baseMapLayers","title"],additionalProperties:!1},"clippingArea_schema.json":{title:"clippingArea",type:"object",description:"Defines area to be clipped for display.",properties:{clip:{type:"boolean",description:"enable / disable clipping"},geometry:{description:"envelope of clip area",$ref:"#/definitions/extent_schema.json"}}},"ground_schema.json":{title:"Ground",type:"object",description:"Ground defines the main surface of the web scene, based on elevation layers.",properties:{layers:{type:"array",description:"An array of elevationLayer objects defining the elevation of the ground in the web scene.",items:{$ref:"#/definitions/elevationLayers_schema.json"},uniqueItems:!0},navigationConstraint:{description:"Object determining whether the camera is constrained to navigate only above, or also under the ground surface. If not specified, navigation is constrained to above ground.",$ref:"#/definitions/navigationConstraint_schema.json"},surfaceColor:{description:"Defines the color of the ground surface, displayed underneath the basemap. If no color, the default grid is shown.",$ref:"#/definitions/color_schema.json"},transparency:{type:"integer",description:"The transparency of the ground surface. It is used for seeing through the ground, therefore this property also changes the transparency of the basemap. Draped operational layers are not affected by this property. The value has to lie between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100}},required:["layers"],additionalProperties:!1},"heightModelInfo_schema.json":{title:"heightModelInfo",type:"object",description:"An object that defines the characteristics of the vertical coordinate system used by the web scene.",properties:{heightModel:{type:"string",description:"The surface type or height model of the vertical coordinate system.",enum:["ellipsoidal","gravity_related_height"],default:"ellipsoidal"},heightUnit:{type:"string",description:'The unit of the vertical coordinate system.<a href="#heightUnit"><sup>1</sup></a>',enum:["150-kilometers","50-kilometers","benoit-1895-b-chain","clarke-foot","clarke-link","clarke-yard","foot","gold-coast-foot","indian-1937-yard","indian-yard","meter","sears-1922-truncated-chain","sears-chain","sears-foot","sears-yard","us-foot"]},vertCRS:{type:"string",description:"(Optional) The datum realization of the vertical coordinate system."}},required:["heightModel","heightUnit"],additionalProperties:!1},"initialState_schema.json":{title:"initialState",type:"object",description:"An object that provides information about the initial environment settings and viewpoint of the web scene.",properties:{environment:{type:"object",$ref:"#/definitions/environment_schema.json"},timeExtent:{description:"Represents the time extent for the data displayed on the scene.",$ref:"#/definitions/timeExtent_schema.json"},viewpoint:{description:"Describes a point of view for a 2D or 3D view. In a 3D view, it is determined using a camera position.",$ref:"#/definitions/viewpoint_schema.json"}},required:["environment","viewpoint"],additionalProperties:!1},"mapFloorInfo_schema.json":{title:"mapFloorInfo",type:"object",description:"Contains floor-awareness information for the web scene. Defines the layers and required fields for each layer that are used for floor filtering.",properties:{facilityLayer:{description:"Defines the layer and field properties for the Facility layer used for floor filtering.",$ref:"#/definitions/facilityLayerInfo_schema.json"},levelLayer:{description:"Defines the layer and field properties for the Level layer used for floor filtering.",$ref:"#/definitions/levelLayerInfo_schema.json"},siteLayer:{description:"Defines the layer and field properties for the Site layer used for floor filtering. This property is optional.",$ref:"#/definitions/siteLayerInfo_schema.json"}},required:["facilityLayer","levelLayer"],additionalProperties:!1},"mapRangeInfo_schema.json":{title:"Map Range Information",description:"Map range information",properties:{activeRangeName:{type:"string",description:"Active range ID that slider/picker acts upon."},currentRangeExtent:{type:"array",description:"Current range for the active range.",items:{type:"number"},minItems:2,maxItems:2},fullRangeExtent:{type:"array",description:"Full range extent for the active range to be presented in the UI.",items:{type:"number"},minItems:2,maxItems:2}},required:["activeRangeName"],additionalProperties:!1},"operationalLayers_schema.json":{title:"operationalLayers",type:"object",description:"Operational layers contain your data. Usually, a basemap sits beneath your operational layers to give them geographic context.",oneOf:[{$ref:"#/definitions/buildingSceneLayer_schema.json"},{$ref:"#/definitions/catalogLayer_schema.json"},{$ref:"#/definitions/csvLayer_schema.json"},{$ref:"#/definitions/dimensionLayer_schema.json"},{$ref:"#/definitions/featureLayer_schema.json"},{$ref:"#/definitions/geojsonLayer_schema.json"},{$ref:"#/definitions/groupLayer_schema.json"},{$ref:"#/definitions/imageServiceLayer_schema.json"},{$ref:"#/definitions/integratedMesh3DTilesLayer_schema.json"},{$ref:"#/definitions/integratedMeshLayer_schema.json"},{$ref:"#/definitions/kmlLayer_schema.json"},{$ref:"#/definitions/lineOfSightLayer_schema.json"},{$ref:"#/definitions/mapServiceLayer_schema.json"},{$ref:"#/definitions/mediaLayer_schema.json"},{$ref:"#/definitions/ogcFeatureLayer_schema.json"},{$ref:"#/definitions/orientedImageryLayer_schema.json"},{$ref:"#/definitions/pointCloudLayer_schema.json"},{$ref:"#/definitions/rasterDataLayer_schema.json"},{$ref:"#/definitions/sceneLayer_schema.json"},{$ref:"#/definitions/tiledImageServiceLayer_schema.json"},{$ref:"#/definitions/tiledMapServiceLayer_schema.json"},{$ref:"#/definitions/vectorTileLayer_schema.json"},{$ref:"#/definitions/viewshedLayer_schema.json"},{$ref:"#/definitions/voxelLayer_schema.json"},{$ref:"#/definitions/webTiledLayer_schema.json"},{$ref:"#/definitions/wfsLayer_schema.json"},{$ref:"#/definitions/wmsLayer_schema.json"}]},"presentation_schema.json":{title:"presentation",type:"object",description:"A presentation consists of multiple slides, where each slide is a specific view into the web scene.",properties:{slides:{type:"array",description:"Array of slide objects.",items:{$ref:"#/definitions/slide_schema.json"}}},additionalProperties:!1},"spatialReference_schema.json":{title:"spatialReference",type:"object",description:"The spatialReference object is located at the top level of the JSON hierarchy. In addition to this, it is also available within the operationalLayer and basemap objects.\n\nMany predefined spatial references have already been defined and are available for use. A spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT). The default tolerance and resolution values for the associated coordinate system are used.\n\nA spatial reference can optionally include a definition for a vertical coordinate system (VCS), which is used to interpret the z values of a geometry.",properties:{latestVcsWkid:{type:"integer",description:"(Optional) The current wkid value of the vertical coordinate system."},latestWkid:{type:"integer",description:"(Optional) Identifies the current wkid value associated with the same spatial reference. For example a WKID of '102100' (Web Mercator) has a latestWKid of '3857'."},vcsWkid:{type:"integer",description:"(Optional) The wkid value of the vertical coordinate system."},wkid:{type:"integer",description:"The well-known ID (WKID) of the coordinate system. Specify either WKID or the well-known text (WKT) of the coordinate system."},wkt:{type:"string",description:"The well-known text (WKT) of the coordinate system. Specify either WKT or WKID of the coordinate system."}},oneOf:[{required:["wkt"]},{required:["wkid"]}],additionalProperties:!1},"table_schema.json":{title:"table",type:"object",description:"Root element in the web scene specifying an array of table objects.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item."},id:{type:"string",description:"Unique string identifier for the table."},itemId:{type:"string",description:"Unique string value indicating an item registered in ArcGIS Online or your organization's portal."},layerDefinition:{description:"Additional properties that define configurations for the table.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{capabilities:{},definitionExpression:{}}}]},path:{type:"string",description:"For offline data, a path to a geodatabase file. A URI format is used starting with `file:` followed by a file system path with a `.geodatabase` extension. A query parameter `itemId` must also be present specifying the ID of a table in the geodatabase's `GDB_ServiceItems` table. A relative path must be from the file which defines the table. For example `file:../p20/northamerica.geodatabase?itemId=5`.",pattern:"^file:.+\\.geodatabase\\?itemId=\\d+$"},popupInfo:{description:"An object defining the content of popup windows when you query a record and the sort option for child related records.",$ref:"#/definitions/popupInfo_schema.json"},title:{type:"string",description:"String value for the title of the table."},url:{type:"string",description:"String value indicating the URL reference of the hosted table."}},required:["title","url"],additionalProperties:!1},"transportationNetwork_schema.json":{title:"Transportation Network",type:"object",description:"The transportation network used for routing in the scene.",properties:{name:{type:"string",description:"The name of the transportation network. The name must be unique within a scene."},path:{type:"string",description:"The path to the geodatabase which contains the network.",pattern:"^file:.+\\.geodatabase?$"},title:{type:"string",description:"A title or alias of the network layer. This can be used in the client user interface."}},required:["name","path","title"],additionalProperties:!1},"version_schema.json":{title:"version",type:"string",description:"Root element in the web scene specifying a string value indicating the web scene version.",enum:["1.8","1.9","1.10","1.11","1.12","1.13","1.14","1.15","1.16","1.17","1.18","1.19","1.20","1.21","1.22","1.23","1.24","1.25","1.26","1.27","1.28","1.29","1.30","1.31","1.32","1.33","1.34","1.35"]},"widgets_schema.json":{title:"widgets",type:"object",description:"The widgets object contains widgets that should be exposed to the user.",properties:{floorFilter:{description:"Configurable properties used by the floor filter widget.",$ref:"#/definitions/floorFilter_schema.json"},range:{description:"Active range.",$ref:"#/definitions/range_schema.json"},timeSlider:{type:"object",description:"Time animation is controlled by a configurable time slider. The time slider widget will act upon all the time aware layers in the scene.",$ref:"#/definitions/timeSlider_schema.json"}},additionalProperties:!1},"customParameters_schema.json":{title:"customParameters",type:"object",description:"A sequence of custom parameters appended to the URL of all requests related to a layer.",patternProperties:{".*":{type:"string"}}},"buildingSceneLayer_filter_schema.json":{title:"Building Scene Layer Filter",type:"object",description:"Filter allows client applications to reduce the drawn elements of a building to specific types and values. Filters on the webscene override the list of existing filters on the service.",properties:{description:{type:"string",description:"Description of the filter."},filterAuthoringInfo:{description:"Metadata about the athoring process for this filter.",$ref:"#/definitions/buildingSceneLayer_filterAuthoringInfoCheckbox_schema.json"},filterBlocks:{type:"array",description:"Array of filter blocks defining the filter. A filter contains at least one filter block.",items:{$ref:"#/definitions/buildingSceneLayer_filterBlock_schema.json"},minItems:1},id:{type:"string",description:"unique filter id (uuid). Either a new id to extend the list of filters, or an existing id to override properties of an existing filter."},name:{type:"string",description:"Name of the filter."}},required:["filterBlocks","id","name"],additionalProperties:!1},"layerDefinition_schema.json":{title:"layerDefinition",type:"object",description:"An object that defines the attribute schema and drawing information for a layer drawn using client-side graphics.",properties:{allowGeometryUpdates:{type:"boolean",description:"Boolean value indicating whether the geometry of the features in the layer can be edited."},capabilities:{type:"string",description:"A comma separated list of supported capabilities, e.g. `Query,Editing`."},copyrightText:{type:"string",description:"String value for the copyright text information for the layer."},currentVersion:{type:"number",description:"Numeric value indicating the server version of the layer."},defaultVisibility:{type:"boolean",description:"Boolean value indicating whether the layer's visibility is turned on."},definitionExpression:{type:["string","null"],description:"SQL-based definition expression string that narrows the data to be displayed in the layer."},description:{type:"string",description:"String value of the layer as defined in the map service."},displayField:{type:"string",description:"A string value that summarizes the feature."},drawingInfo:{description:"Contains the drawing and labeling information.",$ref:"#/definitions/drawingInfo_schema.json"},elevationInfo:{$ref:"#/definitions/elevationInfo_schema.json"},excludeObjectIds:{type:"array",description:"Excludes features with the given object ids from rendering.",items:{type:"number"}},extent:{description:"An object defining the rectangular area.",oneOf:[{$ref:"#/definitions/extent_schema.json"},{type:"null"}]},featureReduction:{$ref:"#/definitions/featureReduction_schema.json"},fields:{type:"array",description:"An array of field objects containing information about the attribute fields for the feature collection or layer.",items:{$ref:"#/definitions/field_schema.json"},uniqueItems:!0},filters:{type:"array",description:"Filters for PointCloud layers",items:{$ref:"#/definitions/pointCloudFilter_schema.json"}},floorInfo:{description:"Contains floor-awareness information for the layer.",$ref:"#/definitions/layerFloorInfo_schema.json"},geometryType:{type:"string",description:"A string defining the type of geometry. Possible geometry types are: `esriGeometryPoint`, `esriGeometryMultipoint`, `esriGeometryPolyline`, `esriGeometryPolygon`, and `esriGeometryEnvelope`."},globalIdField:{type:"string",description:"The unique identifier for a feature or table row within a geodatabase."},hasAttachments:{type:"boolean",description:"Indicates whether attachments should be loaded for the layer."},hasM:{type:"boolean",description:"Boolean value indicating whether layer has M values."},hasStaticData:{type:"boolean",description:"Boolean value indicating whether data changes. True if it does not."},hasZ:{type:"boolean",description:"Boolean value indicating whether layer has Z values."},htmlPopupType:{type:"string",description:"String value indicating the HTML popup type.",enum:["esriServerHTMLPopupTypeAsHTMLText","esriServerHTMLPopupTypeAsURL","esriServerHTMLPopupTypeNone"]},id:{type:"integer",description:"The identifier assigned to the layer."},isDataVersioned:{type:"boolean",description:"Boolean value indicating whether the data is versioned."},maxRecordCount:{type:"integer",description:"Numeric value indicating tbe maximum number of records that will be returned at once for a query."},maxScale:{type:"number",description:"Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification."},minScale:{type:"number",description:"Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification."},name:{type:"string",description:"Contains a unique name for the layer that can be displayed in a legend."},objectIdField:{type:"string",description:"Indicates the name of the object ID field in the dataset."},orderBy:{type:"array",description:"An array of orderByField objects specifying the feature display order. Features can be sorted in ascending or descending order of a numeric or date field only. If `ascending`, features with smaller values will be drawn on top of features with larger values. For date values, `ascending` order means features with older dates will be drawn on top of features with recent dates. If `descending`, the sort behavior is reversed. When this property is not defined, features are displayed in the order in which they are received by the client.",items:{$ref:"#/definitions/orderByField_schema.json"},minItems:1},overrideSymbols:{type:"boolean",description:"Dictates whether a client can support having an end user modify symbols on individual features."},polygonFilter:{description:"[Polygon filter](polygonFilter.md) for scene layer features.",oneOf:[{$ref:"#/definitions/polygonFilter_schema.json"},{type:"null"}]},rangeInfos:{type:"array",description:"Indicates range information",items:{$ref:"#/definitions/rangeInfo_schema.json"},minItems:1},source:{description:"An object indicating the layerDefinition's layer source.",$ref:"#/definitions/source_schema.json"},spatialReference:{description:"An object containing the WKID or WKT identifying the spatial reference of the layer's geometry.",$ref:"#/definitions/spatialReference_schema.json"},supportedQueryFormats:{type:"string",description:"String value indicating the output formats that are supported in a query."},supportsAdvancedQueries:{type:"boolean",description:"Boolean value indicating whether the layer supports orderByFields in a query operation."},supportsAttachmentsByUploadId:{type:"boolean",description:"Boolean value indicating whether the layer supports uploading attachments with the Uploads operation. This can then be used in the Add Attachment and Update Attachment operations."},supportsCalculate:{type:"boolean",description:"Boolean value indicating whether the layer supports the Calculate REST operation when updating features."},supportsRollbackOnFailureParameter:{type:"boolean",description:"Boolean value indicating whether the layer supports rolling back edits made on a feature layer if some of the edits fail."},supportsStatistics:{type:"boolean",description:"Boolean value indicating whether feature layer query operations support statistical functions."},supportsValidateSql:{type:"boolean",description:"Boolean value indicating whether the validateSQL operation is supported across a feature service layer."},type:{type:"string",description:"Indicates whether the layerDefinition applies to a Feature Layer or a Table.",enum:["Feature Layer","Table"],default:"Feature Layer"},typeIdField:{type:"string",description:"Contains the name of the field holding the type ID for the features."},types:{type:"array",description:"Contains information about an attribute field.",items:{$ref:"#/definitions/type_schema.json"},uniqueItems:!0},visibilityField:{type:"string",description:"String value indicating the attribute field that is used to control the visibility of a feature. If applicable, when rendering a feature the client should use this field to control visibility. The field's values are 0 = do not display, 1 = display."}},additionalProperties:!1},"buildingSceneLayer_sublayer_schema.json":{title:"Building Scene Sublayer",type:"object",description:"The BuildingSceneLayer sublayer is a part of a building scene layer.",properties:{disablePopup:{type:"boolean",description:"disablePopups allows a client to ignore popups defined by the service item."},id:{type:"integer",description:"Identifies the sublayer inside the building scene layer."},layerDefinition:{type:"object",description:"Additional properties that can define drawing information and a definition expression for the sublayer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{renderer:{}},additionalProperties:!1}]}},additionalProperties:!1}]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list.",enum:["hide","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the sublayer on the client side, where 0 is full transparency and 1 is no transparency. This is multiplied with the opacity of the containing layers.",minimum:0,maximum:1,default:1},popupInfo:{description:"A popupInfo object defining the content of pop-up windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},title:{type:"string",description:"A user-friendly string title for the sublayer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the sublayer is initially visible in the web scene"}},required:["id"],additionalProperties:!1},"timeExtent_schema.json":{title:"timeExtent",type:"array",description:"Time extent is a two element array that can represent time extent or time instant. The two elements represent values for start and end time. The number represents the number of milliseconds since epoch (January 1, 1970) in UTC. A null as start indicates all inclusive since the beginning of time and a null as end value indicates indefinite time with no end. Time instant can be represented by setting same value for both start and end.",items:{type:["integer","null"]},minItems:2,maxItems:2,additionalProperties:!1},"drawingInfo_schema.json":{title:"drawingInfo",type:"object",description:"The drawingInfo object contains drawing information for a feature layer.",properties:{labelingInfo:{type:"array",description:"An array of labelingInfo objects.",items:{$ref:"#/definitions/labelingInfo_schema.json"}},renderer:{$ref:"#/definitions/renderer_schema.json"},showLabels:{type:"boolean",description:"Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it.",default:!1},transparency:{type:"number",description:"Number value ranging between 0 (no transparency) to 100 (completely transparent).",minimum:0,maximum:100}},additionalProperties:!1},"popupInfo_schema.json":{title:"popupInfo",type:"object",description:"Defines the look and feel of popup windows when a user clicks or queries a feature.",properties:{description:{type:["string"],description:"A string that appears in the body of the popup window as a description. A basic subset of HTML may also be used to enrich the text. The supported HTML for ArcGIS Online can be seen in the [Supported HTML](https://doc.arcgis.com/en/arcgis-online/reference/supported-html.htm) page."},expressionInfos:{type:"array",description:"List of Arcade expressions added to the pop-up.",items:{$ref:"#/definitions/popupExpressionInfo_schema.json"}},fieldInfos:{type:"array",description:"Array of fieldInfo information properties. This information is provided by the service layer definition.",items:{$ref:"#/definitions/fieldInfo_schema.json"}},layerOptions:{description:"Additional options that can be defined for the popup layer.",$ref:"#/definitions/popupLayerOptions_schema.json"},mediaInfos:{type:"array",description:"Array of various mediaInfo to display. Can be of type `image`, `piechart`, `barchart`, `columnchart`, or `linechart`. The order given is the order in which it displays.",items:{$ref:"#/definitions/mediaInfo_schema.json"}},popupElements:{type:"array",description:"An array of popupElement objects that represent an ordered list of popup elements.",items:{$ref:"#/definitions/popupElement_schema.json"}},relatedRecordsInfo:{description:"Indicates whether to enable related records if they exist on a layer.",$ref:"#/definitions/relatedRecordsInfo_schema.json"},showAttachments:{type:"boolean",description:"Indicates whether attachments will be loaded for feature layers that have attachments."},showLastEditInfo:{type:"boolean",description:"Indicates whether popup will display information about when and who last edited the feature. Applicable only to layers that have been configured to keep track of such information.",default:!0},title:{type:"string",description:"A string that appears at the top of the popup window as a title."}},additionalProperties:!1},"viewing_schema.json":{title:"viewing",type:"object",description:"View-specific properties of application and UI elements.",properties:{search:{$ref:"#/definitions/search_schema.json"}},additionalProperties:!1},"baseMapLayer_schema.json":{title:"baseMapLayer",type:"object",description:"A basemap layer is a layer that provides geographic context to the scene. The following is a list of possible basemap layer types.",oneOf:[{$ref:"#/definitions/imageServiceLayer_schema.json"},{$ref:"#/definitions/mapServiceLayer_schema.json"},{$ref:"#/definitions/openStreetMapLayer_schema.json"},{$ref:"#/definitions/rasterDataLayer_schema.json"},{$ref:"#/definitions/sceneLayer_schema.json"},{$ref:"#/definitions/tiledImageServiceLayer_schema.json"},{$ref:"#/definitions/tiledMapServiceLayer_schema.json"},{$ref:"#/definitions/vectorTileLayer_schema.json"},{$ref:"#/definitions/webTiledLayer_schema.json"},{$ref:"#/definitions/wmsLayer_schema.json"}]},"elevationLayers_schema.json":{title:"elevationLayers",type:"object",description:"Elevation layers provide elevation for the ground.",oneOf:[{$ref:"#/definitions/rasterDataElevationLayer_schema.json"},{$ref:"#/definitions/tiledElevationLayer_schema.json"}]},"extent_schema.json":{title:"extent",type:"object",description:"This object defines the bounding geometry given the lower-left and upper-right corners of the bounding box. A [spatial reference](spatialReference.md) is also required.",properties:{spatialReference:{description:"An object used to specify the spatial reference of the given geometry.",$ref:"#/definitions/spatialReference_schema.json"},xmax:{type:"number",description:"A numeric value indicating the top-right X-coordinate of an extent envelope."},xmin:{description:"A numeric value indicating the bottom-left X-coordinate of an extent envelope.",oneOf:[{type:"number"},{type:"null"},{type:"string",enum:["NaN"]}]},ymax:{type:"number",description:"A numeric value indicating the top-right Y-coordinate of an extent envelope."},ymin:{type:"number",description:"A numeric value indicating the bottom-left Y-coordinate of an extent envelope."}},additionalProperties:!1},"navigationConstraint_schema.json":{title:"Navigation Constraint",type:"object",description:"Object determining whether the camera is constrained to navigate only above, or also under the ground surface.",properties:{type:{type:"string",enum:["none","stayAbove"]}},required:["type"],additionalProperties:!1},"color_schema.json":{title:"color",type:"array",description:"Color is represented as a three or four-element array. The four elements represent values for red, green, blue, and alpha in that order. Values range from 0 through 255. If color is undefined for a symbol, the color value is null.",items:{type:"integer",minimum:0,maximum:255},minItems:3,maxItems:4,additionalProperties:!1},"environment_schema.json":{title:"environment",type:"object",description:"Represents settings that affect the environment in which the web scene is displayed. It is entirely stored as part of the initial state of the web scene, and partially in the slides in the presentation.",properties:{atmosphereEnabled:{type:"boolean",description:"Whether the atmosphere should be visualized. This includes sky and haze effects.",default:!0},background:{description:"The background is what is displayed behind any scene objects, stars and atmosphere.",$ref:"#/definitions/environment_background_color_schema.json"},lighting:{$ref:"#/definitions/lighting_schema.json"},starsEnabled:{type:"boolean",description:"Whether stars should be displayed in the sky.",default:!0},weather:{type:"object",description:"Indicates the type of weather visualization in the scene.",oneOf:[{$ref:"#/definitions/cloudyWeather_schema.json"},{$ref:"#/definitions/foggyWeather_schema.json"},{$ref:"#/definitions/rainyWeather_schema.json"},{$ref:"#/definitions/snowyWeather_schema.json"},{$ref:"#/definitions/sunnyWeather_schema.json"}],default:{$ref:"#/definitions/sunnyWeather_schema.json"}}},additionalProperties:!1},"viewpoint_schema.json":{title:"viewpoint",type:"object",description:"The location or camera position from which to view the scene.",properties:{camera:{$ref:"#/definitions/camera_schema.json"},rotation:{type:"number",description:"The rotation of due north in relation to the top of the view in degrees.",minimum:0,maximum:360},scale:{type:"number",description:"The scale of the viewpoint."},targetGeometry:{description:"The target geometry framed by the viewpoint.",oneOf:[{description:"",$ref:"#/definitions/extent_schema.json"},{description:"",$ref:"#/definitions/geometry_schema.json"}]}},required:["camera"],additionalProperties:!1},"facilityLayerInfo_schema.json":{title:"facilityLayer",type:"object",description:"Defines the layer and field properties for the Facility layer used for floor filtering.",properties:{facilityIdField:{type:"string",description:"The name of the attribute field that contains a facility feature's unique identifier."},layerId:{type:"string",description:"A layer ID that refers to an operational layer in the map. The layer provides access to Facility features to use for floor filtering."},nameField:{type:"string",description:"The name of the attribute field that contains a facility feature's name."},siteIdField:{type:"string",description:"The name of the attribute field that contains a facility feature's site identifier (a foreign key to the Site layer)."}},required:["facilityIdField","layerId","nameField"],additionalProperties:!1},"levelLayerInfo_schema.json":{title:"levelLayer",type:"object",description:"Defines the layer and field properties for the Level layer used for floor filtering.",properties:{facilityIdField:{type:"string",description:"The name of the attribute field that contains a level feature's facility identifier (a foreign key to the Facility layer)."},layerId:{type:"string",description:"A layer ID that refers to an operational layer in the map. The layer provides access to Level features to use for floor filtering."},levelIdField:{type:"string",description:"The name of the attribute field that contains a level feature's unique identifier."},levelNumberField:{type:"string",description:"The name of the attribute field that contains a level feature's level number specific to its facility."},longNameField:{type:"string",description:"The name of the attribute field that contains a level feature's long name."},shortNameField:{type:"string",description:"The name of the attribute field that contains a level feature's short name."},verticalOrderField:{type:"string",description:"The name of the attribute field that contains a level feature's vertical order. The vertical order defines the order of display in the floor filter widget, and it also references the floor levels of an Indoor Positioning System."}},required:["facilityIdField","layerId","levelIdField","levelNumberField","longNameField","shortNameField","verticalOrderField"],additionalProperties:!1},"siteLayerInfo_schema.json":{title:"siteLayer",type:"object",description:"Defines the layer and field properties for the Site layer used for floor filtering.",properties:{layerId:{type:"string",description:"A layer ID that refers to an operational layer in the map. The layer provides access to Site features to use for floor filtering."},nameField:{type:"string",description:"The name of the attribute field that contains a site feature's name."},siteIdField:{type:"string",description:"The name of the attribute field that contains a site feature's unique identifier."}},required:["layerId","nameField","siteIdField"],additionalProperties:!1},"buildingSceneLayer_schema.json":{title:"Building Scene Layer (BuildingSceneLayer)",type:"object",description:"The BuildingSceneLayer is a layer type designed for on-demand streaming and displaying building data.",properties:{activeFilterId:{type:"string",description:"specifies the id of the currently active filter"},customParameters:{$ref:"#/definitions/customParameters_schema.json"},filters:{type:"array",description:"A list of filters available for this layer. Overrides filters defined on the service.",items:{$ref:"#/definitions/buildingSceneLayer_filter_schema.json"},uniqueItems:!0},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{type:"object",description:"Additional properties that can define an elevation offset for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{elevationInfo:{},maxScale:{},minScale:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["BuildingSceneLayer"]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",enum:["hide","hide-children","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},sublayers:{type:"array",description:"An array of objects specifying overrides for building scene layer sublayers",items:{$ref:"#/definitions/buildingSceneLayer_sublayer_schema.json"},uniqueItems:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the service."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title","url"],additionalProperties:!1},"catalogLayer_schema.json":{title:"Catalog Layer (CatalogLayer)",type:"object",description:"A catalog layer is a collection of item references to local and shared datasets, layers, services, and workspaces from across various work environments. The references, which are catalogued and added as items in a dataset, can be visualized, filtered, and queried dynamically in maps and scenes.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},dynamicGroupLayer:{description:"Object representing the dynamic group layer that loads layers selectively based on the current map extent, their scale visibility, time and range.",$ref:"#/definitions/catalogDynamicGroupLayer_schema.json"},footprintLayer:{description:"Object representing the dynamic group layer to visualize the catalog layer items as a polygon feature layer.",$ref:"#/definitions/catalogFootprintLayer_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{definitionExpression:{},maxScale:{},minScale:{},orderBy:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["CatalogLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","hide-children","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to disable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the service."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title"],additionalProperties:!1},"csvLayer_schema.json":{title:"CSV Layer (CSV)",type:"object",description:"The CSV layer type references a CSV or TXT file from a publicly-accessible web server. It then dynamically loads into the scene at run time. The CSV layer will maintain a reference to the CSV resource.",properties:{columnDelimiter:{type:"string",description:"A string defining the character used to separate columns in a CSV file.",enum:["\t"," ",",",";","|"]},customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{},transparency:{}},additionalProperties:!1}]},elevationInfo:{},featureReduction:{},fields:{},floorInfo:{},maxScale:{},minScale:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["CSV"]},listMode:{type:"string",description:"To show or hide the layer in the layer list",enum:["hide","show"],default:"show"},locationInfo:{description:"A locationInfo object defining how location information will be retrieved from a CSV file.",$ref:"#/definitions/locationInfo_schema.json"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},screenSizePerspective:{type:"boolean",description:"Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",default:!0},showLabels:{type:"boolean",description:"Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title","url"],additionalProperties:!1},"dimensionLayer_schema.json":{title:"Dimension Layer",type:"object",description:"Dimension layers can be used to visually annotate the lengths between points in a map. They can be created by embedding the dimensions directly in the layer.",properties:{dimensions:{type:"array",description:"A collection of dimension objects embedded in the layer.",items:{$ref:"#/definitions/dimensionLayer_lengthDimension_schema.json"}},id:{type:"string",description:"A unique identifying string for the layer."},layerType:{type:"string",description:"String indicating the layer type.",enum:["ArcGISDimensionLayer"]},listMode:{type:"string",description:"To show or hide the layer in the layer list.",enum:["hide","show"]},style:{description:"Specification of how dimensions and their labels are displayed.",$ref:"#/definitions/dimensionLayer_dimensionSimpleStyle_schema.json"},title:{type:"string",description:"A human readable string title for the layer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is visible in the scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title"],additionalProperties:!1},"featureLayer_schema.json":{title:"Feature Layer (ArcGISFeatureLayer)",type:"object",description:"Feature layers can be created by referencing a layer from either a map service or a feature service. Use a map service if you just want to retrieve geometries and attributes from the server and symbolize them yourself. Use a feature service if you want to take advantage of symbols from the service's source map document. Also, use a feature service if you plan on doing editing with the feature layer. Feature layers honor any feature templates configured in the source map document. Feature collection objects are used to create a feature layer based on the supplied definition.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{capabilities:{},definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{},transparency:{}},additionalProperties:!1}]},elevationInfo:{},featureReduction:{},floorInfo:{},maxScale:{},minScale:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["ArcGISFeatureLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to a geodatabase file. A URI format is used starting with `file:` followed by a file system path with a `.geodatabase` extension. A query parameter `itemId` must also be present specifying the ID of a table in the geodatabase's `GDB_ServiceItems` table. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.geodatabase?itemId=1`.",pattern:"^file:.+\\.geodatabase\\?itemId=\\d+$"},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},screenSizePerspective:{type:"boolean",description:"Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",default:!0},showLabels:{type:"boolean",description:"Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it. This property can get stored in the web scene config and in the item/data.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer. If the layer is not from a web service but rather a feature collection, then the url property is omitted."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title"],additionalProperties:!1},"geojsonLayer_schema.json":{title:"GeoJSON Layer (GeoJSON)",type:"object",description:"The GeoJSON layer type references a GeoJSON file from a publicly-accessible web server. It then dynamically loads into the scene at run time. The GeoJSON layer will maintain a reference to the GeoJSON resource.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item.",default:!1},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{}},additionalProperties:!1}]},elevationInfo:{},featureReduction:{},fields:{},objectIdField:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["GeoJSON"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled.",default:0},screenSizePerspective:{type:"boolean",description:"Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",default:!0},showLabels:{type:"boolean",description:"Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it. This property can get stored in the web scene config.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer.",format:"uri"},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["layerType","title","url"],additionalProperties:!1},"groupLayer_schema.json":{title:"Group Layer",type:"object",description:"GroupLayer provides the ability to organize several sublayers into one common layer. Suppose there are several FeatureLayers that all represent water features in different dimensions. For example, wells (points), streams (lines), and lakes (polygons). The GroupLayer provides the functionality to treat them as one layer called Water Features even though they are stored as separate feature layers.<br><br>With respect to scale visibility, sublayers of a group layer will be visible only within the scale range defined for the group layer. A sublayer may further restrict itself to a narrow scale range. In other words, a sublayer will be visible only when the current map scale intersects the scale range of that sublayer as well as the scale range of all its parent group layers.",properties:{blendMode:{$ref:"#/definitions/blendMode_schema.json"},id:{type:"string",description:"A unique identifying string for the layer"},layers:{type:"array",description:"list of child operationalLayers",items:{$ref:"#/definitions/operationalLayers_schema.json"},uniqueItems:!0},layerType:{type:"string",description:"String indicating the layer type",enum:["GroupLayer"]},listMode:{type:"string",description:"To show or hide the group layer in the layer list",enum:["hide","hide-children","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",default:1},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service"},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene",default:!0},visibilityMode:{type:"string",description:"How visibility of children is affected:independent, exclusive, inherited",enum:["exclusive","independent","inherited"]},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title"],additionalProperties:!1},"imageServiceLayer_schema.json":{title:"Image Service Layer (ArcGISImageServiceLayer)",type:"object",description:"An image service provides access to raster data through a web service. Multiple rasters can be served as one image service through mosaic dataset technology, dynamically processed and mosaicked on the fly. An image service supports accessing both the mosaicked image and its catalog, as well as individual rasters in the catalog. Also, image services can be cached (tiled) or uncached (dynamic). This object specifically details properties within uncached image services.",properties:{bandIds:{type:"array",description:"An array of bandIds that are visible for a multispectral dataset. It can specify bands to export or rearrange band order(from image service).",items:{type:"integer"}},compressionQuality:{type:"number",description:"Controls how much loss the image will be subjected to by the compression algorithm (from image service).",minimum:0,maximum:100},compressionTolerance:{type:"number",description:"Controls the tolerance of the lerc compression algorithm. The tolerance defines the maximum possible error of pixel values in the compressed image.",minimum:0,default:.01},customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"Boolean property indicating whether to ignore popups defined by the service item."},format:{type:"string",description:"String value representing image format.",enum:["bip","bmp","bsq","gif","jpg","jpgpng","lerc","png","png24","png32","png8","tiff"],default:"png"},id:{type:"string",description:"A unique identifying string for the layer."},interpolation:{type:"string",description:"The algorithm used for interpolation.",enum:["RSP_BilinearInterpolation","RSP_CubicConvolution","RSP_Majority","RSP_NearestNeighbor"]},isReference:{type:"boolean",description:"This is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers , or below (false)."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{type:"object",$ref:"#/definitions/rasterLayerDefinition_schema.json"},layerType:{type:"string",description:"String indicating the layer type.",enum:["ArcGISImageServiceLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},mosaicRule:{description:"An object representing a subset from multidimensional data. The data is based on slices or ranges in one or more dimensions in [mosaicRule.description](mosaicRule.md). When the [multidimensionalSubset](multidimensionalSubset.md) is defined then the [mosaicRule.multidimensionalDefinition](mosaicRule.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",$ref:"#/definitions/mosaicRule_schema.json"},multidimensionalSubset:{description:"An object representing a subset from multidimensional data. The data is based on slices or ranges in one or more dimensions in [mosaicRule.description](mosaicRule.md). When the [multidimensionalSubset](multidimensionalSubset.md) is defined then the [mosaicRule.multidimensionalDefinition](mosaicRule.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",$ref:"#/definitions/multidimensionalSubset_schema.json"},noData:{type:"integer",description:"The pixel value that represents no information."},noDataInterpretation:{type:"string",description:"A string value of interpretation of noData setting. Default is `esriNoDataMatchAny` when noData is a number, and `esriNoDataMatchAll` when noData is an array.",enum:["esriNoDataMatchAll","esriNoDataMatchAny"],default:"esriNoDataMatchAny"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},pixelType:{type:"string",description:"Pertains to the type of values stored in the raster, such as signed integer, unsigned integer, or floating point.",enum:["C128","C64","F32","F64","S16","S32","S8","U1","U16","U2","U32","U4","U8","UNKNOWN"]},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click on the image.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},renderingRule:{description:"Specifies the rendering rule for how the requested image should be rendered.",$ref:"#/definitions/renderingRule_schema.json"},showLegend:{type:"boolean",description:"Indicates whether to allow map authors the ability to control what layers should be shown in a client's legend control.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web map.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["layerType","title","url"],additionalProperties:!1},"integratedMesh3DTilesLayer_schema.json":{title:"Integrated Mesh 3D Tiles Layer",type:"object",description:"An integrated mesh layer with OGC 3D Tiles as the data source. This layer type can represent built and natural 3D features, such as building walls, trees, valleys, and cliffs, and can include realistic textures and elevation information.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{type:"object",description:"A layerDefinition object defining the attribute schema and drawing information for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{elevationInfo:{},maxScale:{},minScale:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["IntegratedMesh3DTilesLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to integrated mesh data in a 3D Tiles Archive Format file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.3tz`. A relative path must be from the file which defines the layer. For example `file:../p20/edinburgh.3tz`.",pattern:"^file:.+\\.3tz$"},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service."},url:{type:"string",description:"The URL to the tileset JSON file for the OGC 3D Tiles layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},anyOf:[{required:["id","layerType","title","url"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"integratedMeshLayer_schema.json":{title:"Integrated Mesh Layer (IntegratedMeshLayer)",type:"object",description:"An integrated mesh can represent built and natural 3D features, such as building walls, trees, valleys, and cliffs, with realistic textures and includes elevation information.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{type:"object",description:"A layerDefinition object defining the attribute schema and drawing information for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{elevationInfo:{},maxScale:{},minScale:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["IntegratedMeshLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},modifications:{type:"string",description:"URL to modifications json file, typically stored in `ITEM/resources`. Content of the file follows the $ref:[Modifications schema](modifications_schema.json)."},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to integrated mesh data in a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/edinburgh.slpk`.",pattern:"^file:.+\\.slpk$"},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service."},url:{type:"string",description:"The URL to the layer. If the layer is not from a web service but rather a feature collection, then the url property is omitted."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},anyOf:[{required:["id","layerType","title","url"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"kmlLayer_schema.json":{title:"KML Layer (KML)",type:"object",description:"Keyhole Markup Language (KML) is an XML-based format for storing geographic data and associated content and is an official Open Geospatial Consortium (OGC) standard. KML is a common format for sharing geographic data with non-GIS users as it can be easily delivered on the Internet.",properties:{id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"A string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerType:{type:"string",description:"String indicating the layer type.",enum:["KML"]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list.",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to a KML file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.kml` or `.kmz`.  A relative path must be from the file which defines the layer. For example `file:../commondata/kml/paris.kml`.",pattern:"^file:.+\\.km[lz]$"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},showLegend:{type:"boolean",description:"Indicates whether to allow scene authors the ability to control what layers should be shown in a client's legend.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibleFolders:{type:"array",description:"Array of numeric IDs of folders that will be made visible.",items:{type:"integer"},uniqueItems:!0}},anyOf:[{required:["id","layerType","title","url"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"lineOfSightLayer_schema.json":{title:"LineOfSight Layer",type:"object",description:"The LineOfSight layer is a layer for storing line of sight analyses in web scenes.",properties:{id:{type:"string",description:"A unique identifying string for the layer."},layerType:{type:"string",description:"String indicating the layer type.",enum:["LineOfSightLayer"]},listMode:{type:"string",description:"To show or hide the layer in the layer list.",enum:["hide","show"]},observer:{$ref:"#/definitions/lineOfSightObserver_schema.json"},targets:{type:"array",description:"A Collection of LineOfSight target objects used for visibility analysis from observer position.",items:{$ref:"#/definitions/lineOfSightTarget_schema.json"}},title:{type:"string",description:"A human readable string title for the layer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is visible in the scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","observer","targets","title"],additionalProperties:!1},"mapServiceLayer_schema.json":{title:"Map Service Layer (ArcGISMapServiceLayer)",type:"object",description:"Map services can be cached (tiled) or uncached (dynamic). This object specifically details properties within uncached map services.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},isReference:{type:"boolean",description:"This is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers, or below (false)."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layers:{type:"array",description:"An array of layer objects defining the styling, geometry, and attribute information for the features.",items:{allOf:[{$ref:"#/definitions/layer_schema.json"},{properties:{defaultVisibility:{},disablePopup:{},id:{},layerDefinition:{},listMode:{},maxScale:{},minScale:{},name:{},parentLayerId:{},popupInfo:{},showLegend:{},subLayerIds:{}},additionalProperties:!1}]},uniqueItems:!0},layerType:{type:"string",description:"String indicating the layer type.",enum:["ArcGISMapServiceLayer"]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",enum:["hide","hide-children","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"},visibleLayers:{type:"array",description:"An array of sublayer ids that should appear visible. Used with map service layers that are not tiled.",items:{type:"integer"},uniqueItems:!0}},required:["id","layerType","title","url"],additionalProperties:!1},"mediaLayer_schema.json":{title:"Media Layer (Media)",type:"object",description:"Media layer displays one media on the scene positioned by 4 control points.",properties:{georeference:{$ref:"#/definitions/mediaLayer_georeference_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the `Media Layer` item registered on ArcGIS Online or your organization's portal."},layerType:{type:"string",description:"String indicating the layer type.",enum:["MediaLayer"]},listMode:{type:"string",description:"To show or hide the layer in the layer list.",enum:["hide","show"]},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0,default:0},mediaType:{type:"string",description:"Indicates the type of media that the `url` points to. The currently supported media types are `image` and `video`. Images must be in JPEG or PNG format. For supported video formats, please refer to the [MDN video codec guide](https://developer.mozilla.org/en-US/docs/Web/Media/Formats/Video_codecs). `mediaType` is applicable only when the `url` is defined.",enum:["image","video"]},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0,default:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"URL to the media file. It is not applicable when `itemId` is defined. The url is relative when the media file is stored as an item resource. Relative urls are only applicable for `image` media types. `video` media types only support external video urls at this time. See related `mediaType` property."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},oneOf:[{required:["layerType","mediaType","title","url"]},{required:["itemId","layerType","title"]}],additionalProperties:!1},"ogcFeatureLayer_schema.json":{title:"OGC Feature Layer",type:"object",description:"OGC Feature Layer is a dynamic feature service that follows the specifications of OGC API - Features.",properties:{collectionId:{type:"string",description:"A unique identifying string for a feature collection."},customParameters:{type:"object",description:"A sequence of parameters used to append custom or vendor specific parameters to all OGC API - Features requests.",patternProperties:{".*":{type:"string"}}},disablePopup:{type:"boolean",description:"Indicates whether popup is enabled or not."},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Unique string value indicating an item registered in ArcGIS Online or your organization's portal."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{}},additionalProperties:!1}]},elevationInfo:{},featureReduction:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["OGCFeatureLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","hide-children","show"],default:"show"},maxScale:{type:"number",description:"Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification."},minScale:{type:"number",description:"Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification."},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},screenSizePerspective:{type:"boolean",description:"Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",default:!0},showLabels:{type:"boolean",description:"Boolean value indicating whether to display labels for this layer.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display legend for this layer.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL of the OGC API Features service landing page.",format:"uri"},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web map.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["layerType","title","url"],additionalProperties:!1},"orientedImageryLayer_schema.json":{title:"Oriented Imagery Layer (OrientedImageryLayer)",type:"object",description:"An Oriented Imagery layer is an extended point feature layer with additional properties that support the oriented imagery workflow. It is defined by camera locations (features) and has a geometry that allows it to be rendered in either a 2D MapView or 3D SceneView as a graphic with spatial context.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{capabilities:{},definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{},transparency:{}},additionalProperties:!1}]},elevationInfo:{},featureReduction:{},floorInfo:{},maxScale:{},minScale:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["OrientedImageryLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},orientedImageryProperties:{type:"object",description:"Object containing information about the chosen Oriented Imagery layer and schema.",$ref:"#/definitions/orientedImageryProperties_schema.json"},path:{type:"string",description:"For offline data, a path to a geodatabase file. A URI format is used starting with `file:` followed by a file system path with a `.geodatabase` extension. A query parameter `itemId` must also be present specifying the ID of a table in the geodatabase's `GDB_ServiceItems` table. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.geodatabase?itemId=1`.",pattern:"^file:.+\\.geodatabase\\?itemId=\\d+$"},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},screenSizePerspective:{type:"boolean",description:"Apply perspective scaling to screen-size symbols.",default:!0},showLabels:{type:"boolean",description:"Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it. This property can get stored in the web scene config and in the item/data.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0}},required:["id","layerType","title"],additionalProperties:!1},"pointCloudLayer_schema.json":{title:"PointCloud Layer",type:"object",description:"Point cloud data is post-processed spatially organized lidar data that consists of large collections of 3D points. Elevations for the ground, buildings, forest canopy, highway overpasses, and anything else encountered during the lidar survey make up the point cloud data. Point cloud layers allow for fast visualisation of point cloud data in the browser.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"disablePopups allows a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer"},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal"},layerDefinition:{type:"object",description:"A layerDefinition object defining the attribute schema and drawing information for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{renderer:{},transparency:{}},additionalProperties:!1}]},elevationInfo:{},filters:{},maxScale:{},minScale:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type",enum:["PointCloudLayer"]},listMode:{type:"string",description:"To show or hide the layer in the layer list",enum:["hide","show"],default:"show"},path:{type:"string",description:"For offline data, a path to point cloud layer data in a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/zurich.slpk`.",pattern:"^file:.+\\.slpk$"},popupInfo:{description:"A popupInfo object defining the content of pop-up windows when you click a point.",$ref:"#/definitions/popupInfo_schema.json"},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service"},url:{type:"string",description:"The URL to the layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},anyOf:[{required:["id","layerType","title","url"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"rasterDataLayer_schema.json":{title:"Raster Data Layer",type:"object",description:"A layer for displaying raster data. This layer only applies to offline data.",properties:{copyright:{type:"string",description:"Attribution to the Raster Data Layer provider. It is displayed in the attribution on the scene. Input required by the user when the layer is added to the scene."},id:{type:"string",description:"A unique identifying string for the layer."},layerType:{type:"string",description:"String indicating the layer type.",enum:["RasterDataLayer"]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list.",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to a raster data file. A URI format is used, starting with `file:` followed by a file system path. A relative path must be from the file which defines the layer. For example `file:../commondata/raster_data/beijing.tif`.",pattern:"^file:.+$"},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the scene.",default:!0}},required:["id","layerType","path","title"],additionalProperties:!1},"sceneLayer_schema.json":{title:"Scene Layer (ArcGISSceneServiceLayer)",type:"object",description:"The SceneLayer is a layer type designed for on-demand streaming and displaying large amounts of data in a SceneView. SceneLayers support two geometry types: Point and 3D Objects (e.g. buildings). The SceneLayer displays data published to a Scene Service. Scene Services can hold large volumes of features in an open format that is suitable for web streaming. The SceneLayer loads these features progressively, starting from coarse representations and refines them to higher detail as necessary for close-up views.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"disablePopups allows a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{type:"object",description:"A layerDefinition object defining the attribute schema and drawing information for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{},transparency:{}},additionalProperties:!1}]},elevationInfo:{},excludeObjectIds:{},featureReduction:{},floorInfo:{},maxScale:{},minScale:{},polygonFilter:{},rangeInfos:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["ArcGISSceneServiceLayer"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to a scene layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.slpk`. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.slpk`.",pattern:"^file:.+\\.slpk$"},popupInfo:{description:"A popupInfo object defining the content of pop-up windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},screenSizePerspective:{type:"boolean",description:"Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",default:!0},showLabels:{type:"boolean",description:"If the layer has a labelingInfo property then labels show on the scene only if the showLabels property it true.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the service."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},anyOf:[{required:["id","layerType","title","url"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"tiledImageServiceLayer_schema.json":{title:"Tiled Image Service Layer (ArcGISTiledImageServiceLayer)",type:"object",description:"An ArcGIS Tiled Image Service layer displays content from an ArcGIS Server Image service that has been cached (tiled).",properties:{bandIds:{type:"array",description:"An array of bandIds that are visible for a multispectral dataset. It can specify bands to export or rearrange band order(from image service).",items:{type:"integer"},minItems:1},blendMode:{$ref:"#/definitions/blendMode_schema.json"},customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},interpolation:{type:"string",description:"String indicating the interpolation type.",enum:["RSP_BilinearInterpolation","RSP_CubicConvolution","RSP_Majority","RSP_NearestNeighbor"]},isReference:{type:"boolean",description:"Applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers , or below (false)."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{allOf:[{$ref:"#/definitions/rasterLayerDefinition_schema.json"},{properties:{activePresetRendererName:{},drawingInfo:{},presetRenderers:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["ArcGISTiledImageServiceLayer"]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list.",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},multidimensionalDefinition:{type:"array",description:"An array of objects representing a slice from multidimensional data. The dimensional definitions in this array are used to filter display data based on slices in one or more dimensions.",items:{$ref:"#/definitions/multidimensionalDefinition_schema.json"}},multidimensionalSubset:{description:"An object representing a subset from multidimensional data. This includes subsets of both variables and dimensions. When the multidimensionalSubset is defined on a tiled image service layer, the layer's [multidimensionalDefinition](multidimensionalDefinition.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",$ref:"#/definitions/multidimensionalSubset_schema.json"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click on the image.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},renderingRule:{description:"Specifies the rendering rule for how the requested image should be rendered.",$ref:"#/definitions/renderingRule_schema.json"},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"URL to the ArcGIS Server Image Service."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title","url"],additionalProperties:!1},"tiledMapServiceLayer_schema.json":{title:"Tiled Map Service Layer (ArcGISTiledMapServiceLayer)",type:"object",description:"An ArcGIS Tiled Map Service layer displays map content from an ArcGIS Server Map service that has been cached (tiled).",properties:{blendMode:{$ref:"#/definitions/blendMode_schema.json"},customParameters:{$ref:"#/definitions/customParameters_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},isReference:{type:"boolean",description:"This property is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers , or below (false)."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layers:{type:"array",description:"An array of layer objects defining a URL for queries and the popup window content.",items:{allOf:[{$ref:"#/definitions/layer_schema.json"},{properties:{disablePopup:{},id:{},layerItemId:{},layerUrl:{},listMode:{},name:{},popupInfo:{},showLegend:{}},additionalProperties:!1}]},uniqueItems:!0},layerType:{type:"string",description:"String indicating the layer type.",enum:["ArcGISTiledMapServiceLayer"]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",enum:["hide","hide-children","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to a tile package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.tpk` or `.tpkx`. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.tpk`.",pattern:"^file:.+\\.tpkx?$"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service."},url:{type:"string",description:"URL to the ArcGIS Server tiled Map Service"},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the map.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},anyOf:[{required:["id","layerType","title","url"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"vectorTileLayer_schema.json":{title:"Vector Tile Layer (VectorTileLayer)",type:"object",description:"A vector tile layer references a set of web-accessible vector tiles and the corresponding style for how those tiles should be drawn.",properties:{blendMode:{$ref:"#/definitions/blendMode_schema.json"},customParameters:{$ref:"#/definitions/customParameters_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},isReference:{type:"boolean",description:"This property is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers, or below (false)."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerType:{type:"string",description:"String indicating the layer type.",enum:["VectorTileLayer"]},listMode:{type:"string",description:"To show or hide the layer in the layer list",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator."},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator."},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},path:{type:"string",description:"For offline data, a path to a vector tile layer package file. A URI format is used, starting with `file:` followed by a file system path with an extension of `.vtpk`. A relative path must be from the file which defines the layer. For example `file:../p20/northamerica.vtpk`.",pattern:"^file:.+\\.vtpk$"},styleUrl:{type:"string",description:"A url to a JSON file containing the stylesheet information used to render the layer. You may also pass an object containing the stylesheet information identical to the JSON file."},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},anyOf:[{required:["id","layerType","styleUrl","title"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"viewshedLayer_schema.json":{title:"Viewshed Layer",type:"object",description:"The viewshed layer is a layer for storing viewshed analyses in web scenes.",properties:{id:{type:"string",description:"A unique identifying string for the layer."},layerType:{type:"string",description:"String indicating the layer type.",enum:["ViewshedLayer"]},listMode:{type:"string",description:"To show or hide the layer in the layer list.",enum:["hide","show"]},title:{type:"string",description:"A human readable string title for the layer that can be used in a table of contents."},viewsheds:{type:"array",description:"A collection of viewshed objects embedded in the layer.",items:{$ref:"#/definitions/viewshedLayer_viewshed_schema.json"}},visibility:{type:"boolean",description:"Boolean property determining whether the layer is visible in the scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title"],additionalProperties:!1},"voxelLayer_schema.json":{title:"Voxel Layer",type:"object",description:"A voxel layer represents multidimensional spatial and temporal information in a 3D volumetric visualization. You can analyze the volume by slicing, creating sections or isosurfaces. For example, you can visualize atmospheric or oceanic data, a geological underground model, or space-time cubes as voxel layers.",properties:{customParameters:{$ref:"#/definitions/customParameters_schema.json"},disablePopup:{type:"boolean",description:"disablePopups allows a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerDefinition:{type:"object",description:"Additional properties that define drawing of a voxel layer.",$ref:"#/definitions/voxelLayerDefinition_schema.json"},layerType:{type:"string",description:"String indicating the layer type.",enum:["Voxel"]},listMode:{type:"string",description:"To show or hide the layer in the layer list.",enum:["hide","show"],default:"show"},popupInfo:{description:"A popupInfo object defining the content of pop-up windows when you click a point.",$ref:"#/definitions/popupInfo_schema.json"},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service."},url:{type:"string",description:"The URL to the layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title","url"]},"webTiledLayer_schema.json":{title:"Web Tiled Layer (WebTiledLayer)",type:"object",description:"A tile layer is a derived from a set of web-accessible tiles which reside on a server. The tiles are accessed by a direct URL request from the web browser. Because the tiles in a tile layer are not available as a service, they must be in a specific format for a web app such as the ArcGIS.com scene viewer to display the layer on a scene.",properties:{blendMode:{$ref:"#/definitions/blendMode_schema.json"},copyright:{type:"string",description:"Attribution to the Web Tiled Layer provider. It is displayed in the attribution on the web scene. Input required by the user when the layer is added to the web scene."},fullExtent:{description:"An extent object representing the full extent envelope for the layer.",$ref:"#/definitions/extent_schema.json"},id:{type:"string",description:"A unique identifying string for the layer."},isReference:{type:"boolean",description:"This is applicable if used as a basesceneLayer. A boolean value indicating whether or not the basesceneLayer draws on top (true) of other layers, including operationalLayers , or below (false)."},itemId:{type:"string",description:"Unique string value indicating an item registered in ArcGIS Online or your organization's portal."},layerType:{type:"string",description:"String indicating the layer type.",enum:["WebTiledLayer"]},listMode:{type:"string",description:"To show or hide the layer in the layer list",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},subDomains:{type:"array",description:"If subdomains are detected, they must be specified. The scene viewer detects if the Web Tiled Layer has subdomains by parsing the templateURL value for {subDomain}.",items:{type:"string"}},templateUrl:{type:"string",description:"URL to the Web Tiled Layer. Input required by the user when the layer is added to the web scene. The template URL contains a parameterized URL. The URL can contain the following templated parameters: `level`, `col`, `row`, and `subDomain`."},tileInfo:{description:"Contains the spatial reference and the tiling scheme of the layer. Typically retrieved from a WMTS OGC Web Service. If missing the layer must be in the WGS 1984 Web Mercator (Auxiliary Sphere) tiling scheme.",$ref:"#/definitions/tileInfo_schema.json"},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"},wmtsInfo:{description:"Object containing information about the chosen WMTS service layer and tiling schema.",$ref:"#/definitions/wmtsInfo_schema.json"}},required:["id","layerType","title"],additionalProperties:!1},"wfsLayer_schema.json":{title:"Web Feature Service Layer (WFS)",type:"object",description:"OGC Web Feature Service (WFS) is a dynamic feature service that follows the specifications of OGC.",properties:{disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},itemId:{type:"string",description:"Unique string value indicating an item registered in ArcGIS Online or your organization's portal."},layerDefinition:{allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{},transparency:{}},additionalProperties:!1}]},elevationInfo:{},featureReduction:{},fields:{},geometryType:{},spatialReference:{}},additionalProperties:!1}]},layerType:{type:"string",description:"String indicating the layer type.",enum:["WFS"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification."},minScale:{type:"number",description:"Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification."},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},screenSizePerspective:{type:"boolean",description:"Apply [perspective scaling](https://developers.arcgis.com/javascript/latest/api-reference/esri-layers-FeatureLayer.html#screenSizePerspectiveEnabled) to screen-size symbols.",default:!0},showLabels:{type:"boolean",description:"Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it. This property can get stored in the web map config and in the item/data.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display legend for this layer.",default:!0},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the layer. If the layer is not from a web service but rather a feature collection, than the url property is omitted."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web map.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"},wfsInfo:{description:"Object that defines and provides information about layers in a WFS service.",$ref:"#/definitions/wfsInfo_schema.json"}},required:["layerType","title","url"],additionalProperties:!1},"wmsLayer_schema.json":{title:"WMS Layer (WMS)",type:"object",description:"A layer consuming a Web Map Service (WMS). The WMS specification is an international specification for serving and consuming dynamic maps on the web.",properties:{copyright:{type:"string",description:"A string containing copyright and access information for a WMS layer. This is copied from the capabilities document exposed by the WMS service."},customLayerParameters:{type:"object",description:"A sequence of custom parameters to WMS layer requests. These parameters are applied to `GetMap` and `GetFeatureInfo` requests. The `customLayerParameters` property takes precedence if `customParameters` is also present.",patternProperties:{".*":{type:"string"}}},customParameters:{description:"A sequence of custom parameters to all WMS requests. These parameters are applied to `GetCapabilities`, `GetMap`, and `GetFeatureinfo` requests. If used with the `customLayerParameters` property, `customParameters` will not take precedence.",$ref:"#/definitions/customParameters_schema.json"},extent:{type:"array",description:"The minimum bounding rectangle, in decimal degrees, of the area covered by the layer as specified in the capabilities.",items:{type:"array",items:{type:"number"},minItems:2,maxItems:2},minItems:2,maxItems:2},featureInfoFormat:{type:"string",description:"Format of the feature, e.g.`text/plain`"},featureInfoUrl:{type:"string",description:"The URL for the WMS GetFeatureInfo call."},format:{type:"string",description:"A string containing the image format to be requested from a WMS service. Default is `png`.",enum:["bmp","gif","jpg","png","svg"],default:"png"},id:{type:"string",description:"A unique identifying string for the layer."},isReference:{type:"boolean",description:"This is applicable if used as a baseMapLayer. A boolean value indicating whether or not the baseMapLayer draws on top (true) of other layers, including operationalLayers, or below (false)."},itemId:{type:"string",description:"Unique string value indicating an item registered in ArcGIS Online or your organization's portal."},layers:{type:"array",description:"An array of layer objects from the WMS service.",items:{$ref:"#/definitions/wmsLayer_layer_schema.json"},uniqueItems:!0},layerType:{type:"string",description:"String indicating the layer type.",enum:["WMS"]},listMode:{type:"string",description:"To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",enum:["hide","hide-children","show"],default:"show"},mapUrl:{type:"string",description:"A string containing the URL of the WMS map. When using a WMS layer, you should also supply the url property. `mapUrl` is the URL returned by the capabilities to be used for the getMap requests."},maxHeight:{type:"number",description:"A number defining the maximum height, in pixels, that should be requested from the service."},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},maxWidth:{type:"number",description:"A number defining the maximum width, in pixels, that should be requested from the service."},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled."},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend.",default:!0},spatialReferences:{type:"array",description:"An array of numbers containing well-known IDs for spatial references supported by the service.",items:{type:"integer",description:"well-known ID"}},timeAnimation:{type:"boolean",description:"Indicates whether to enable time animation if the layer supports it.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},url:{type:"string",description:"The URL to the WMS service (`getCapabilities` URL)."},version:{type:"string",description:"A string containing the version number of the service."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web map.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"},visibleLayers:{type:"array",description:"An array of layers that should appear visible. The array contains the names of the visible layers.",items:{type:"string"},uniqueItems:!0}},required:["layerType","title"],additionalProperties:!1},"slide_schema.json":{title:"slide",type:"object",description:"A slide object used within a presentation.",properties:{baseMap:{description:"The basemap to be displayed on the slide.",allOf:[{$ref:"#/definitions/baseMap_schema.json"},{properties:{baseMapLayers:{},id:{},title:{}},additionalProperties:!1}]},description:{description:"Text description of the individual presentation slide.",$ref:"#/definitions/description_schema.json"},environment:{description:"Represents settings that affect the environment in which the web scene is displayed.",allOf:[{$ref:"#/definitions/environment_schema.json"},{properties:{lighting:{},weather:{}},additionalProperties:!1}]},ground:{description:"The ground properties to be set in the slide.",$ref:"#/definitions/slide_ground_schema.json"},hidden:{type:"boolean",description:"A boolean used to indicate the visibility of the slide inside a presentation.",default:!1},id:{type:"string",description:"The unique id of a slide within the slides property of a Presentation."},layout:{type:"string",description:"Indicates the placement of visual elements in the slide.",enum:["caption","cover","none"],default:"caption"},thumbnail:{$ref:"#/definitions/thumbnail_schema.json"},timeExtent:{description:"Represents the time extent for the slide. If the slide has a time extent set, it will update the time extent of the view when the slide is selected. Otherwise, if the time extent is not set on the slide, the time extent of the view will be removed when the slide is selected, disabling time filtering.",$ref:"#/definitions/timeExtent_schema.json"},title:{$ref:"#/definitions/title_schema.json"},viewpoint:{$ref:"#/definitions/viewpoint_schema.json"},visibleLayers:{type:"array",description:"An array of objects used to indicate the visible layers of the web scene.",items:{$ref:"#/definitions/visibleLayer_schema.json"}}},required:["id","thumbnail","title","viewpoint","visibleLayers"],additionalProperties:!1},"floorFilter_schema.json":{title:"floorFilter",type:"object",description:"Configurable properties used by the floor filter widget.",properties:{enabled:{type:"boolean",description:"Indicates whether the floor filter is actively filtering the displayed content according to the floor filter selection.",default:!1},facility:{type:"string",description:"Contains a facility identifier for the initially selected facility in the floor filter."},level:{type:"string",description:"Contains a level identifier for the initially selected floor, which is used when filtering floor-aware layers."},longNames:{type:"boolean",description:"Indicates whether the floor filter is showing long names instead of short names for floor levels.",default:!1},minimized:{type:"boolean",description:"Indicates whether the floor filter has been minimized to show only the levels list instead of showing the full set of breadcrumbs.",default:!1},pinnedLevels:{type:"boolean",description:"Indicates whether the floor filter has been pinned to show the floor levels list, instead of including the levels as a breadcrumb dropdown.",default:!1},site:{type:"string",description:"Contains a site identifier for the initially selected site in the floor filter."}},additionalProperties:!1},"range_schema.json":{title:"Range",description:"Range object.",properties:{interactionMode:{type:"string",description:"Indicates the mode in which the active range should be presented to the user.",enum:["picker","slider"]},numberOfStops:{type:"integer",description:"This is used to generate the activeRangeValues if activeRangeValues are not specified."},stopInterval:{type:"number",description:"Interval in which stops should be generated."}},oneOf:[{required:["interactionMode","numberOfStops"]},{required:["interactionMode","stopInterval"]}],additionalProperties:!1},"timeSlider_schema.json":{title:"timeSlider",type:"object",description:" Time animation is controlled by a configurable time slider. Those configurations are saved to the web scene as a timeSlider widget.",properties:{properties:{type:"object",$ref:"#/definitions/timeSliderProperties_schema.json"}},additionalProperties:!1},"buildingSceneLayer_filterAuthoringInfoCheckbox_schema.json":{title:"Metadata for checkbox based filter UI",type:"object",description:"Client UI with checkbox representation for each filter type and filter value.",properties:{filterBlocks:{type:"array",description:"Array of filter block authoring infos.",items:{$ref:"#/definitions/buildingSceneLayer_filterAuthoringInfo_filterBlock_schema.json"}},type:{type:"string",description:"Type of filter authoring info.",enum:["checkbox"]}},required:["filterBlocks","type"],additionalProperties:!1},"buildingSceneLayer_filterBlock_schema.json":{title:"Building Scene Layer Filter Block",type:"object",description:"A filter block defines what elements will be filtered with a specific filter mode.  To ensure performance on client applications, it is not recommended to declare multiple filter blocks with the same filter mode. Filter blocks are contained in a filter for a building scene layer. Each filter includes at least one filter block.",properties:{filterExpression:{type:"string",description:"SQL expression to select features that belong to this filter block."},filterMode:{$ref:"#/definitions/buildingSceneLayer_filterMode_schema.json"},title:{type:"string",description:"Title of the filter block."}},required:["filterExpression","filterMode","title"],additionalProperties:!1},"elevationInfo_schema.json":{title:"elevationInfo",type:"object",description:"Elevation info defines how features are aligned to ground or other layers.",properties:{featureExpression:{type:"object",description:'<em>Deprecated with 1.8, use `featureExpressionInfo` instead</em>. `{"value":0}` ignores geometry z-values.',properties:{value:{type:"number",enum:[0]}},minProperties:1,additionalProperties:!1},featureExpressionInfo:{description:"An object that defines an expression for per-feature elevation. If not set, geometry.z values are used for elevation. `unit` is applied to the resulting expression value.",$ref:"#/definitions/featureExpressionInfo_schema.json"},mode:{type:"string",description:"Determines how the service elevation values are combined with the elevation of the scene.",enum:["absoluteHeight","onTheGround","relativeToGround","relativeToScene"]},offset:{type:"number",description:"Offset is always added to the result of the above logic except for onTheGround where offset is ignored."},unit:{type:"string",description:'A string value indicating the unit for the values in elevationInfo. Applies to both `offset` and `featureExpressionInfo`. Defaults to `meter` if not set. <a href="#unit"><sup>1</sup></a>',default:"meter"}},required:["mode"],additionalProperties:!1},"featureReduction_schema.json":{title:"Feature Reduction",description:"An object that specifies how features are reduced or aggregated, with the goal of decluttering the view or presenting the user with an aggregate visualization. If unset, every feature is rendered individually.",oneOf:[{$ref:"#/definitions/featureReduction_select_schema.json"},{type:"null"}]},"field_schema.json":{title:"field",type:"object",description:"Contains information about an attribute field.",properties:{alias:{type:"string",description:"A string defining the field alias."},defaultValue:{type:["null","number","string"],description:"The default value set for the field."},domain:{description:"The domain objects if applicable.",$ref:"#/definitions/domain_schema.json"},editable:{type:"boolean",description:"A Boolean defining whether this field is editable."},length:{type:"integer",description:"A number defining how many characters are allowed in a string field."},name:{type:"string",description:"A string defining the field name."},nullable:{type:"boolean",description:"A Boolean defining whether this field can have a null value."},type:{type:"string",description:"A string defining the field type.",enum:["esriFieldTypeBigInteger","esriFieldTypeBlob","esriFieldTypeDate","esriFieldTypeDateOnly","esriFieldTypeDouble","esriFieldTypeGeometry","esriFieldTypeGlobalID","esriFieldTypeGUID","esriFieldTypeInteger","esriFieldTypeLong","esriFieldTypeOID","esriFieldTypeRaster","esriFieldTypeSingle","esriFieldTypeSmallInteger","esriFieldTypeString","esriFieldTypeTimeOnly","esriFieldTypeTimestampOffset","esriFieldTypeXML"]}},additionalProperties:!1},"pointCloudFilter_schema.json":{title:"PointCloud Filter",type:"object",description:"Filters applied to the pointCloud which will hide a point.",oneOf:[{$ref:"#/definitions/pointCloudBitfieldFilter_schema.json"},{$ref:"#/definitions/pointCloudReturnFilter_schema.json"},{$ref:"#/definitions/pointCloudValueFilter_schema.json"}]},"layerFloorInfo_schema.json":{title:"floorInfo",type:"object",description:"Contains floor-awareness information for a layer.",properties:{floorField:{type:"string",description:"The name of the attribute field that contains a floor's level ID used for floor filtering."}},required:["floorField"],additionalProperties:!1},"orderByField_schema.json":{title:"orderByField",type:"object",description:"Object defining the display order of features or records based on a field value, and whether they should be sorted in ascending or descending order.",properties:{field:{type:"string",description:"Name of a field. The value of this field will drive the sorting."},order:{type:"string",description:"Indicates whether features or records are sorted in ascending or descending order of the field values.",enum:["asc","desc"]}}},"polygonFilter_schema.json":{title:"Polygon filter",type:"object",description:"Filter features using polygons stored in external resources.",properties:{geometries:{type:"string",description:"URL to a polygon filter geometries json file, typically stored in `ITEM/resources`. Content of the file follows the $ref:[polygon filter geometries schema](polygonFilterGeometries_schema.json)."},spatialRelationship:{description:"Specifies the spatial relatonship used for the filter. `disjoint`: Display features that do not intersect any filter polygon. `contains`: Display features completely inside any filter polygon",enum:["contains","disjoint"]}},required:["geometries","spatialRelationship"]},"rangeInfo_schema.json":{title:"Range Information",description:"Range Information.",properties:{currentRangeExtent:{type:"array",description:"Contains the min and max values within which the features are visible.",items:{type:"number"},minItems:2,maxItems:2},field:{type:"string",description:"Field name to used for the range."},fullRangeExtent:{type:"array",description:"Contains the min and max values of all the features for this rangeInfo.",items:{type:"number"},minItems:2,maxItems:2},name:{type:"string",description:"A unique name that can be referenced by webMap.activeRanges."},type:{type:"string",description:"Type of range object.",enum:["rangeInfo"]}},required:["field","name","type"],additionalProperties:!1},"source_schema.json":{title:"source",type:"object",description:"Contains [dynamicDataLayer](dynamicDataLayer_source.md) object and [dynamicMapLayer](dynamicDataLayer_source.md) object. If the service supports dynamic layers, the [layerDefinition's ](layerDefinition.md) source property must be specified.",oneOf:[{$ref:"#/definitions/dynamicDataLayer_source_schema.json"},{$ref:"#/definitions/dynamicMapLayer_source_schema.json"}]},"type_schema.json":{title:"type",type:"object",description:"Contains information about an attribute field.",properties:{domains:{type:"object",description:"A comma-delimited series of domain objects for each domain in the type.",patternProperties:{".*":{$ref:"#/definitions/domain_schema.json"}}},id:{type:["number","string"],description:"A unique string or numerical ID for the type."},name:{type:"string",description:"A user-friendly name for the type."},templates:{type:"array",description:"Defined as a property of the layer definition when there are no types defined; otherwise, templates are defined as properties of the types.",items:{$ref:"#/definitions/template_schema.json"},uniqueItems:!0}},required:["id"],additionalProperties:!1},"labelingInfo_schema.json":{title:"labelingInfo",type:"object",description:"The labelingInfo object specifies the label definition for a layer.",properties:{labelExpression:{type:"string",description:"Defines the expression for text of labels. <em>Deprecated on FeatureLayer, Use `labelExpressionInfo` instead.</em>"},labelExpressionInfo:{$ref:"#/definitions/labelExpressionInfo_schema.json"},labelPlacement:{type:"string",description:"This string property specifies the label placement with respect to that of its feature. A list of label placement values categorized by feature geometry types.",enum:["esriServerLinePlacementAboveAfter","esriServerLinePlacementAboveAlong","esriServerLinePlacementAboveBefore","esriServerLinePlacementAboveEnd","esriServerLinePlacementAboveStart","esriServerLinePlacementBelowAfter","esriServerLinePlacementBelowAlong","esriServerLinePlacementBelowBefore","esriServerLinePlacementBelowEnd","esriServerLinePlacementBelowStart","esriServerLinePlacementCenterAfter","esriServerLinePlacementCenterAlong","esriServerLinePlacementCenterBefore","esriServerLinePlacementCenterEnd","esriServerLinePlacementCenterStart","esriServerPointLabelPlacementAboveCenter","esriServerPointLabelPlacementAboveLeft","esriServerPointLabelPlacementAboveRight","esriServerPointLabelPlacementBelowCenter","esriServerPointLabelPlacementBelowLeft","esriServerPointLabelPlacementBelowRight","esriServerPointLabelPlacementCenterCenter","esriServerPointLabelPlacementCenterLeft","esriServerPointLabelPlacementCenterRight","esriServerPolygonPlacementAlwaysHorizontal"]},maxScale:{type:"number",description:"Represents the maximum scale at which the layer definition will be applied."},minScale:{type:"number",description:"Represents the minimum scale at which the layer definition will be applied."},name:{type:"string",description:"The name of the label class."},symbol:{description:"The text symbol used to label.",$ref:"#/definitions/labelSymbol3D_schema.json"},useCodedValues:{type:"boolean",description:"Boolean value indicating whether to display the coded values for the specified field name(s)."},where:{type:"string",description:"String template used to determine which features to label."}},additionalProperties:!1},"renderer_schema.json":{title:"renderer",type:"object",description:"The renderer object contains the drawing information for the operationalLayer. This is a list of links to all the renderer objects.",oneOf:[{$ref:"#/definitions/classBreaksRenderer_schema.json"},{$ref:"#/definitions/heatmapRenderer_schema.json"},{$ref:"#/definitions/pointCloudRenderers_schema.json"},{$ref:"#/definitions/rasterRenderers_schema.json"},{$ref:"#/definitions/simpleRenderer_schema.json"},{$ref:"#/definitions/uniqueValueFromStyleRenderer_schema.json"},{$ref:"#/definitions/uniqueValueRenderer_schema.json"}]},"popupExpressionInfo_schema.json":{title:"popupExpressionInfo",description:"Arcade expression added to the pop-up.",properties:{expression:{type:"string",description:"The Arcade expression."},name:{type:"string",description:"Unique identifier for the expression."},returnType:{type:"string",description:"Return type of the Arcade expression, can be number or string. Defaults to string value. Number values are assumed to be `double`. This can be determined by the authoring client by executing the expression using a sample feature(s), although it can be corrected by the user. Knowing the returnType allows the authoring client to present fields in relevant contexts. For example, numeric fields in numeric contexts such as charts.",enum:["number","string"]},title:{type:"string",description:"Title of the expression."}},additionalProperties:!1},"fieldInfo_schema.json":{title:"fieldInfo",type:"object",description:"Defines how a field in the dataset participates (or does not participate) in a popup window.",properties:{fieldName:{type:"string",description:"A string containing the field name as defined by the service. Anywhere that a fieldname is referenced as `{field-name} in popupInfo, an Arcade expression can also be referenced as `{expression/<expression-name>}`."},format:{description:"A format object used with numerical or date fields to provide more detail about how the value should be displayed in a popup window.",$ref:"#/definitions/format_schema.json"},isEditable:{type:"boolean",description:"A Boolean determining whether users can edit this field. Not applicable to Arcade expressions.",default:!0},label:{type:"string",description:"A string containing the field alias. This can be overridden by the web map author. Not applicable to Arcade expressions as `title` is used instead."},statisticType:{type:"string",description:"Used in a 1:many or many:many relationship to compute the statistics on the field to show in the popup.",enum:["avg","count","max","min","stddev","sum","var"]},stringFieldOption:{type:"string",description:"A string determining what type of input box editors see when editing the field. Applies only to string fields. Not applicable to Arcade expressions.",enum:["richtext","textarea","textbox"],default:"textbox"},tooltip:{type:"string",description:"A string providing an editing hint for editors of the field. Not applicable to Arcade expressions."},visible:{type:"boolean",description:"A Boolean determining whether the field is visible in the popup window."}},additionalProperties:!1},"popupLayerOptions_schema.json":{title:"Layer Options",type:"object",description:"Additional options available for the popup layer.",properties:{returnTopmostRaster:{type:"boolean",description:"Indicates whether or not only the topmost raster should be displayed."},showNoDataRecords:{type:"boolean",description:"Indicates whether or not the NoData records should be displayed."}}},"mediaInfo_schema.json":{title:"mediaInfo",type:"object",description:"Defines an image or a chart to be displayed in a popup window.",oneOf:[{$ref:"#/definitions/mediaInfo_chart_schema.json"},{$ref:"#/definitions/mediaInfo_image_schema.json"}]},"popupElement_schema.json":{title:"popupElement",type:"object",description:"Popup elements allow users to author popups, using multiple elements such as tabular views, string description, media (charts and images), and attachments of the attributes and control the order in which they appear. Specifically, popupElements do the following: 1) provide the ability to explicitly add a field/ value table in addition to a description, 2) allow adding multiple description elements, and 3) allow a user to author and consume elements of a popup in the order of their choosing.",oneOf:[{$ref:"#/definitions/popupElement_attachments_schema.json"},{$ref:"#/definitions/popupElement_expression_schema.json"},{$ref:"#/definitions/popupElement_fields_schema.json"},{$ref:"#/definitions/popupElement_media_schema.json"},{$ref:"#/definitions/popupElement_relationship_schema.json"},{$ref:"#/definitions/popupElement_text_schema.json"}]},"relatedRecordsInfo_schema.json":{title:"relatedRecordsInfo",type:"object",description:"The sort in the popupInfo for the parent feature. This impacts the sorting order for the returned child records.",properties:{orderByFields:{type:"array",description:"Array of orderByField objects indicating the field display order for the related records and whether they should be sorted in ascending `asc` or descending `desc` order.",items:{$ref:"#/definitions/orderByField_schema.json"}},showRelatedRecords:{type:"boolean",description:"Required boolean value indicating whether to display related records. If <code>true</code>, client should let the user navigate to the related records. Defaults to <code>true</code> if the layer participates in a relationship AND the related layer/table has already been added to the scene (either as an operationalLayer or as a table)."}}},"search_schema.json":{title:"search",type:"object",description:"An object specifying the search parameters set within the web scene.",properties:{enabled:{type:"boolean",description:"A boolean value indicating whether search functionality is enabled in the web scene.",default:!0},hintText:{type:"string",description:"A string value used to indicate the hint provided with the search dialog."},layers:{type:"array",description:"An array of objects that define search fields and search criteria for layers.",items:{$ref:"#/definitions/search_layer_schema.json"},uniqueItems:!0},tables:{type:"array",description:"An array of objects that define search fields and search criteria for tables in the web map.",items:{$ref:"#/definitions/search_table_schema.json"},uniqueItems:!0}},required:[],additionalProperties:!1},"openStreetMapLayer_schema.json":{title:"OpenStreetMap Layer (OpenStreetMap)",type:"object",description:"Allows use of OpenStreetMap data for use in basemaps only.",properties:{blendMode:{$ref:"#/definitions/blendMode_schema.json"},id:{type:"string",description:"An identifying string for the layer, unique per scene."},layerType:{type:"string",description:"String indicating the layer type.",enum:["OpenStreetMap"]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},maxScale:{type:"number",description:"A numeric property used to determine the maximum scale at which the layer is displayed.",minimum:0},minScale:{type:"number",description:"A numeric property used to determine the minimum scale at which the layer is displayed.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},title:{type:"string",description:"String title for the basemap layer."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},required:["id","layerType","title"],additionalProperties:!1},"rasterDataElevationLayer_schema.json":{title:"Raster Data Elevation Layer",type:"object",description:"RasterDataElevationLayer is a single-band raster layer used for rendering elevation.",properties:{id:{type:"string",description:"A unique identifying string for the layer"},layerType:{type:"string",description:"String indicating the layer type",enum:["RasterDataElevationLayer"]},listMode:{type:"string",description:"To show or hide the elevation layer in the layer list",enum:["hide","show"],default:"show"},path:{type:"string",description:"For offline data, a path to an ArcGIS Runtime supported raster data file. A URI format is used, starting with file: followed by a file system path. A relative path must be from the file which defines the layer. For example `file:../commondata/raster_data/beijing.tif`",pattern:"^file:.+$"},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0}},required:["id","layerType","path","title"],additionalProperties:!1},"tiledElevationLayer_schema.json":{title:"Tiled Elevation Layer",type:"object",description:"Tiled Elevation Layer is a tile layer used for rendering elevation.",properties:{id:{type:"string",description:"A unique identifying string for the layer"},itemId:{type:"string",description:"Optional string containing the item ID of the service if it's registered on ArcGIS Online or your organization's portal."},layerType:{type:"string",description:"String indicating the layer type",enum:["ArcGISTiledElevationServiceLayer"]},listMode:{type:"string",description:"To show or hide the elevation layer in the layer list",enum:["hide","show"],default:"show"},path:{type:"string",description:"For offline data, a path to a tile package containing elevation data. A URI format is used, starting with `file:` followed by a file system path with an extension of `.tpk` or `.tpkx`. A relative path must be from the file which defines the layer. For example `file:../p20/portland.tpk`.",pattern:"^file:.+\\.tpkx?$"},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents. If this is not included, a title is derived from the service."},url:{type:"string",description:"The URL to the layer.If the layer is not from a web service but rather a feature collection, then the url property is omitted"},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0},visibilityTimeExtent:{description:"Represents time extent that will control when a layer should be visible based on webscene's current time. Visibility time extent only affects the layer visibility and will not filter the data.",$ref:"#/definitions/timeExtent_schema.json"}},anyOf:[{required:["id","layerType","title","url"]},{required:["id","layerType","path","title"]}],additionalProperties:!1},"environment_background_color_schema.json":{title:"color background",type:"object",description:"Specifies a single color to fill the background of the scene with. The scene background is displayed behind any scene objects, stars and atmosphere.",properties:{color:{$ref:"#/definitions/color_schema.json"},transparency:{type:"integer",description:"The value has to lie between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100},type:{type:"string",description:"The type of background.",enum:["color"]}},default:{type:"color",color:[0,0,0]},required:["color","type"],additionalProperties:!1},"lighting_schema.json":{title:"lighting",type:"object",description:"Object containing information on how the scene is lit.",oneOf:[{$ref:"#/definitions/sunLighting_schema.json"},{$ref:"#/definitions/virtualLighting_schema.json"}]},"cloudyWeather_schema.json":{title:"cloudyWeather",type:"object",description:"Object containing information for changing the weather conditions in the scene to cloudy.",properties:{cloudCover:{type:"number",description:"Specifies the amount of cloud cover in the sky.",minimum:0,maximum:1,default:.5},type:{type:"string",description:"The type of weather.",enum:["cloudy"]}},required:["type"],additionalProperties:!1},"foggyWeather_schema.json":{title:"foggyWeather",type:"object",description:"Object containing information for changing the weather conditions in the scene to foggy.",properties:{fogStrength:{type:"number",description:"Specifies the amount of fog used in the scene.",minimum:0,maximum:1,default:.5},type:{type:"string",description:"The type of weather.",enum:["foggy"]}},required:["type"],additionalProperties:!1},"rainyWeather_schema.json":{title:"rainyWeather",type:"object",description:"Object containing information for changing the weather conditions in the scene to rainy.",properties:{cloudCover:{type:"number",description:"Specifies the amount of cloud cover in the sky.",minimum:0,maximum:1,default:.5},precipitation:{type:"number",description:"Specifies the amount of rainfall in the scene.",minimum:0,maximum:1,default:.5},type:{type:"string",description:"The type of weather.",enum:["rainy"]}},required:["type"],additionalProperties:!1},"snowyWeather_schema.json":{title:"snowyWeather",type:"object",description:"Object containing information for changing the weather conditions in the scene to snowy.",properties:{cloudCover:{type:"number",description:"Specifies the amount of cloud cover in the sky.",minimum:0,maximum:1,default:.5},precipitation:{type:"number",description:"Specifies the amount of snowfall in the scene.",minimum:0,maximum:1,default:.5},snowCover:{type:"string",description:"Display surfaces covered with snow.",enum:["disabled","enabled"],default:"disabled"},type:{type:"string",description:"The type of weather.",enum:["snowy"]}},required:["type"],additionalProperties:!1},"sunnyWeather_schema.json":{title:"sunnyWeather",type:"object",description:"Object containing information for changing the weather conditions in the scene to sunny.",properties:{cloudCover:{type:"number",description:"Specifies the amount of cloud cover in the sky.",minimum:0,maximum:1,default:.5},type:{type:"string",description:"The type of weather.",enum:["sunny"]}},required:["type"],additionalProperties:!1},"camera_schema.json":{title:"Camera",type:"object",description:"The camera defines the position, tilt, field of view, and heading of the point from which the SceneView's visible extent is observed.",properties:{fov:{type:"number",description:"The diagonal field of view (fov) angle for the camera. The range of angles must be between 1 and 170 degrees.",minimum:1,maximum:170,default:55},heading:{type:"number",description:"The heading of the camera in degrees. Heading is zero when north is the top of the screen. It increases as the view rotates clockwise."},position:{description:"The position of the camera defined by a map point.",$ref:"#/definitions/point_geometry_schema.json"},tilt:{type:"number",description:"The tilt of the camera in degrees with respect to the surface as projected down from the camera position. Tilt is zero when looking straight down at the surface and 90 degrees when the camera is looking parallel to the surface."}},required:["heading","position","tilt"],additionalProperties:!1},"geometry_schema.json":{title:"geometry",type:"object",description:"Please refer to the links below for objects which define the JSON formats of geometry and spatial reference objects.",oneOf:[{$ref:"#/definitions/multipoint_geometry_schema.json"},{$ref:"#/definitions/point_geometry_schema.json"},{$ref:"#/definitions/polygon_geometry_schema.json"},{$ref:"#/definitions/polyline_geometry_schema.json"}]},"catalogDynamicGroupLayer_schema.json":{title:"Catalog Dynamic Group Layer (CatalogDynamicGroupLayer)",type:"object",description:"The Catalog Dynamic Group Layer automatically changes to show catalog items that are currently visible in the view. Can exist only as a child of a catalog layer.",properties:{id:{type:"string",description:"A unique identifying string for the layer."},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","hide-children","show"],default:"show"},maximumVisibleSublayers:{type:"integer",description:" Gets or sets upper bound for number of layers in view for the dynamic group layer.",minimum:0,default:10},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator.",minimum:0},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the legends for sub layers that are dynamically loaded. Default value is `true`.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene",default:!0}},required:["id","title"],additionalProperties:!1},"catalogFootprintLayer_schema.json":{title:"Catalog Footprint Layer (CatalogFootprintLayer)",type:"object",description:"The Footprints feature layer shows the boundary or extent of each item participating in the catalog dataset. Can exist only as a child of a catalog layer.",properties:{disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore popups defined by the service item."},id:{type:"string",description:"A unique identifying string for the layer."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{}},additionalProperties:!1}]},elevationInfo:{},maxScale:{},minScale:{}},additionalProperties:!1}]},listMode:{type:"string",description:"To show or hide layers in the layer list",enum:["hide","show"],default:"show"},opacity:{type:"number",description:"The degree of transparency applied to the layer on the client side, where 0 is full transparency and 1 is no transparency.",minimum:0,maximum:1,default:1},popupInfo:{description:"A popupInfo object defining the content of popup windows when you click or query a feature.",$ref:"#/definitions/popupInfo_schema.json"},showLabels:{type:"boolean",description:"Labels will display if this property is set to `true` and the layer also has a [labelingInfo](labelingInfo.md) property associated with it.",default:!1},showLegend:{type:"boolean",description:"Boolean value indicating whether to display the layer in the legend. Default value is `true`.",default:!0},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."},visibility:{type:"boolean",description:"Boolean property determining whether the layer is initially visible in the web scene.",default:!0}},required:["id","title"],additionalProperties:!1},"locationInfo_schema.json":{title:"locationInfo",type:"object",description:"Defines how location information will be retrieved from a [CSV](csvLayer.md) file referenced through the web, ie. referenced by URL.",properties:{latitudeFieldName:{type:"string",description:"A string defining the field name that holds the latitude (Y) coordinate."},locationType:{type:"string",description:"String value indicating location type.",enum:["coordinates"]},longitudeFieldName:{type:"string",description:"A string defining the field name that holds the longitude (X) coordinate."}},required:["locationType"],additionalProperties:!1},"dimensionLayer_lengthDimension_schema.json":{title:"Length Dimension",type:"object",description:"Defines the shape of a dimension that measures the distance between two points.",properties:{endPoint:{description:"The position of the point that a dimension is measured to (last input point).",$ref:"#/definitions/point_geometry_schema.json"},measureType:{type:"string",description:"Defines whether the horizontal, vertical or direct distance between the start and end points is measured.",enum:["direct","horizontal","vertical"]},offset:{type:"number",description:"The distance of the dimension line from the nearest input point in meters."},orientation:{type:"number",description:"The direction that the offset of a dimension with a 'direct' measureType extends in."},startPoint:{description:"The position of the point that a dimension is measured from (first input point).",$ref:"#/definitions/point_geometry_schema.json"},type:{type:"string",description:"Specifies the type of the dimension.",enum:["length"]}},required:["endPoint","measureType","offset","orientation","startPoint","type"],additionalProperties:!1},"dimensionLayer_dimensionSimpleStyle_schema.json":{title:"Dimension Simple Style",type:"object",description:"Specification of how dimensions and their labels are displayed.",properties:{color:{description:"Color of dimension lines.",$ref:"#/definitions/color_schema.json"},fontSize:{type:"number",description:"Font size of dimension label text in points.",minimum:0},lineSize:{type:"number",description:"Width of dimension lines in points.",minimum:0},textBackgroundColor:{description:"Background color of dimension labels.",$ref:"#/definitions/color_schema.json"},textColor:{description:"Color of dimension label text.",$ref:"#/definitions/color_schema.json"},type:{type:"string",description:"Specifies the type of style used.",enum:["simple"]}},required:["color","fontSize","lineSize","textBackgroundColor","textColor","type"],additionalProperties:!1},"blendMode_schema.json":{title:"blendMode",type:"string",description:"Blend modes are used to create various effects by blending colors of top and background layers. `normal` blend mode is the default.",enum:["average","color","color-burn","color-dodge","darken","destination-atop","destination-in","destination-out","destination-over","difference","exclusion","hard-light","hue","invert","lighten","lighter","luminosity","minus","multiply","normal","overlay","plus","reflect","saturation","screen","soft-light","source-atop","source-in","source-out","vivid-light","xor"],default:"normal"},"rasterLayerDefinition_schema.json":{title:"Raster Layer Definition",type:"object",description:"An object that defines the drawing information for image service based layers.",properties:{activePresetRendererName:{description:"The name of the active preset renderer, if the image service layer's current renderer is from preset renderers.",oneOf:[{type:"string"},{type:"null"}]},definitionExpression:{type:["string","null"],description:"SQL-based definition expression string that narrows the data to be displayed in the layer."},drawingInfo:{type:"object",description:"Contains the drawing information.",$ref:"#/definitions/rasterDrawingInfo_schema.json"},presetRenderers:{type:"array",description:"An array of preset renderers for image service layers, each contains associated information of multidimensional variable or raster function template.",items:{type:"object",$ref:"#/definitions/rasterPresetRenderer_schema.json"}}},additionalProperties:!1},"mosaicRule_schema.json":{title:"mosaicRule",type:"object",description:"The image service uses a mosaic rule to mosaick multiple rasters on the fly. The mosaic rule parameter is used by many image service operations, for example, export image and identify operations.",properties:{ascending:{type:"boolean",description:"Indicate whether to use ascending or descending order.",default:!0},fids:{type:"array",description:"A list that defines a subset of rasters used in the mosaic, be aware that the rasters may not be visible at all scales.",items:{type:"integer"}},itemRenderingRule:{description:"The rendering rule applies on items before mosaicking.",$ref:"#/definitions/renderingRule_schema.json"},lockRasterIds:{type:"array",description:"Lock a few rasters in the image service. Used together with `esriMosaicLockRaster`.",items:{type:"integer"}},mosaicMethod:{type:"string",description:"A string value that determines how the selected rasters are ordered.",enum:["esriMosaicAttribute","esriMosaicCenter","esriMosaicLockRaster","esriMosaicNadir","esriMosaicNone","esriMosaicNorthwest","esriMosaicSeamline","esriMosaicViewpoint"]},mosaicOperation:{type:"string",description:"Use the mosaic operation to resolve overlap pixel values: from first or last raster, use the min, max or mean of the pixel values, or blend them.",enum:["MT_BLEND","MT_FIRST","MT_LAST","MT_MAX","MT_MEAN","MT_MIN","MT_SUM"],default:"MT_FIRST"},multidimensionalDefinition:{type:"array",description:"An array of objects representing a slice from multidimensional data or multiple slices that are dynamically mosaicked and processed by the server. The dimensional definitions in this array are used to filter display data based on slices in one or more dimensions.",items:{$ref:"#/definitions/multidimensionalDefinition_schema.json"}},sortField:{type:"string",description:"The field name used together with `esriMosaicAttribute` method."},sortValue:{type:["null","number","string"],description:"The base sort value used together with `esriMosaicAttribute` method and `sortField` parameter.",default:0},viewpoint:{description:"Use a view point along with `esriMosaicViewpoint`.",$ref:"#/definitions/point_geometry_schema.json"},where:{type:"string",description:"<em>Deprecated. Use `layerDefinition.definitionExpression` instead.</em>"}},required:["mosaicMethod"],additionalProperties:!1},"multidimensionalSubset_schema.json":{title:"Multidimensional Subset",type:"object",description:"Represents a multidimensional subset of raster data. This includes subsets of both variables and dimensions. When the multidimensionalSubset is defined on a layer, the [layer.multidimensionalDefinition](multidimensionalDefinition.md) or the [mosaicRule.multidimensionalDefinition](mosaicRule.md) must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",properties:{areaOfInterest:{description:"An optional area of interest for the entire multidimensional subset.",oneOf:[{$ref:"#/definitions/extent_schema.json"},{$ref:"#/definitions/polygon_geometry_schema.json"}]},subsetDefinitions:{type:"array",description:"An optional array of objects representing dimension range subsets for selected variables from multidimensional data.",items:{$ref:"#/definitions/multidimensionalDefinition_schema.json"}}},additionalProperties:!1},"renderingRule_schema.json":{title:"renderingRule",type:"object",description:"Specifies the rendering rule for how the requested image should be rendered.",properties:{outputPixelType:{type:"string",description:"the output pixel type defines the output image's pixel type.",enum:["C128","C64","F32","F64","S16","S32","S8","U1","U16","U2","U32","U4","U8","UNKNOWN"],default:"UNKNOWN"},rasterFunction:{type:"string",description:"The raster function name identifies the processing or rendering to be performed. For a list of possible types, please see the [Raster Functions](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm) help topic for additional information on this."},rasterFunctionArguments:{type:"object",description:"The arguments for the referenced `rasterFunction`. For a description of arguments per raster function type, please see the [Raster Functions](https://developers.arcgis.com/documentation/common-data-types/raster-function-objects.htm) help topic for additional information on this."},rasterFunctionDefinition:{description:"Specifies the raster function template for how the requested image should be processed.",$ref:"#/definitions/rasterFunctionTemplate_schema.json"},variableName:{type:"string",description:"Variable name for the raster function."}},additionalProperties:!1},"lineOfSightObserver_schema.json":{title:"LineOfSight Observer",type:"object",description:"Defines the observer of a line of sight analysis in a LineOfSight layer.",properties:{elevationInfo:{$ref:"#/definitions/elevationInfo_schema.json"},feature:{description:"References a feature which is excluded from the intersection testing. It is used to ensure that the analysis results remain independent of changes in the level of detail of the scene geometry.",$ref:"#/definitions/featureReference_schema.json"},position:{description:"Position of the line of sight observer.",$ref:"#/definitions/point_geometry_schema.json"}},required:["position"],additionalProperties:!1},"lineOfSightTarget_schema.json":{title:"LineOfSight Target",type:"object",description:"Defines the target of a line of sight analysis in a LineOfSight layer.",properties:{elevationInfo:{$ref:"#/definitions/elevationInfo_schema.json"},feature:{description:"References a feature which is excluded from the intersection testing. It is used to ensure that the analysis results remain independent of changes in the level of detail of the scene geometry.",$ref:"#/definitions/featureReference_schema.json"},position:{description:"Position of the line of sight target.",$ref:"#/definitions/point_geometry_schema.json"}},required:["position"],additionalProperties:!1},"layer_schema.json":{title:"layer",type:"object",description:"A layer object may allow overrides on popup content and drawing behavior for individual layers of a web service. This object also contains geographic features and their attributes when used in a feature collection.",properties:{defaultVisibility:{type:"boolean",description:"Default visibility of the layers in the map service."},disablePopup:{type:"boolean",description:"Indicates whether to allow a client to ignore the popups defined on the layer. The popupInfo object could be saved in the map or item.",default:!1},id:{type:"integer",description:"The layer id, as a numeric value."},layerDefinition:{description:"Additional properties that define drawing information and other configurations for the layer.",allOf:[{$ref:"#/definitions/layerDefinition_schema.json"},{properties:{definitionExpression:{},drawingInfo:{allOf:[{$ref:"#/definitions/drawingInfo_schema.json"},{properties:{labelingInfo:{},renderer:{},showLabels:{},transparency:{}},additionalProperties:!1}]},source:{}},additionalProperties:!1}]},layerItemId:{type:"string",description:"The associated query layer's itemId. Only available when there is a `layerUrl`.  You will see this if [popups are configured](https://doc.arcgis.com/en/arcgis-online/manage-data/publish-tiles-from-features.htm) on it."},layerUrl:{type:"string",description:"A URL to a service that should be used for all queries against the layer."},listMode:{type:"string",description:"To show or hide the sublayer in the layer list. If the layer has sublayers, selecting `hide-children` will hide them in the layer list.",enum:["hide","hide-children","show"]},maxScale:{type:"number",description:"A number representing the maximum scale at which the layer will be visible. The number is the scale's denominator."},minScale:{type:"number",description:"A number representing the minimum scale at which the layer will be visible. The number is the scale's denominator."},name:{type:"string",description:"The name of the layer."},parentLayerId:{type:"integer",description:"If working with nested layers, this is the numeric value indicating the layer id of the next layer (parent) directly above the current referenced layer."},popupInfo:{description:"A popupInfo object defining the popup window content for the layer.",$ref:"#/definitions/popupInfo_schema.json"},showLegend:{type:"boolean",description:"A Boolean indicating if the layer should be shown in the legend in client applications.",default:!0},subLayerIds:{type:"array",description:"If the layer is a parent layer, it will have one or more sub layers included in an array.",items:{type:"integer"}}},additionalProperties:!1},"mediaLayer_georeference_schema.json":{title:"Georeference",type:"object",description:"The georeference used to place a media in a [MediaLayer](mediaLayer.md).",properties:{coefficients:{type:"array",description:"An array of 8 coefficients representing the [projective transformation](https://desktop.arcgis.com/en/arcmap/latest/manage-data/editing-existing-features/about-spatial-adjustment-transformations.htm#ESRI_SECTION1_EBB9C52B96934FE08A32CE852788EA02).",items:{type:"number"},minItems:8,maxItems:8},controlPoints:{type:"array",description:"An array of 4 user defined control points placed on the media.",items:{$ref:"#/definitions/mediaLayer_georeference_controlPoint_schema.json"},minItems:4,maxItems:4},height:{type:"number",description:"Numeric value indicating the height of media in pixels."},spatialReference:{type:"object",description:"The spatial reference can be defined using a well-known ID (wkid) or well-known text (WKT).",$ref:"#/definitions/spatialReference_schema.json"},width:{type:"number",description:"Numeric value indicating the width of media in pixels."}},required:["coefficients","controlPoints","height","spatialReference","width"],additionalProperties:!1},"orientedImageryProperties_schema.json":{title:"Oriented Imagery Properties",type:"object",description:"Object that defines properties of layers in a feature service with layer type [OrientedImageryLayer](orientedImageryLayer.md). If a property is not defined in the layer field, corresponding value for that property defined here would be used instead.",properties:{cameraHeading:{type:"number",description:"Camera orientation defining the first rotation around z-axis of the camera. Defined in degrees. Heading values are measured in the positive clockwise direction where north is defined as 0 degrees. -999 is used when the orientation is unknown."},cameraHeight:{type:"number",description:"The height of camera above the ground when the imagery was captured. The units are in meters. Camera height is used to determine the visible extent of the image, large values will result in a greater view extent. Values should not be less than 0."},cameraPitch:{type:"number",description:"Camera orientation defining the second rotation around x-axis of the camera in the positive counterclockwise direction. Defined in degrees. The pitch is 0 degrees when the camera is facing straight down to ground. The valid range of pitch value is from 0 to 180 degrees, with 180 degrees for a camera facing straight up and 90 degrees for a camera facing horizon."},cameraRoll:{type:"number",description:"Camera orientation defining the final rotation around z-axis of the camera in the positive clockwise direction. The camera housing rotation is defined in degrees. Valid values range from -90 to +90."},coveragePercent:{type:"number",description:"Modifies the extent of the image's ground footprint. The ground footprint of each image is computed to search for images that contain the selected location, which is identified as the red cross on the map. Valid values are from -50 to 50. Negative percentage values shrink the size of the ground footprint and positive values increase the size of the ground footprint."},farDistance:{type:"number",description:"The farthest usable distance of the imagery from the camera position. FarDistance is used to determine the extent of the image footprint, which is used to determine if an image is returned when you click on the map, and for creating optional footprint features. The units are in meters. Far distance should be always greater than 0."},horizontalFieldOfView:{type:"number",description:"The camera's scope in horizontal direction. The units are in degrees and valid values range from 0 to 360."},imagePathPrefix:{type:"string",description:"Prefix used to build the image url path in conjunction with the image attribute."},imagePathSuffix:{type:"string",description:"Suffix used to build the image url path in conjunction with the image attribute."},imageRotation:{type:"number",description:"The orientation of the camera in degrees relative to the scene when the image was captured. The rotation is added in addition to the camera roll. The value can range from -360 to +360."},maximumDistance:{type:"number",description:"Maximum search distance to be used while querying the feature service specified in the Oriented Imagery Layer. The maximum distance can never be less than zero.",minimum:0},nearDistance:{type:"number",description:"The nearest usable distance of the imagery from the camera position. The units are in meters. Near distance can never be less than 0."},orientationAccuracy:{type:"string",description:"Semicolon-delimited string used to store standard deviation values. The standard deviation values are in the following order and format: Camera location in XY direction; camera height; camera heading; camera pitch; camera roll; near distance; far distance; elevation (all in meters)."},orientedImageryType:{type:"string",description:"String that defines the imagery type used in the particular Oriented Imagery Layer.",enum:["360","Horizontal","Inspection","Nadir","Oblique"]},timeIntervalUnit:{type:"string",description:"Defines the unit of time used in the viewer's time selector tool. Images will be filtered in the viewer based on the Time Unit value defined here.",enum:["Days","Hours","Minutes","Months","Weeks","Years"]},verticalFieldOfView:{type:"number",description:"The camera's scope in the vertical direction. The units are in degrees and valid values range from 0 to 180."}},additionalProperties:!1},"multidimensionalDefinition_schema.json":{title:"Dimensional Definition",type:"object",description:"The dimensional definition defines a display filter based on one variable and one dimension. It is typically used when filtering data based on slices or ranges in one or more dimensions with [mosaicRule.description](mosaicRule.md). If the [multidimensionalSubset](multidimensionalSubset.md) is defined on the [TiledImageServiceLayer](tiledImageServiceLayer_schema.md) or the [mosaicRule](mosaicRule.md) then the multidimensionalDefinition must be within the defined multidimensionalSubset, otherwise nothing will be displayed.",properties:{dimensionName:{type:"string",description:"Type of dimension being used (ex. StdTime)."},isSlice:{type:"boolean",description:"Is slice?",default:!0},values:{type:"array",description:"Array where every element is either a single value, or an array of two values (lower and upper bound).",items:{oneOf:[{type:"number"},{type:"array",items:{type:"number"},minItems:2,maxItems:2}]}},variableName:{type:"string",description:"Name of the variable."}},anyOf:[{required:["variableName"]},{required:["dimensionName","values"]}],additionalProperties:!1,dependencies:{dimensionName:["values"],values:["dimensionName"]}},"viewshedLayer_viewshed_schema.json":{title:"Viewshed",type:"object",description:"Defines the shape of a viewshed that applies a visibility analysis.",properties:{farDistance:{type:"number",description:"The maximum distance from the observer in which to perform the viewshed analysis (in meters).",minimum:0},feature:{description:"References a feature from which the observer is internally offset, provided that its sides are close enough to the observer. It is used to ensure that the analysis results remain independent of changes in the level of detail of the scene geometry.",$ref:"#/definitions/featureReference_schema.json"},heading:{type:"number",description:"The compass heading of the observer's view direction (in degrees). A heading of zero points the viewshed to north and it increases rotating in clock-wise order.",minimum:0,exclusiveMaximum:360},horizontalFieldOfView:{type:"number",description:"The horizontal field of view (FOV) angle defines the width of the scope being analyzed (in degrees). A value of 360 means the observer's horizontal FOV captures their entire surroundings. Values closer to 0 narrow the horizontal FOV in the direction of the heading.",minimum:0,maximum:360},observer:{description:"The position the viewshed is computed from.",$ref:"#/definitions/point_geometry_schema.json"},tilt:{type:"number",description:"The tilt of the observer's view direction (in degrees). A tilt of zero points the viewshed looking straight down and 90 degrees points it looking parallel to the surface.",minimum:0,maximum:180},verticalFieldOfView:{type:"number",description:"The vertical field of view (FOV) angle defines the height of the scope being analyzed (in degrees). This value can vary from 0 to 180. Values closer to 0 narrow the vertical FOV in the direction of the tilt.",minimum:0,maximum:180}},required:["farDistance","heading","horizontalFieldOfView","observer","tilt","verticalFieldOfView"],additionalProperties:!1},"voxelLayerDefinition_schema.json":{title:"voxelLayerDefinition",type:"object",description:"The voxelLayerDefinition contains drawing information for a voxel layer.",properties:{maxScale:{type:"number",description:"Represents the maximum scale (most zoomed in) at which the layer is visible in the view. If the web scene is zoomed in beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a maximum scale. If set, the maxScale value should always be smaller than the minScale value, and greater than or equal to the service specification."},minScale:{type:"number",description:"Represents the minimum scale (most zoomed out) at which the layer is visible in the view. If the web scene is zoomed out beyond this scale, the layer will not be visible. A value of 0 means the layer does not have a minimum scale. If set, the minScale value should always be larger than the maxScale value, and lesser than or equal to the service specification."},sections:{type:"array",description:"Array of metadata about sections. A section is a static plane through the voxel layer showing the variable the section was created at.",items:{$ref:"#/definitions/voxelSection_schema.json"},minItems:0},style:{description:"Voxel style describes how the layer will be drawn including rendering and voxel representions that will be visible.",$ref:"#/definitions/voxelStyle_schema.json"}},additionalProperties:!1},"tileInfo_schema.json":{title:"tileInfo",type:"object",description:"Tile information, returned from the WMTS OGC Web Service. The tileInfo will contain the spatial reference of the layer. tileInfo is the same json representation as the ArcGIS Map/Image service tileInfo except that it may contain a levelValue on the lod objects that should be used instead of the level in the templateUrl.",properties:{cols:{type:"integer",description:"Requested tile's column."},compressionQuality:{type:"number",description:"Compression quality of the tile.",minimum:0,maximum:100},dpi:{type:"number",description:"The dpi of the tiling scheme."},lods:{type:"array",description:"An array of levels of detail that define the tiling scheme.",items:{$ref:"#/definitions/lod_schema.json"},uniqueItems:!0},origin:{description:"The tiling scheme origin.",$ref:"#/definitions/point_geometry_schema.json"},rows:{type:"integer",description:"Requested tile's row."},spatialReference:{description:"The spatial reference of the tiling schema.",$ref:"#/definitions/spatialReference_schema.json"}},additionalProperties:!1},"wmtsInfo_schema.json":{title:"Web Map Tile Service Info",type:"object",description:"Object defines and provides information about layers in a [WMTSLayer](webTiledLayer.md) service.",properties:{customLayerParameters:{type:"object",description:"A sequence of parameters used to append different custom parameters to a WMTS tile request. These parameters are applied to `GetTile`. The `customLayerParameters` property takes precedence if `customParameters` is also present.",patternProperties:{".*":{type:"string"}}},customParameters:{description:"A sequence of parameters used to append custom parameters to all WMTS requests. These parameters are applied to `GetCapabilities` and `GetTile`. If used with the `customLayerParameters` property, `customParameters` will not take precedence.",$ref:"#/definitions/customParameters_schema.json"},layerIdentifier:{type:"string",description:"Identifier for the specific layer used in the WMTS service. Required input by the user."},tileMatrixSet:{type:"string",description:"Tiling schema, set by the WMTS service."},url:{type:"string",description:"URL to the WMTS web service. Required input by the user."}},additionalProperties:!1},"wfsInfo_schema.json":{title:"Web Feature Service Info",type:"object",description:"Object that defines and provides information about layers in a WFS service.",properties:{customParameters:{type:"object",description:"A sequence of parameters used to append custom parameters to all WFS requests. These parameters are applied to `GetCapabilities`, `DescribeFeatureType`, and `GetFeatures`.",patternProperties:{".*":{type:"string"}}},featureUrl:{type:"string",description:"URL of the WFS service operation."},maxFeatures:{type:"integer",description:"Set this to limit the number of requested features that a GetFeature request presents."},name:{type:"string",description:"The name of the WFS layer. This is used to set layer visibility."},supportedSpatialReferences:{type:"array",description:"List of supported spatial reference IDs",items:{type:"integer"}},swapXY:{type:"boolean",description:"Boolean value indicating whether X and Y axis are flipped.",default:!1},version:{type:"string",description:"Value indicating which version of the WFS specification is used."},wfsNamespace:{type:"string",description:"String indicating namespace."}},additionalProperties:!1},"wmsLayer_layer_schema.json":{title:"WMSLayer layer",type:"object",description:"A layer object may allow overrides on popup content and drawing behavior for individual layers of a web service.",properties:{legendUrl:{type:"string",description:"A string URL to a legend graphic for the layer."},name:{type:"string",description:"A string containing a unique name for the layer."},queryable:{type:"boolean",description:"Boolean specifying whether a layer is queryable or not."},showPopup:{type:"boolean",description:"Boolean specifying whether to display popup or not."},title:{type:"string",description:"A user-friendly string title for the layer that can be used in a table of contents."}},additionalProperties:!1},"description_schema.json":{title:"description",type:"object",description:"Description object with text.",properties:{text:{type:"string",description:"Text to display as slide description."}},required:["text"],additionalProperties:!1},"slide_ground_schema.json":{title:"Ground",type:"object",description:"Ground defines the main surface of the web scene, based on elevation layers. This object defines the ground properties to be set in the slides.",properties:{transparency:{type:"integer",description:"The transparency of the ground surface (including basemap). The value has to lie between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100}},additionalProperties:!1},"thumbnail_schema.json":{title:"thumbnail",type:"object",description:"Object containing a thumbnail image.",properties:{url:{type:"string",description:"The URI pointing to the thumbnail image. Can be a URL or a base64-encoded image."}},required:["url"],additionalProperties:!1},"title_schema.json":{title:"title",type:"object",description:"Text for the title of the individual slide.",properties:{text:{type:"string",description:"Text to display as slide title."}},required:["text"],additionalProperties:!1},"visibleLayer_schema.json":{title:"visibleLayer",type:"object",description:"Object with layer id, used to indicate layer visibility in a slide.",properties:{id:{type:"string",description:"The id of the layer as listed on the operational layer."},subLayerIds:{type:"array",description:"List of visible sublayer ids, as listed on the service of the layer.",items:{type:"integer"}}},additionalProperties:!1},"timeSliderProperties_schema.json":{title:"TimeSlider properties",type:"object",description:"Configurable properties used within the TimeSlider widget.",properties:{currentTimeExtent:{description:"An optional array indicating the start and the end of the slider's current time extent.",$ref:"#/definitions/timeExtent_schema.json"},endTime:{type:"number",description:"The optional end of the time slider. If not specified, the slider defaults to the full time extent of all time-enabled layers."},loop:{type:"boolean",description:"When `true`, the time slider will play its animation in a loop. The default is `false`.",default:!1},numberOfStops:{type:"integer",description:"Number of stops within the timeSlider widget."},startTime:{type:"number",description:"The optional start of the time slider. If not specified, the slider defaults to the full time extent of all time-enabled layers."},stops:{type:"array",description:"An optional array of numbers that defines stops for the time slider. Can be used to create irregularly spaced stops representing specific moments of importance.",items:{type:"integer"}},thumbCount:{type:"integer",description:"The default value for the thumbCount is 2.",default:2},thumbMovingRate:{type:"number",description:"Rate at which the time animation plays. Units are in milliseconds. Default is `2000`.",default:2e3},timeStopInterval:{type:"object",description:"The interval which has been defined for the time slider.",$ref:"#/definitions/timeStopInterval_schema.json"}},additionalProperties:!1},"buildingSceneLayer_filterAuthoringInfo_filterBlock_schema.json":{title:"Filter Authoring Info for Filter Blocks",type:"object",description:"The filter authoring info object contains metadata about the authoring process for creating a filter block object. This allows the authoring client to save specific, overridable settings.  The next time it is accessed via an authoring client, their selections are remembered. Non-authoring clients can ignore it.",properties:{filterTypes:{type:"array",description:"Array of defined filter types. Each filter type has an array of filter values.",items:{$ref:"#/definitions/buildingSceneLayer_filterAuthoringInfo_filterType_schema.json"}}},required:["filterTypes"],additionalProperties:!1},"buildingSceneLayer_filterMode_schema.json":{title:"Filter Mode",type:"object",description:"Filter mode represents the way elements draw when participating in a filter block.",oneOf:[{$ref:"#/definitions/buildingSceneLayer_filterModeSolid_schema.json"},{$ref:"#/definitions/buildingSceneLayer_filterModeWireFrame_schema.json"},{$ref:"#/definitions/buildingSceneLayer_filterModeXray_schema.json"}]},"featureExpressionInfo_schema.json":{title:"featureExpressionInfo",type:"object",description:"An object that defines an expression for per-feature elevation.",properties:{expression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number.",minLength:1},title:{type:"string",description:"Title of the expression.",minLength:1}},required:["expression"],additionalProperties:!1},"featureReduction_select_schema.json":{title:"Feature Reduction: Selection",type:"object",description:"Feature reduction of type `selection` declutters the screen by hiding features that would otherwise intersect with other features on screen. The default behavior of this feature reduction type is to select features by depth order, i.e. hide all features that would otherwise be overlapped by at least one other feature which is closer to the viewer.",properties:{type:{type:"string",description:"A string value indicating the feature reduction type.",enum:["selection"]}}},"domain_schema.json":{title:"domain",type:"object",description:"Domains specify the set of valid values for a field. The links below are the available domains.",oneOf:[{$ref:"#/definitions/codedValue_domain_schema.json"},{$ref:"#/definitions/inherited_domain_schema.json"},{$ref:"#/definitions/range_domain_schema.json"}]},"pointCloudBitfieldFilter_schema.json":{title:"PointCloud Bitfield Filter",type:"object",description:"Filters points based on the value of the specified bitfield attribute.",properties:{field:{type:"string",description:"The name of the field that is used for the filter."},requiredClearBits:{type:"array",description:"List ALL bit numbers that must cleared (=0) for the point to be kept. bit 0 is LSB.",items:{type:"integer"}},requiredSetBits:{type:"array",description:" List ALL bit numbers that must set (=1) for the point to be kept. bit 0 is LSB.",items:{type:"integer"}},type:{type:"string",enum:["pointCloudBitfieldFilter"]}},anyOf:[{required:["requiredClearBits"]},{required:["requiredSetBits"]}],required:["field","type"],additionalProperties:!1},"pointCloudReturnFilter_schema.json":{title:"PointCloud Return Filter",type:"object",description:"Filters points based on the value of the return number/return count.",properties:{field:{type:"string",description:"The name of the field that is used for the filter."},includedReturns:{type:"array",description:"All points with at least one specified return status will be kept. Status values: `last`, `firstOfMany`, `lastOfMany`, `single`",items:{type:"string",enum:["firstOfMany","last","lastOfMany","single"]}},type:{type:"string",enum:["pointCloudReturnFilter"]}},required:["field","includedReturns","type"],additionalProperties:!1},"pointCloudValueFilter_schema.json":{title:"PointCloud Value Filter",type:"object",description:"Filters points based on the value of an specified attribute.",properties:{field:{type:"string",description:"The name of the field that is used for the filter."},mode:{type:"string",description:"Defines if values should be included or excluded.",enum:["exclude","include"]},type:{type:"string",description:"Filters points based on the value of an specified attribute.",enum:["pointCloudValueFilter"]},values:{type:"array",description:"list of values",items:{type:"number"}}},required:["field","mode","type","values"],additionalProperties:!1},"dynamicDataLayer_source_schema.json":{title:"Dynamic data layer",type:"object",description:"A dynamic data layer derived from a registered workspace. More information on this can be found in the [ArcGIS REST API help](http://resources.arcgis.com/en/help/rest/apiref/layersource.html).",properties:{dataSource:{description:"The layer's data source.",$ref:"#/definitions/dataSource_schema.json"},fields:{type:"array",description:"An array of objects specifying information about an attribute field.",items:{$ref:"#/definitions/field_schema.json"},uniqueItems:!0},type:{type:"string",description:"A string value indicating the type.",enum:["dataLayer"]}},required:["dataSource","type"],additionalProperties:!1},"dynamicMapLayer_source_schema.json":{title:"Dynamic map layer",type:"object",description:"A dynamic map layer refers to a layer in the current map service. More information on this can be found in the [ArcGIS REST API help](http://resources.arcgis.com/en/help/rest/apiref/layersource.html).",properties:{gdbVersion:{type:"string",description:"If applicable, specify this to use an alternate geodatabase version."},mapLayerId:{type:"integer",description:"The current map layer's id."},type:{type:"string",description:"A string value indicating the type.",enum:["mapLayer"]}},required:["mapLayerId","type"],additionalProperties:!1},"template_schema.json":{title:"template",type:"object",description:"Templates describe features that can be created in a layer. They are generally used with feature collections and editable web-based CSV layers. Templates are not used with ArcGIS feature services as these already have templates defined in the service. They are also defined as properties of the layer definition when there are no defined types. Otherwise, templates are defined as properties of the types.",properties:{description:{type:"string",description:"A string value containing a detailed description of the template."},drawingTool:{type:"string",description:"An optional string that can define a client-side drawing tool to be used with this feature.",enum:["esriFeatureEditToolAutoCompletePolygon","esriFeatureEditToolCircle","esriFeatureEditToolDownArrow","esriFeatureEditToolEllipse","esriFeatureEditToolFreehand","esriFeatureEditToolLeftArrow","esriFeatureEditToolLine","esriFeatureEditToolNone","esriFeatureEditToolPoint","esriFeatureEditToolPolygon","esriFeatureEditToolRectangle","esriFeatureEditToolRightArrow","esriFeatureEditToolText","esriFeatureEditToolTriangle","esriFeatureEditToolUpArrow"]},name:{type:"string",description:"A string containing a user-friendly name for the template."},prototype:{description:"A feature object representing a prototypical feature for the template.",allOf:[{$ref:"#/definitions/feature_schema.json"},{properties:{attributes:{}},additionalProperties:!1}]}},additionalProperties:!1},"labelExpressionInfo_schema.json":{title:"labelExpressionInfo",type:"object",description:"The labelExpressionInfo allows label text to be read similar to that of Popups's description property.",properties:{expression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to either a string or a number."}},additionalProperties:!1},"labelSymbol3D_schema.json":{title:"LabelSymbol3D",type:"object",description:"LabelSymbol3D is used to render labels for features from a FeatureLayer in a 3D SceneView.",properties:{callout:{$ref:"#/definitions/callout_schema.json"},symbolLayers:{type:"array",description:"A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",items:{oneOf:[{$ref:"#/definitions/textSymbol3DLayer_schema.json"}]}},type:{type:"string",description:"Specifies the type of symbol used.",enum:["LabelSymbol3D"]},verticalOffset:{$ref:"#/definitions/verticalOffset_schema.json"}},required:["symbolLayers","type"],additionalProperties:!1},"classBreaksRenderer_schema.json":{title:"ClassBreaks Renderer",type:"object",description:"A class breaks renderer symbolizes based on the value of some numeric attribute. The classBreakInfo define the values at which the symbology changes.",properties:{authoringInfo:{description:"An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",$ref:"#/definitions/authoringInfo_schema.json"},backgroundFillSymbol:{description:"Supported only for polygon features.",$ref:"#/definitions/polygonSymbol3D_schema.json"},classBreakInfos:{type:"array",description:"Array of classBreakInfo objects.",items:{$ref:"#/definitions/classBreakInfo_schema.json"}},defaultLabel:{type:"string",description:"Label for the default symbol used to draw unspecified values."},defaultSymbol:{description:"Symbol used when a value cannot be classified.",$ref:"#/definitions/symbol3D_schema.json"},field:{type:"string",description:"Attribute field used for renderer."},legendOptions:{description:"A legend containing one title, which is a string describing the renderer in the legend.",$ref:"#/definitions/rendererLegendOptions_schema.json"},minValue:{type:"number",description:"The minimum numeric data value needed to begin class breaks."},normalizationField:{type:"string",description:"Used when normalizationType is field. The string value indicating the attribute field by which the data value is normalized."},normalizationTotal:{type:"number",description:"Used when normalizationType is percent-of-total, this number property contains the total of all data values."},normalizationType:{type:"string",description:"Determine how the data was normalized.",enum:["esriNormalizeByField","esriNormalizeByLog","esriNormalizeByPercentOfTotal"]},type:{type:"string",description:"Specifies the type of renderer used.",enum:["classBreaks"]},valueExpression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number."},valueExpressionTitle:{type:"string",description:"The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property."},visualVariables:{type:"array",description:"An array of objects used to set rendering properties.",items:{$ref:"#/definitions/visualVariable_schema.json"}}},required:["classBreakInfos","type"],additionalProperties:!1},"heatmapRenderer_schema.json":{title:"Heatmap Renderer",type:"object",description:"The HeatmapRenderer renders point data into a raster visualization that emphasizes areas of higher density or weighted values.",properties:{colorStops:{type:"array",description:"An array of colorStop objects describing the renderer's color ramp with more specificity than just colors.",items:{$ref:"#/definitions/heatmap_colorStop_schema.json"}},field:{type:"string",description:"This is optional as this renderer can be created if no field is specified. Each feature gets the same value/importance/weight or with a field where each feature is weighted by the field's value."},legendOptions:{description:"Options for describing the heatmap in the legend.",allOf:[{$ref:"#/definitions/heatmapLegendOptions_schema.json"},{properties:{maxLabel:{},minLabel:{},title:{}},additionalProperties:!1}]},maxDensity:{type:"number",description:"The density value assigned to the final color in the `colorStops`."},minDensity:{type:"number",description:"The density value assigned to the first color in the `colorStops`."},radius:{type:"number",description:"The radius (in points) of the circle representing each point."},referenceScale:{type:"number",description:"An optional reference scale to make the heatmap visualization static (i.e. then the heatmap radius is defined as points at this certain view scale). A value of 0 means that the heatmap does not have a fixed reference scale.",default:0},type:{type:"string",description:"Specifies the type of renderer used.",enum:["heatmap"]}},required:["colorStops","type"],additionalProperties:!1},"pointCloudRenderers_schema.json":{title:"PointCloud Renderers",type:"object",description:"Renderers for PointCloud Layers.",oneOf:[{$ref:"#/definitions/pointCloudClassBreaksRenderer_schema.json"},{$ref:"#/definitions/pointCloudRGBRenderer_schema.json"},{$ref:"#/definitions/pointCloudStretchRenderer_schema.json"},{$ref:"#/definitions/pointCloudUniqueValueRenderer_schema.json"}]},"rasterRenderers_schema.json":{title:"Raster Renderers",type:"object",description:"Renderers for Raster Layers.",oneOf:[{$ref:"#/definitions/rasterClassBreaksRenderer_schema.json"},{$ref:"#/definitions/rasterColormapRenderer_schema.json"},{$ref:"#/definitions/rasterShadedReliefRenderer_schema.json"},{$ref:"#/definitions/rasterStretchRenderer_schema.json"},{$ref:"#/definitions/rasterUniqueValueRenderer_schema.json"}]},"simpleRenderer_schema.json":{title:"Simple Renderer",type:"object",description:"A simple renderer is a renderer that uses one symbol only.",properties:{authoringInfo:{description:"An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",$ref:"#/definitions/authoringInfo_schema.json"},description:{type:"string",description:"Description of the renderer."},label:{type:"string",description:"The text string that is displayed in the table of contents."},symbol:{description:"An object that represents how all features will be drawn.",$ref:"#/definitions/symbol3D_schema.json"},type:{type:"string",description:"Specifies the type of renderer used.",enum:["simple"]},visualVariables:{type:"array",description:"An array of objects used to set rendering properties.",items:{$ref:"#/definitions/visualVariable_schema.json"}}},required:["symbol","type"],additionalProperties:!1},"uniqueValueFromStyleRenderer_schema.json":{title:"UniqueValueFromStyle Renderer",type:"object",description:"This is a special version of UniqueValue Renderer, which loads uniqueValue infos from a style definition.",properties:{defaultLabel:{type:"string",description:"Default string for the default symbol used to draw unspecified values."},defaultSymbol:{description:"The defaultSymbol on the renderer that get assigned to features with no value or features that do not fall within the configured data.",$ref:"#/definitions/symbol3D_schema.json"},field1:{type:"string",description:"String value specifying the first field used to render matching values."},styleName:{type:"string",description:"A registered web style name. Can not be used in conjunction with styleUrl."},styleUrl:{type:"string",description:"URL that points to the web style definition. Can not be used in conjunction with styleName."},type:{type:"string",description:"Specifies the type of renderer used.",enum:["uniqueValue"]},visualVariables:{type:"array",description:"An array of visualVariable objects used for continuous color or size, simple, and unique values with feature opacity.",items:{$ref:"#/definitions/visualVariable_schema.json"}}},oneOf:[{required:["styleName","type"]},{required:["styleUrl","type"]}],additionalProperties:!1},"uniqueValueRenderer_schema.json":{title:"UniqueValue Renderer",type:"object",description:"A unique value renderer symbolizes based on the value of an attribute.",properties:{authoringInfo:{description:"An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",$ref:"#/definitions/authoringInfo_schema.json"},backgroundFillSymbol:{description:"Used for polygon features. It can only be used for bivariate types and size rendering.",$ref:"#/definitions/polygonSymbol3D_schema.json"},defaultLabel:{type:"string",description:"Default label for the default symbol used to draw unspecified values."},defaultSymbol:{description:"The defaultSymbol on the renderer that get assigned to features with no value or features that do not fall within the configured data.",$ref:"#/definitions/symbol3D_schema.json"},field1:{type:"string",description:"Attribute field renderer uses to match values."},field2:{type:"string",description:"If needed, specify an additional attribute field the renderer uses to match values."},field3:{type:"string",description:"If needed, specify an additional attribute field the renderer uses to match values."},fieldDelimiter:{type:"string",description:"String inserted between the values if multiple attribute fields are specified.",default:", "},legendOptions:{description:"A legend containing one title, which is a string describing the renderer in the legend.",$ref:"#/definitions/rendererLegendOptions_schema.json"},type:{type:"string",description:"Specifies the type of renderer used.",enum:["uniqueValue"]},uniqueValueGroups:{type:"array",description:"An array of uniqueValueGroup objects. If present, then `uniqueValueGroups` should be used in favor of `uniqueValueInfos`.",items:{$ref:"#/definitions/uniqueValueGroup_schema.json"}},uniqueValueInfos:{type:"array",description:"An array of uniqueValueInfo objects. If `uniqueValueGroups` is present, then this property should be ignored on rad. In the future, this property will be deprecated and eventually removed.",items:{$ref:"#/definitions/uniqueValueInfo_schema.json"}},valueExpression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to either a string or a number."},valueExpressionTitle:{type:"string",description:"The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property."},visualVariables:{type:"array",description:"An array of objects used to set rendering properties.",items:{$ref:"#/definitions/visualVariable_schema.json"}}},required:["type","uniqueValueInfos"],additionalProperties:!1},"format_schema.json":{title:"format",type:"object",description:"The format object can be used with numerical or date fields to provide more detail about how values should be formatted for display.",properties:{dateFormat:{type:"string",description:"A string used with date fields to specify how the date should be formatted.",enum:["dayShortMonthYear","dayShortMonthYearLongTime","dayShortMonthYearLongTime24","dayShortMonthYearShortTime","dayShortMonthYearShortTime24","longDate","longDateLongTime","longDateLongTime24","longDateShortTime","longDateShortTime24","longMonthDayYear","longMonthDayYearLongTime","longMonthDayYearLongTime24","longMonthDayYearShortTime","longMonthDayYearShortTime24","longMonthYear","shortDate","shortDateLE","shortDateLELongTime","shortDateLELongTime24","shortDateLEShortTime","shortDateLEShortTime24","shortDateLongTime","shortDateLongTime24","shortDateShortTime","shortDateShortTime24","shortMonthYear","year"]},digitSeparator:{type:"boolean",description:"A Boolean used with numerical fields. A value of true allows the number to have a digit (or thousands) separator. Depending on the locale, this separator is a decimal point or a comma. A value of false means that no separator will be used."},places:{type:"integer",description:"An integer used with numerical fields to specify the number of decimal places. Any places beyond this value are rounded."}},additionalProperties:!1},"mediaInfo_chart_schema.json":{title:"mediaInfo chart",type:"object",description:"Defines a chart to be displayed in a popup window.",properties:{altText:{type:"string",description:"A string providing the alternate text for the media."},caption:{type:"string",description:"A string caption describing the media."},title:{type:"string",description:"A string title for the media."},type:{type:"string",description:"A string defining the type of media.",enum:["barchart","columnchart","linechart","piechart"]},value:{$ref:"#/definitions/mediaInfo_chart_value_schema.json"}},required:["type","value"],additionalProperties:!1},"mediaInfo_image_schema.json":{title:"mediaInfo image",type:"object",description:"Defines an image to be displayed in a popup window.",properties:{altText:{type:"string",description:"A string providing the alternate text for the media."},caption:{type:"string",description:"A string caption describing the media."},refreshInterval:{type:"number",description:"Refresh interval of the layer in minutes. Non-zero value indicates automatic layer refresh at the specified interval. Value of 0 indicates auto refresh is not enabled. If the property does not exist, it's equivalent to having a value of 0."},title:{type:"string",description:"A string title for the media."},type:{type:"string",description:"A string defining the type of media.",enum:["image"]},value:{$ref:"#/definitions/mediaInfo_image_value_schema.json"}},required:["type","value"],additionalProperties:!1},"popupElement_attachments_schema.json":{title:"popupElement attachments",type:"object",description:"Configures attachments in popup elements.",properties:{description:{type:"string",description:"An optional string value describing the element in detail. This property does not apply when the `type` is `text` or `expression`."},displayType:{type:"string",description:"This property applies to elements of type `attachments`. A string value indicating how to display the attachment. If `list` is specified, attachments show as links. If `preview` is specified, attachments expand to the width of the pop-up. Setting the value to `auto` allows applications to choose the most suitable default experience for their application. ",enum:["auto","list","preview"]},title:{type:"string",description:"An optional string value indicating what the element represents. This property does not apply when the `type` is `text` or `expression`."},type:{type:"string",enum:["attachments"]}},required:["type"],additionalProperties:!1},"popupElement_expression_schema.json":{title:"popupElement expression",type:"object",description:"A pop-up element defined by an arcade expression.",properties:{expressionInfo:{description:"This property applies to elements of type `expression` . An Arcade expression that defines the pop-up element content. The return type will always be `dictionary` as outlined [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element).",$ref:"#/definitions/popupElementExpressionInfo_schema.json"},type:{type:"string",description:"String value indicating which elements to use. When an element is of type `expression` then `expressionInfo` is the only valid property.",enum:["expression"]}}},"popupElement_fields_schema.json":{title:"popupElement fields",type:"object",description:"Configures fields in popup elements.",properties:{attributes:{type:"object",description:"A dictionary of key value pairs representing attributes to be used instead of fields and their values. This property is only used when an element of type `fields` is being returned inside an element of type `expression` and should be returned as part of the arcade expression itself. This property allows passing arcade derived attribute values into `fields` elements. More details can be found [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element)."},description:{type:"string",description:"An optional string value describing the element in detail. This property does not apply when the `type` is `text` or `expression`."},fieldInfos:{type:"array",description:"It is an array of `fieldInfo` objects representing a field/value pair displayed as a table within the popupElement. If the `fieldInfos` property is not provided, the popupElement will display whatever is specified directly in the `popupInfo.fieldInfos` property.",items:{$ref:"#/definitions/fieldInfo_schema.json"}},title:{type:"string",description:"An optional string value indicating what the element represents. This property does not apply when the `type` is `text` or `expression`."},type:{type:"string",enum:["fields"]}},required:["type"],additionalProperties:!1},"popupElement_media_schema.json":{title:"popupElement media",type:"object",description:"Configures media in popup elements.",properties:{attributes:{type:"object",description:"A dictionary of key value pairs representing attributes to be used instead of fields and their values.  This property is only used when an element of type  `media` is being returned inside an element of type `expression` and should be returned as part of the arcade expression itself. This property allows passing arcade derived attribute values into `mediaInfos` such as charts. More details can be found [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element)."},description:{type:"string",description:"An optional string value describing the element in detail. This property does not apply when the `type` is `text` or `expression`."},mediaInfos:{type:"array",description:"An array of `mediaInfo` objects representing an image or chart for display. If no `mediaInfos` property is provided, the popupElement will display whatever is specified in the `popupInfo.mediaInfos` property.",items:{$ref:"#/definitions/mediaInfo_schema.json"}},title:{type:"string",description:"An optional string value indicating what the element represents. This property does not apply when the `type` is `text` or `expression`."},type:{type:"string",enum:["media"]}},required:["type"],additionalProperties:!1},"popupElement_relationship_schema.json":{title:"popupElement relationship",type:"object",description:"Provides the ability to navigate and view related records from a layer or table associated within the pop-up.",properties:{description:{type:"string",description:"A string that describes the element in detail."},displayCount:{type:"integer",description:"An integer that indicates the maximum number of records to display."},displayType:{type:"string",description:"A string that defines how the related records should be displayed.",enum:["list"]},orderByFields:{type:"array",description:"Array of `orderByField` objects indicating the display order for the related records, and whether they should be sorted in ascending `'asc'` or descending `'desc'` order. If `orderByFields` is not provided, the popupElement will display whatever is specified directly in the `popupInfo.relatedRecordsInfo.orderByFields` property.",items:{type:"object",$ref:"#/definitions/orderByField_schema.json"}},relationshipId:{type:"integer",description:"The id of the relationship as defined in the feature layer definition"},title:{type:"string",description:"A string value indicating what the element represents."},type:{type:"string",description:"String value indicating which type of element to use.",enum:["relationship"]}},required:["displayType","relationshipId","type"],additionalProperties:!1},"popupElement_text_schema.json":{title:"popupElement text",type:"object",description:"Configures text in popup elements.",properties:{text:{type:"string",description:"This property applies to elements of type `text`. This is string value indicating the text to be displayed within the popupElement. If no `text` property is provided, the popupElement will display whatever is specified in the [popupInfo.description](popupInfo.md) property."},type:{type:"string",enum:["text"]}},required:["type"],additionalProperties:!1},"search_layer_schema.json":{title:"search layer",type:"object",description:"Layer configuration for search.",properties:{field:{$ref:"#/definitions/search_field_schema.json"},id:{type:"string",description:"A string identifying the layer."},subLayer:{type:"integer",description:"Optional index for a sublayer."}},required:["field","id"],additionalProperties:!1},"search_table_schema.json":{title:"search table",type:"object",description:"Search configuration for table.",properties:{field:{$ref:"#/definitions/search_field_schema.json"},id:{type:"string",description:"A string identifying the table."}},required:["field","id"],additionalProperties:!1},"sunLighting_schema.json":{title:"sunLighting",type:"object",description:"Object containing information for the sun lighting type. The position of the light is set to the sun's location.",properties:{datetime:{type:"number",description:"The current date and time of the simulated sun. It is a number representing the number of milliseconds since 1 January, 1970 UTC."},directShadows:{type:"boolean",description:"Indicates whether to show shadows cast by the main light source.",default:!1},displayUTCOffset:{type:"number",description:"UTC offset in decimal hours. Not to be applied to datetime for sun position, only to adjust display of datetime in UI. If displayUTCOffset is null, offset is calculated for the current location (approximate only)."},type:{type:"string",description:"The type of lighting",enum:["sun"],default:"sun"}},additionalProperties:!1},"virtualLighting_schema.json":{title:"virtualLighting",type:"object",description:"Object containing information for the virtual lighting type. The position of the light follows the camera and is set behind the camera with a small offset to the left side.",properties:{directShadows:{type:"boolean",description:"Indicates whether to show shadows cast by the main light source.",default:!1},type:{type:"string",description:"The type of lighting",enum:["virtual"]}},required:["type"],additionalProperties:!1},"point_geometry_schema.json":{title:"pointGeometry",type:"object",description:"Defines the JSON formats of the point and spatial reference objects.",properties:{m:{type:"number",description:"M coordinate which contains measures used for linear referencing."},spatialReference:{description:"The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",$ref:"#/definitions/spatialReference_schema.json"},x:{description:"X coordinate which is measured along the east/west axis.",oneOf:[{type:"number"},{type:"null"},{type:"string",enum:["NaN"]}]},y:{type:"number",description:"Y coordinate which is measured along the north/south axis."},z:{type:"number",description:"Z coordinate which measures height or elevation."}},required:["x"],additionalProperties:!1},"multipoint_geometry_schema.json":{title:"multipointGeometry",type:"object",description:"Contains an array of points, along with a spatial reference field.",properties:{hasM:{type:"boolean",description:"Indicates whether the geometry contains M coordinate values."},hasZ:{type:"boolean",description:"Indicates whether the geometry contains Z coordinate values."},points:{type:"array",description:"An array that corresponds to 2D and 3D points.",items:{type:"array",items:{type:"number"},minItems:2,maxItems:4}},spatialReference:{description:"The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",$ref:"#/definitions/spatialReference_schema.json"}},required:["points"],additionalProperties:!1},"polygon_geometry_schema.json":{title:"polygonGeometry",type:"object",description:"A polygon contains an array of rings and a spatial reference.",properties:{hasM:{type:"boolean",description:"Indicates whether the geometry contains M coordinate values."},hasZ:{type:"boolean",description:"Indicates whether the geometry contains Z coordinate values."},rings:{type:"array",description:"Represents an array of rings. Each ring is an array of points.",items:{type:"array",items:{type:"array",items:{type:"number"},minItems:2,maxItems:4},minItems:3},minItems:1},spatialReference:{description:"The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",$ref:"#/definitions/spatialReference_schema.json"}},required:["rings"],additionalProperties:!1},"polyline_geometry_schema.json":{title:"polylineGeometry",type:"object",description:"Contains an array of paths and a spatialReference.",properties:{hasM:{type:"boolean",description:"Indicates whether the geometry contains M coordinate values."},hasZ:{type:"boolean",description:"Indicates whether the geometry contains Z coordinate values."},paths:{type:"array",description:"Three nested array levels describing a polyline. The first level describes individual paths; the second describes the points of these paths; and the third gives the coordinates of the points.",items:{type:"array",items:{type:"array",items:{type:"number"},minItems:2,maxItems:4},minItems:2}},spatialReference:{description:"The spatial reference can be defined using a well-known ID (WKID) or well-known text (WKT).",$ref:"#/definitions/spatialReference_schema.json"}},required:["paths"],additionalProperties:!1},"rasterDrawingInfo_schema.json":{title:"Raster Drawing Info",type:"object",description:"Contains the drawing information for image service based layers.",properties:{renderer:{description:"An object defining the symbology for the layer.",$ref:"#/definitions/renderer_schema.json"}},additionalProperties:!1},"rasterPresetRenderer_schema.json":{title:"Raster Preset Renderer Definition",type:"object",description:"Defines a raster renderer with associated associated information of multidimensional variable or raster function template.",properties:{bandIds:{type:"array",description:"An array of band ids that are visible.",items:{type:"integer"}},method:{type:"string",description:"The method by which to apply the renderer.",enum:["none","rasterFunctionTemplate","variable"]},name:{type:"string",description:"The unique name of the renderer."},renderer:{type:"object",description:"The renderer object that can be applied to the layer.",$ref:"#/definitions/renderer_schema.json"},value:{type:"string",description:"The corresponding variable or raster function template name."}},required:["bandIds","method","name","renderer","value"],additionalProperties:!1},"rasterFunctionTemplate_schema.json":{title:"Raster Function Template",type:"object",description:"Raster function template containing one or more raster functions chained together to produce a processing workflow.",properties:{_object_id:{type:"integer",description:"The id of the raster function template."},aliases:{type:"array",description:"Aliases for the `function` referenced in the template.",items:{type:"string"}},arguments:{description:"The arguments for the `function` referenced in the raster function template.",$ref:"#/definitions/rasterFunctionTemplateArguments_schema.json"},definition:{type:"string",description:"A query definition to filter rasters when the layer's data source is a mosaic dataset."},description:{type:"string",description:"The description of the raster function template."},function:{description:"The raster function referenced by the raster function template.",$ref:"#/definitions/rasterFunctionInfo_schema.json"},functionType:{type:"number",description:"Defines whether the `function` is applied to a mosaic dataset. Indicates the level of the mosaic processing is used. Only applies to mosaic based image services. `0` - function is applied after mosaicking; `1` - function is applied on each raster item before mosaicking; `2` - function is applied to a group of raster items before mosaicking.",enum:[0,1,2]},group:{type:"string",description:"Group field name for item group function template "},help:{type:"string",description:"The short description of what the raster function template does."},name:{type:"string",description:"Name of the raster function template."},properties:{type:"object",description:"The properties used to define multidimensional dataset processing rules.",properties:{_object_id:{type:"integer"},type:{type:"string"}},required:["type"],additionalProperties:{type:"object",properties:{_object_id:{type:"integer"},isDataset:{type:"boolean"},isPublic:{type:"boolean"},name:{type:"string"},type:{type:"string"},value:{type:["integer","string","object","array","boolean"]}},required:["name","type"],additionalProperties:!1}},tag:{type:"string",description:"Tag field name for item group function template ."},thumbnail:{type:"string",description:"Thumbnail url of the raster function template."},thumbnailEx:{type:"string",description:"Base64 encoded thumbnail of the raster function template."},type:{type:"string",description:"Type of the raster function template.",enum:["RasterFunctionTemplate"]}},required:["arguments","description","function","functionType","name"],additionalProperties:!1},"featureReference_schema.json":{title:"featureReference",type:"object",description:"References a feature by object and layer id.",properties:{layerId:{type:["number","string"],description:"Identifies the layer to which the feature belongs."},objectId:{type:["string","number"],description:"ObjectId identifying the feature within the layer."}},required:["layerId","objectId"],additionalProperties:!1},"mediaLayer_georeference_controlPoint_schema.json":{title:"Control Point",type:"object",description:"A location in pixels in the media of the [MediaLayer](mediaLayer.md). The origin is located at the top-left corner of the media.",properties:{x:{type:"number",description:"The X coordinate in pixels. The value must be between 0, and the width of the [georeference](mediaLayer_georeference_schema.md)"},y:{type:"number",description:"The Y coordinate in pixels. The value must be between 0, and the height of the [georeference](mediaLayer_georeference_schema.md)"}},required:["x","y"],additionalProperties:!1},"voxelSection_schema.json":{title:"Voxel Section",type:"object",description:"A section that is locked in for a specific variable and/or time. Allowing you to view the section together with other variables. A locked (or static) section is `float32` raster. This JSON object contains the meta-data needed to display (and potentially re-generate) this raster.\n\n When creating a web scene the locked sections must have this information as all properties come the web scene. If no section information is given, it is assumed there are no sections in the web scene.",properties:{enabled:{type:"boolean",description:"Boolean defining section visibility.",default:!0},href:{type:"string",description:"Relative href to the static section raster binary. href is relative to the layer document which contains the href. Inside an SLPK archive, must be of the form `resources/sections/<file>.bin.gz` and the section raster should be gzip-compressed."},id:{type:"integer",description:"The id of the section. Must be unique from other sections in the array."},label:{type:"string",description:"The label for the section."},normal:{type:"array",description:"Normal vector to the plane in voxel space.",items:{type:"number"},minItems:3,maxItems:3},point:{type:"array",description:"Point belonging to the section plane in voxel space.",items:{type:"number"},minItems:3,maxItems:3},sizeInPixel:{type:"array",description:"Array of size 2 which describes the dimension of the raster data in pixels.",items:{type:"integer"},minItems:2,maxItems:2},slices:{type:"array",description:"A copy of the slices that were applied to the volume when the section was created when new UVs are discovered for subsequent time slices.",items:{$ref:"#/definitions/voxelSlice_schema.json"}},timeId:{type:"integer",description:"Time slice id at which the section was created. `timeId` is only applicable when time is the 4th dimension."},variableId:{type:"integer",description:"Id of the variable."}},required:["href","id","normal","point","sizeInPixel","variableId"],additionalProperties:!1},"voxelStyle_schema.json":{title:"Voxel Style",type:"object",description:"A voxel style allows you to define the visualization of the voxel layer. A voxel layer can be represented as volume or surface depending on your analysis needs. You can change the visibility of the different representations and change their drawing options.",properties:{currentVariableId:{type:"integer",description:"Id of the currently visible variable."},enableDynamicSections:{type:"boolean",description:'Enable dynamic section (only if `renderMode = "surfaces"`).'},enableIsosurfaces:{type:"boolean",description:'Enable isosurfaces (only if `renderMode = "surfaces"`).'},enableSections:{type:"boolean",description:"Enable static sections."},enableSlices:{type:"boolean",description:"Enable slices."},renderMode:{type:"string",description:"Current rendering mode for the voxel layer. Depending on the rendering mode different voxel layer representations can be shown. `volume` draws the voxel layer as rectangular cuboid (but any slices defined for the voxel layer will change the volume to the area of interest). `surfaces` will represent the layer as a set of surfaces, for example, dynamic sections can define a plane through the volume or isosurfaces can show a specific value as surface.",enum:["surfaces","volume"]},shading:{description:"Defines the shading properties.",$ref:"#/definitions/voxelShading_schema.json"},variableStyles:{type:"array",description:"Array of variable styles. Each variable can have one variable style.",items:{$ref:"#/definitions/voxelVariableStyle_schema.json"},minItems:1},volumeStyles:{type:"array",description:"Array of volume styles. Currently only one volume style is allowed.",items:{$ref:"#/definitions/voxelVolumeStyle_schema.json"},minItems:1}},required:["currentVariableId"],additionalProperties:!1},"lod_schema.json":{title:"lod",type:"object",properties:{level:{type:"integer",description:"ID for each level."},levelValue:{type:"string",description:"String to be used when constructing URL to access a tile from this LOD."},resolution:{type:"number",description:"Resolution in map units of each pixel in a tile for each level."},scale:{type:"number",description:"Scale for each level."}},additionalProperties:!1},"timeStopInterval_schema.json":{title:"timeStopInterval",type:"object",description:"The interval set for the time slider widget.",properties:{interval:{type:"number",description:"The length of the time interval."},units:{type:"string",description:"The units used to define the interval.",enum:["esriTimeUnitsCenturies","esriTimeUnitsDays","esriTimeUnitsDecades","esriTimeUnitsHours","esriTimeUnitsMilliseconds","esriTimeUnitsMinutes","esriTimeUnitsMonths","esriTimeUnitsSeconds","esriTimeUnitsWeeks","esriTimeUnitsYears"]}},additionalProperties:!1},"buildingSceneLayer_filterAuthoringInfo_filterType_schema.json":{title:"Filter Type",type:"object",description:"The file authoring information for a filter, including the filter type and its value settings.",properties:{filterType:{type:"string",description:"Represents the filter type name. Name is a unique identifier."},filterValues:{type:"array",description:"Array of filter values. Filter values are the attributes that can be stored for individual fields in a layer.",items:{type:"string"}}},required:["filterType","filterValues"],additionalProperties:!1},"buildingSceneLayer_filterModeSolid_schema.json":{title:"Filter mode solid",type:"object",description:"Draw elements of this filter block in solid mode. This mode does not change the display of features.",properties:{type:{type:"string",description:"Declares filter mode of type solid.",enum:["solid"]}},required:["type"],additionalProperties:!1},"buildingSceneLayer_filterModeWireFrame_schema.json":{title:"Filter mode wire frame",type:"object",description:"Draw elements of this filter block in wireframe mode. This mode draws only the edges of the features with the specified edge style.",properties:{edges:{description:"An object defining edges of a feature.",$ref:"#/definitions/edges_schema.json"},type:{type:"string",description:"Declares filter mode of type wire frame.",enum:["wireFrame"]}},required:["edges","type"],additionalProperties:!1},"buildingSceneLayer_filterModeXray_schema.json":{title:"Filter mode x-ray",type:"object",description:"Draw elements of this filter block in x-ray mode. `x-ray` renders elements semi-transparent with white color.",properties:{type:{type:"string",description:"Declares filter mode of type x-ray.",enum:["x-ray"]}},required:["type"],additionalProperties:!1},"codedValue_domain_schema.json":{title:"codedValue",type:"object",description:"The coded value domain includes both the actual value that is stored in a database and a description of what the code value means.",properties:{codedValues:{type:"array",description:"A set of valid values with unique names.",items:{$ref:"#/definitions/codedValue_schema.json"},uniqueItems:!0},name:{type:"string",description:"The domain name."},type:{type:"string",description:"String value representing the domain type.",enum:["codedValue"]}},required:["codedValues","type"],additionalProperties:!1},"inherited_domain_schema.json":{title:"inheritedDomain",type:"object",description:"This domain applies to domains on subtypes. It implies that the domain for a field at the subtype level is the same as the domain for the field at the layer level.",properties:{name:{type:"string",description:"The domain name."},type:{type:"string",description:"String value representing the domain type.",enum:["inherited"]}},required:["type"],additionalProperties:!1},"range_domain_schema.json":{title:"rangeDomain",type:"object",description:"Range domain specifies a range of valid values for a field.",properties:{name:{type:"string",description:"The domain name."},range:{type:"array",description:"The first element is the minValue and the second element is the maxValue.",items:{type:"number"},minItems:2,maxItems:2,uniqueItems:!0},type:{type:"string",description:"String value representing the domain type.",enum:["range"]}},required:["range","type"],additionalProperties:!1},"dataSource_schema.json":{title:"dataSource",type:"object",description:"This object applies if the [layerDefinition](layerDefinition.md) source is set to `DynamicDataLayer`.",oneOf:[{$ref:"#/definitions/joinTableDataSource_schema.json"},{$ref:"#/definitions/queryTableDataSource_schema.json"},{$ref:"#/definitions/rasterDataSource_schema.json"},{$ref:"#/definitions/tableDataSource_schema.json"}]},"feature_schema.json":{title:"feature",type:"object",description:"Contains information about an attribute field and feature geometry.",properties:{attributes:{type:"object",description:"The feature attributes. A JSON object that contains a dictionary of name-value pairs. The names are the feature field names. The values are the field values, and they can be any of the standard JSON types: string, number, and boolean. Note that date values are encoded as numbers. The number represents the number of milliseconds since epoch (January 1, 1970) in UTC."},geometry:{description:"It can be any of the supported geometry types.",$ref:"#/definitions/geometry_schema.json"},symbol:{description:"Symbol used for drawing the feature.",$ref:"#/definitions/symbol3D_schema.json"}},additionalProperties:!1},"callout_schema.json":{title:"Callout",type:"object",description:"Callout configuration for a symbol.",properties:{border:{$ref:"#/definitions/border_schema.json"},color:{description:"The color of the line.",$ref:"#/definitions/color_schema.json"},size:{type:"number",description:"The width of the line in points.",minimum:0},transparency:{type:"integer",description:"A value between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100},type:{type:"string",description:"The type of the callout. A callout of type `line` connects an offset symbol or label with its location.",enum:["line"]}},required:["color","size","type"],additionalProperties:!1},"textSymbol3DLayer_schema.json":{title:"TextSymbol3DLayer",type:"object",description:"Symbol layer for text and font definitions.",properties:{background:{$ref:"#/definitions/textBackground_schema.json"},enable:{type:"boolean"},font:{$ref:"#/definitions/font_schema.json"},halo:{$ref:"#/definitions/halo_schema.json"},horizontalAlignment:{type:"string",description:"One of the following string values representing the horizontal alignment of the text.",enum:["center","left","right"],default:"center"},lineHeight:{type:"number",description:"Multiplier to scale the vertical distance between the baselines of text with multiple lines.",minimum:.1,maximum:4,default:1},material:{$ref:"#/definitions/material_schema.json"},size:{type:"number",description:"Font size in points, positive only",minimum:0},text:{type:"string",description:"Text content in the label. Typically this property is not set, as text content is read from labeling field."},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Text"]},verticalAlignment:{type:"string",description:"One of the following string values representing the vertical alignment of the text.",enum:["baseline","bottom","middle","top"],default:"baseline"}},required:["type"],additionalProperties:!1},"verticalOffset_schema.json":{title:"verticalOffset",type:"object",description:"Shifts the symbol along the vertical world axis by a given length. The length is set in screen space units.",properties:{maxWorldLength:{type:"number",description:"The maximum vertical symbol lift in world units. It acts as an upper bound to avoid lift becoming too big."},minWorldLength:{type:"number",description:"The minimum vertical symbol lift in world units. It acts as a lower bound to avoid lift becoming too small.",default:0},screenLength:{type:"number",description:"Maximal screen length of lift in points."}},required:["screenLength"],additionalProperties:!1},"authoringInfo_schema.json":{title:"authoringInfo",type:"object",description:"The authoringInfo is an object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via an authoring client, their selections are remembered. Non-authoring clients can ignore it. Properties for color/size/transparency sliders, theme selection, classification information, and additional properties are saved within this object.",properties:{classificationMethod:{type:"string",description:"Used for classed color or size. The default value is `esriClassifyManual`.",enum:["esriClassifyDefinedInterval","esriClassifyEqualInterval","esriClassifyManual","esriClassifyNaturalBreaks","esriClassifyQuantile","esriClassifyStandardDeviation"],default:"esriClassifyManual"},colorRamp:{$ref:"#/definitions/colorRamp_schema.json"},field1:{$ref:"#/definitions/field_authoringInfo_schema.json"},field2:{$ref:"#/definitions/field_authoringInfo_schema.json"},fields:{type:"array",description:"An array of string values representing field names used for creating predominance renderers.",items:{type:"string"}},focus:{type:"string",description:"Optional. Used for Relationship renderer. If not set, the legend will default to being square.",enum:["HH","HL","LH","LL"]},lengthUnit:{type:"string",description:"Unit used in user interfaces to display world/map sizes and distances",enum:["centimeters","decimeters","feet","inches","kilometers","meters","miles","millimeters","nautical-miles","yards"]},numClasses:{type:"integer",description:"Number of classes to be associated with the relationship. Used for Relationship renderer.",minimum:2,maximum:4},standardDeviationInterval:{type:"number",description:"Use this property if the classificationMethod is `esriClassifyStandardDeviation`.",enum:[.25,.33,.5,1]},type:{type:"string",enum:["classedColor","classedSize","dotDensity","predominance","relationship","univariateColorSize"]},visualVariables:{type:"array",description:"An array of visualVariable objects containing additional information needed when authoring the renderer.",items:{$ref:"#/definitions/authoringInfo_visualVariable_schema.json"}}},additionalProperties:!1},"polygonSymbol3D_schema.json":{title:"PolygonSymbol3D",type:"object",description:"PolygonSymbol3D is used to render features with Polygon geometry in a 3D SceneView. Polygon features may also be rendered as points with icons or objects at the centroid of each polygon.",properties:{styleOrigin:{$ref:"#/definitions/styleOrigin_schema.json"},symbolLayers:{type:"array",description:"A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",items:{oneOf:[{$ref:"#/definitions/extrudeSymbol3DLayer_schema.json"},{$ref:"#/definitions/fillSymbol3DLayer_schema.json"},{$ref:"#/definitions/iconSymbol3DLayer_schema.json"},{description:"<em>Deprecated</em>, use [fillSymbol3DLayer with outline](fillSymbol3DLayer.md) instead",$ref:"#/definitions/lineSymbol3DLayer_schema.json"},{$ref:"#/definitions/objectSymbol3DLayer_schema.json"},{description:"<em>Deprecated</em>",$ref:"#/definitions/textSymbol3DLayer_schema.json"},{$ref:"#/definitions/waterSymbol3DLayer_schema.json"}]}},type:{type:"string",description:"Specifies the type of symbol used.",enum:["PolygonSymbol3D"]}},required:["symbolLayers","type"],additionalProperties:!1},"classBreakInfo_schema.json":{title:"classBreakInfo",type:"object",description:"The classBreaksInfo object provides information about the class breaks associated with the renderer.",properties:{classMaxValue:{type:"number",description:"A numeric value used to specify the maximum value for a break."},classMinValue:{type:"number",description:"A numeric value used to specify discontinuous class breaks. If this value is null or is missing, the map server will calculate the minimum value based on the preceding class' maximum value."},description:{type:"string",description:"String value used to describe the drawn symbol."},label:{type:"string",description:"String value used to label the drawn symbol."},symbol:{description:"An object used to display the value.",$ref:"#/definitions/symbol3D_schema.json"}},required:["symbol"],additionalProperties:!1},"symbol3D_schema.json":{title:"Symbol3D",type:"object",description:"Symbol3D is the base class for all 3D symbols. It is used to render 2D Point, Polyline, and Polygon features in a FeatureLayer and 3D mesh features in a SceneLayer. All 3D symbols must be used in a SceneView instance; there is no support for 3D rendering in MapViews.",oneOf:[{$ref:"#/definitions/cimSymbolReference_schema.json"},{$ref:"#/definitions/lineSymbol3D_schema.json"},{$ref:"#/definitions/meshSymbol3D_schema.json"},{$ref:"#/definitions/pointSymbol3D_schema.json"},{$ref:"#/definitions/polygonSymbol3D_schema.json"},{$ref:"#/definitions/styleSymbolReference_schema.json"}]},"rendererLegendOptions_schema.json":{title:"Renderer Legend Options",type:"object",description:"Options available for the renderer legend.",properties:{order:{type:"string",description:"Indicates the order in which the legend is displayed.",enum:["ascendingValues","descendingValues"]},title:{type:"string",description:"The title of the legend."}},additionalProperties:!1},"visualVariable_schema.json":{title:"visualVariable",type:"object",description:"An object used to set rendering options. Please see the individual visual variable for specific information on how it is used.",oneOf:[{$ref:"#/definitions/colorInfo_visualVariable_schema.json"},{$ref:"#/definitions/rotationInfo_visualVariable_schema.json"},{$ref:"#/definitions/sizeInfo_visualVariable_schema.json"},{$ref:"#/definitions/transparencyInfo_visualVariable_schema.json"}]},"heatmap_colorStop_schema.json":{title:"heatmap colorStop",description:"A colorStop object describes the renderer's color ramp and how it is mapped to the intensity values. The color of the first stop (i.e. the stop with the lowest ratio value) must have an alpha value of 0 for the underlying basemap to be visible in the app.",properties:{color:{$ref:"#/definitions/color_schema.json"},ratio:{type:"number",description:"A number between 0-1. Describes what portion along the gradient the colorStop is added."}},required:["color","ratio"],additionalProperties:!1},"heatmapLegendOptions_schema.json":{title:"Heatmap Legend Options",type:"object",description:"Options available for the heatmap renderer legend.",properties:{maxLabel:{type:"string",description:"Text in the legend that describes the hottest (most dense) part of the heatmap. Only applicable to Heatmap renderers. If not specified, then a localized label, for 'High' will display on the legend."},minLabel:{type:"string",description:"Text in the legend that describes the coldest (least dense) part of the heatmap. Only applicable to Heatmap renderers. If not specified, then a localized label, for 'Low' will display on the legend."},title:{type:"string",description:"The title of the legend."}},additionalProperties:!1},"pointCloudClassBreaksRenderer_schema.json":{title:"PointCloud ClassBreaks Renderer",type:"object",description:"PointCloudClassBreaksRenderer defines the color of each point in a PointCloudLayer based on the value of a numeric attribute. Colors are assigned based on classes or ranges of data. Each point is assigned a symbol based on the class break in which the value of the attribute falls.",properties:{colorClassBreakInfos:{type:"array",description:"Each element in the array is an object that provides information about a class break associated with the renderer.",items:{$ref:"#/definitions/colorClassBreakInfo_schema.json"}},colorModulation:{$ref:"#/definitions/colorModulationInfo_schema.json"},field:{type:"string",description:"  The name of the field that is used for the renderer."},fieldTransformType:{type:"string",description:"A transform that is applied to the field value before evaluating the renderer.",enum:["absoluteValue","highFourBit","lowFourBit","moduloTen","none"]},legendOptions:{description:"A legend containing one title, which is a string describing the renderer in the legend.",$ref:"#/definitions/rendererLegendOptions_schema.json"},pointSizeAlgorithm:{$ref:"#/definitions/pointSizeAlgorithm_schema.json"},pointsPerInch:{type:"number",description:"Number of point to draw per display inch."},type:{type:"string",description:"Specifies the type of renderer used.",enum:["pointCloudClassBreaksRenderer"]}},required:["colorClassBreakInfos","field","type"],additionalProperties:!1},"pointCloudRGBRenderer_schema.json":{title:"PointCloud RGB Renderer",type:"object",description:"PointCloudRGBRenderer defines the color of each point in a PointCloudLayer based on the value of a color attribute.",properties:{colorModulation:{$ref:"#/definitions/colorModulationInfo_schema.json"},field:{type:"string",description:"The name of the field that is used for the renderer."},pointSizeAlgorithm:{$ref:"#/definitions/pointSizeAlgorithm_schema.json"},pointsPerInch:{type:"number",description:"Number of point to draw per display inch."},type:{type:"string",description:"Specifies the type of renderer used.",enum:["pointCloudRGBRenderer"]}},required:["field","type"],additionalProperties:!1},"pointCloudStretchRenderer_schema.json":{title:"PointCloud Stretch Renderer",type:"object",description:"PointCloudStretchRenderer defines the color of each point in a PointCloudLayer based on the value of a numeric attribute. They allow you to easily map continuous color ramps to minimum and maximum data values of one of the layer's numeric attribute fields.",properties:{colorModulation:{$ref:"#/definitions/colorModulationInfo_schema.json"},field:{type:"string",description:"The name of the field that is used for the renderer."},fieldTransformType:{type:"string",description:"A transform that is applied to the field value before evaluating the renderer.",enum:["absoluteValue","highFourBit","lowFourBit","moduloTen","none"]},legendOptions:{description:"A legend containing one title, which is a string describing the renderer in the legend.",$ref:"#/definitions/rendererLegendOptions_schema.json"},pointSizeAlgorithm:{$ref:"#/definitions/pointSizeAlgorithm_schema.json"},pointsPerInch:{type:"number",description:"Number of points to draw per display inch."},stops:{type:"array",description:"An array of color value pairs.",items:{$ref:"#/definitions/colorStop_schema.json"}},type:{type:"string",description:"Specifies the type of renderer used.",enum:["pointCloudStretchRenderer"]}},required:["field","stops","type"],additionalProperties:!1},"pointCloudUniqueValueRenderer_schema.json":{title:"PointCloud UniqueValue Renderer",type:"object",description:"PointCloudUniqueValueRenderer allows you to colorize points in a PointCloudLayer based on an attribute. This is done by using unique colors to represent points with equal attribute values.",properties:{colorModulation:{$ref:"#/definitions/colorModulationInfo_schema.json"},colorUniqueValueInfos:{type:"array",description:"Unique value infos.",items:{type:"object",$ref:"#/definitions/colorUniqueValueInfo_schema.json"}},field:{type:"string",description:"The name of the field that is used for the renderer."},fieldTransformType:{type:"string",description:"A transform that is applied to the field value before evaluating the renderer.",enum:["absoluteValue","highFourBit","lowFourBit","moduloTen","none"]},legendOptions:{description:"A legend containing one title, which is a string describing the renderer in the legend.",$ref:"#/definitions/rendererLegendOptions_schema.json"},pointSizeAlgorithm:{$ref:"#/definitions/pointSizeAlgorithm_schema.json"},pointsPerInch:{type:"number",description:"Number of points to draw per display inch."},type:{type:"string",description:"Specifies the type of renderer used.",enum:["pointCloudUniqueValueRenderer"]}},required:["colorUniqueValueInfos","field","type"],additionalProperties:!1},"rasterClassBreaksRenderer_schema.json":{title:"Raster ClassBreaks Renderer",type:"object",description:"Defines the color of each raster cell based on the value of a numeric attribute. Colors are assigned based on classed ranges of data.",properties:{authoringInfo:{description:"An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",$ref:"#/definitions/authoringInfo_schema.json"},colorClassBreakInfos:{type:"array",description:"Each element in the array is an object that provides information about a class break associated with the renderer.",items:{$ref:"#/definitions/colorClassBreakInfo_schema.json"}},defaultColor:{description:"The defaultColor on the renderer that get assigned to features with no value or features that do not fall within the configured data.",$ref:"#/definitions/color_schema.json"},defaultLabel:{type:"string",description:"Default label for the default symbol used to draw unspecified values."},field:{type:"string",description:"The name of the field that is used for the renderer."},legendOptions:{description:"A legend containing one title, which is a string describing the renderer in the legend.",$ref:"#/definitions/rendererLegendOptions_schema.json"},normalizationField:{type:"string",description:"Used when normalizationType is field. The string value indicating the attribute field by which the data value is normalized."},normalizationTotal:{type:"number",description:"Used when normalizationType is percent-of-total, this number property contains the total of all data values."},normalizationType:{type:"string",description:"Determine how the data was normalized.",enum:["esriNormalizeByField","esriNormalizeByLog","esriNormalizeByPercentOfTotal"]},type:{type:"string",description:"Specifies the type of renderer used.",enum:["rasterClassBreaksRenderer"]}},required:["colorClassBreakInfos","field","type"],additionalProperties:!1},"rasterColormapRenderer_schema.json":{title:"Raster Colormap Renderer",type:"object",description:"A value/color look up table used for rendering raster data only.",properties:{colormapInfos:{type:"array",description:"An array of colormapInfo objects.",items:{$ref:"#/definitions/colormapInfo_schema.json"}},type:{type:"string",description:"Specifies the type of renderer used.",enum:["rasterColormap"]}},required:["colormapInfos","type"],additionalProperties:!1},"rasterShadedReliefRenderer_schema.json":{title:"Raster Shaded Relief Renderer",type:"object",description:"This renderer produces a grayscale or a colored 3D representation of a shaded relief. This renderer can be used on either layers of type: [ImageServiceLayer](imageServiceLayer.md) or [TiledImageServiceLayer](tiledImageServiceLayer.md).",properties:{altitude:{type:"number",description:"Used when the `hillshadeType` is `traditional`. The  altitude of the illumination source."},azimuth:{type:"number",description:"Used when the `hillshadeType` is `traditional`. The azimuth of the illumination source."},colorRamp:{description:"The color ramp to display the shaded relief. When not specified, a grayscale hillshade is produced. When this is specified, grayscale hillshade is fused with colorized elevation.",$ref:"#/definitions/colorRamp_schema.json"},hillshadeType:{type:"string",description:"Use single (traditional), or multiple illumination sources to generate hillshade.",enum:["multi-directional","traditional"]},pixelSizeFactor:{type:"number",description:"Accounts for changes in scale as the viewer zooms in and out on the map display."},pixelSizePower:{type:"number",description:"Accounts for the altitude changes (or scale) as the viewer zooms in and out on the map display."},scalingType:{type:"string",description:"Apply a constant or adjusted z-factor based on resolution changes. The `adjusted` type is suitable for worldwide elevation dataset. An adjusted `zFactor` is determined using this equation: `Adjusted Z-Factor = (Z Factor) + (Pixel Size)` <sup>(Pixel Size Power)</sup> `x Pixel Size Factor`.",enum:["adjusted","none"]},type:{type:"string",description:"Specifies the type of renderer used.",enum:["rasterShadedRelief"]},zFactor:{type:"number",description:"A conversion factor that adjusts the units of measure for the vertical (or elevation) units when they are different from the horizontal coordinate (x,y) units of the input surface. It is the number of ground x,y units in one surface z-unit."}},required:["altitude","azimuth","hillshadeType","scalingType","type","zFactor"],additionalProperties:!1},"rasterStretchRenderer_schema.json":{title:"Raster Stretch Renderer",description:"Displays continuous raster cell values across a gradual ramp of colors. Use this renderer to draw a single band of continuous data. This renderer works well when you have a large range of values to display, such as with imagery or scientific data.",properties:{colorRamp:{$ref:"#/definitions/colorRamp_schema.json"},computeGamma:{type:"boolean",description:"Indicates if gamma values should be computed by default."},dra:{type:"boolean",description:"Indicates if Dynamic Range Adjustment should be applied."},gamma:{type:"array",description:"The list of Gamma value(s).",items:{type:"number"}},max:{type:"number",description:"The current maximum output value."},maxPercent:{type:"number",description:"The current maximum percent value."},min:{type:"number",description:"The current minimum output value."},minPercent:{type:"number",description:"The current minimum percent value."},numberOfStandardDeviations:{type:"number",description:"The number of standard deviations for standard deviation stretch."},sigmoidStrengthLevel:{type:"number",description:"Set this from (1 to 6) to adjust the curvature of Sigmoid curve used in color stretch."},statistics:{type:"array",description:"The custom raster stretch statistics.",items:{type:"array",items:{type:"number",minItems:4,maxItems:4}}},stretchType:{type:"string",description:"The stretch types for stretch raster function.",enum:["histogramEqualization","minMax","none","percentClip","sigmoid","standardDeviation"]},type:{type:"string",description:"Specifies the type of renderer used.",enum:["rasterStretch"]},useGamma:{type:"boolean",description:"Indicates if the renderer applies Gamma stretch."}},additionalProperties:!1},"rasterUniqueValueRenderer_schema.json":{title:"Raster UniqueValue Renderer",type:"object",description:"Defines the color of each raster cell based on an attribute. This is done by using unique colors to represent cells with equal attribute values.",properties:{authoringInfo:{description:"An object containing metadata about the authoring process for creating a renderer object. This allows the authoring clients to save specific overridable settings so that next time it is accessed via the UI, their selections are remembered. Non-authoring clients can ignore it.",$ref:"#/definitions/authoringInfo_schema.json"},colorUniqueValueInfos:{type:"array",description:"Unique value infos.",items:{$ref:"#/definitions/colorUniqueValueInfo_schema.json"}},defaultColor:{description:"The defaultColor on the renderer that get assigned to features with no value or features that do not fall within the configured data.",$ref:"#/definitions/color_schema.json"},defaultLabel:{type:"string",description:"Default label for the default symbol used to draw unspecified values."},field:{type:"string",description:"The name of the field that is used for the renderer."},legendOptions:{description:"A legend containing one title, which is a string describing the renderer in the legend.",$ref:"#/definitions/rendererLegendOptions_schema.json"},type:{type:"string",description:"Specifies the type of renderer used.",enum:["rasterUniqueValueRenderer"]}},required:["colorUniqueValueInfos","field","type"],additionalProperties:!1},"uniqueValueGroup_schema.json":{title:"Unique Value Group",type:"object",description:"Represents a group of unique value classes (i.e. symbols). This is used to group symbols under a common heading and/or when representing multiple unique values with a single symbol.",properties:{classes:{type:"array",description:"Specifies the classes (i.e. symbols) to group under a common heading. Classes may be included here without a heading when representing multiple values with a single symbol.",items:{$ref:"#/definitions/uniqueValueClass_schema.json",minItems:1}},heading:{type:"string",description:"The heading to be displayed for the collection of classes defined in this group."}},required:["classes"],additionalProperties:!1},"uniqueValueInfo_schema.json":{title:"uniqueValueInfo",type:"object",description:"Info item for the Unique Value Renderer. Its symbol gets selected if the feature's field matches its value.",properties:{description:{type:"string",description:"String value used to describe the drawn symbol."},label:{type:"string",description:"String value used to label the drawn symbol."},symbol:{description:"An object used to display the value.",$ref:"#/definitions/symbol3D_schema.json"},value:{type:"string",description:"String value indicating the unique value."}},additionalProperties:!1},"mediaInfo_chart_value_schema.json":{title:"mediaInfo chart value",type:"object",description:"The value object contains information for popup windows about how charts should be constructed.",properties:{colors:{type:"array",description:"Used with charts. An optional array of colors where each `color` sequentially corresponds to a field in the `fields` property. When the value for `mediaInfo.type` is `linechart`, the first color in the array will drive the line color. If `colors` is longer than `fields`, unmatched colors are ignored. If `colors` is shorter than `fields` or `colors` isn't specified, a default color ramp is applied",items:{$ref:"#/definitions/color_schema.json"}},fields:{type:"array",description:"An array of strings, with each string containing the name of a field to display in the chart.",items:{type:"string"},uniqueItems:!0},normalizeField:{type:"string",description:"An optional string containing the name of a field. The values of all fields in the chart will be normalized (divided) by the value of this field."},tooltipField:{type:"string",description:"String value indicating the tooltip for a chart specified from another field. This field is needed when related records are not used. It is used for showing tooltips from another field in the same layer or related layer/table."}},required:["fields"],additionalProperties:!1},"mediaInfo_image_value_schema.json":{title:"mediaInfo image value",type:"object",description:"The value object contains information for popup windows about how images should be retrieved.",properties:{linkURL:{type:"string",description:"A string containing a URL to be launched in a browser when a user clicks the image."},sourceURL:{type:"string",description:"A string containing the URL to the image."}},required:["sourceURL"],additionalProperties:!1},"popupElementExpressionInfo_schema.json":{title:"popupElementExpressionInfo",description:"An Arcade expression that defines the pop-up element content. The return type will always be a `dictionary` that defines the desired pop-up element as outlined [here](https://developers.arcgis.com/arcade/guide/profiles/#popup-element).",properties:{expression:{type:"string",description:"The Arcade expression."},returnType:{type:"string",description:"Return type of the Arcade expression, the expected return is always `dictionary.`",enum:["dictionary"]},title:{type:"string",description:"Title of the expression."}},additionalProperties:!1},"search_field_schema.json":{title:"search field",type:"object",description:"Contains information about an attribute field in layer and table search configurations.",properties:{exactMatch:{type:"boolean",description:"A Boolean defining whether or not the field is an exact match."},name:{type:"string",description:"A string defining the field name."},type:{type:"string",description:"A string defining the field type.",enum:["esriFieldTypeBigInteger","esriFieldTypeBlob","esriFieldTypeDate","esriFieldTypeDateOnly","esriFieldTypeDouble","esriFieldTypeGeometry","esriFieldTypeGlobalID","esriFieldTypeGUID","esriFieldTypeInteger","esriFieldTypeLong","esriFieldTypeOID","esriFieldTypeRaster","esriFieldTypeSingle","esriFieldTypeSmallInteger","esriFieldTypeString","esriFieldTypeTimeOnly","esriFieldTypeTimestampOffset","esriFieldTypeXML"]}},additionalProperties:!1},"rasterFunctionTemplateArguments_schema.json":{title:"Raster Function Template Arguments",type:"object",description:"Arguments used by the raster function template. The arguments can consist of raster function template, raster function template by reference or raster function variable. The raster variable is a placeholder for a raster dataset that the end user can specify.",oneOf:[{type:"object",description:"Raster function template or raster function variable used as argument.",properties:{_object_id:{type:"integer",description:"The id of the raster function template argument."},type:{type:"string",description:"Type of the raster function argument."}},required:["type"],additionalProperties:{type:"object",properties:{_object_id:{type:"integer"},isDataset:{type:"boolean"},isPublic:{type:"boolean"},name:{type:"string"},type:{type:"string"},description:{type:"string"},value:{oneOf:[{type:"string",description:"A string value for a function argument. For example, (B4 -B3) / (B4-B3) for a BandIndexes argument."},{type:"number",description:"A numeric value for a function argument. For example, 0 for a SlopeType argument."},{type:"array",description:"An array containing values for a function argument. For example, [1, 100] for a OutputValues argument."},{type:"boolean",description:"A boolean value for a function argument. For example, true for a IsScientificOutput argument."},{type:"object",description:"An object containing values used for a raster function argument.",additionalProperties:{properties:{type:{type:"string"}},required:["type"]}}],additionalProperties:{properties:{elements:{type:"array"}}}},function:{description:"The raster function referenced by the raster function template.",$ref:"#/definitions/rasterFunctionInfo_schema.json"},functionType:{type:"integer"},thumbnail:{type:"string"},thumbnailEx:{type:"string"},arguments:{$ref:"#/definitions/rasterFunctionTemplateArguments_schema.json"},properties:{$ref:"#/definitions/rasterFunctionTemplateArguments_schema.json"}},required:["name","type"],additionalProperties:!1}},{type:"object",description:"Raster function template by reference",properties:{_object_ref_id:{type:"integer",description:"The id of the raster function template by reference."}},required:["_object_ref_id"],additionalProperties:!1}]},"rasterFunctionInfo_schema.json":{title:"Raster Function Information",type:"object",description:"Information about the `function` referenced in a raster function template.",properties:{_object_id:{type:"integer",description:"Optional. The id of the raster function info."},description:{type:"string",description:"Description of the raster function."},name:{type:"string",description:"Name of the raster function used by the raster function template."},pixelType:{type:"string",description:"Pixel type of the output image processed by the raster function template.",enum:["C128","C64","F32","F64","S16","S32","S8","U1","U16","U2","U32","U4","U8","UNKNOWN"]},type:{type:"string",description:"Type of the raster function used by the raster function template."}},required:["name","pixelType","type"],additionalProperties:!1},"voxelSlice_schema.json":{title:"Voxel Slice",type:"object",description:"A slice is a plane through the voxel layer. Slices visually cut the voxel layer by removing portion of it. A slice or a combination of slices can define an area of interest showing only a portion of the original extent or show the entire of a voxel layer.",properties:{enabled:{type:"boolean",description:"Boolean defining slice visibility.",default:!0},label:{type:"string",description:"Label for the slice."},normal:{type:"array",description:"Normal vector to the plane in voxel space. Defining the orientation of the slice.",items:{type:"number"},minItems:3,maxItems:3},point:{type:"array",description:"Point belonging to the section plane in voxel space. Defining the position of the slice.",items:{type:"number"},minItems:3,maxItems:3}},required:["normal","point"],additionalProperties:!1},"voxelShading_schema.json":{title:"Voxel Shading",type:"object",description:"Describes the shading properties of the voxel layer. Voxel layers use a simple shading model which provides specular highlights and diffuse shading proportional to the opacity of the voxel being accumulated during raycasting. Opaque voxels are shaded like a surface while semi-transparent voxels mostly contribute their color.",properties:{diffuseFactor:{type:"number",description:"Diffuse light coefficient.",minimum:0,maximum:1,default:.5},specularFactor:{type:"number",description:"Specular highlight coefficient.",minimum:0,maximum:1,default:.5}},additionalProperties:!1},"voxelVariableStyle_schema.json":{title:"Voxel Variable Style",type:"object",description:"The voxel variable style defines how the voxel layer will render for a variable. A variable can be discrete (integer values) or continuous (float values).",properties:{isosurfaces:{type:"array",description:"Array of styles for isosurfaces. (Only if `variable.originalFormat.continuity = continuous`). An isosurface represents a surface at a specific value. A voxel layer can have up to four isosurfaces.",items:{$ref:"#/definitions/voxelIsosurfaceStyle_schema.json"},minItems:1},label:{type:"string",description:"Label for the variable."},transferFunction:{description:"Defines the stretch rendering of the voxel layer. The transfer function maps voxel values to color and transparency. Scalar field only. (Only if `variable.originalFormat.continuity = continuous`).",$ref:"#/definitions/voxelTransferFunctionStyle_schema.json"},uniqueValues:{type:"array",description:"Defines the unique value rendering as an array of unique value styles (Only if `variable.originalFormat.continuity = discrete`).",items:{$ref:"#/definitions/voxelUniqueValue_schema.json"},minItems:0},variableId:{type:"integer",description:"Id of the variable. The variable styles will be defined for the variable with the given Id. If the `variableId` and the voxel style `currentVariableId` are identical the style will be shown. "}},required:["variableId"],additionalProperties:!1},"voxelVolumeStyle_schema.json":{title:"Volume Style",type:"object",description:"The volume style allows you to define the exaggeration and offset.",properties:{dynamicSections:{type:"array",description:"Array of dynamic sections of the volume. (Only if `style.renderMode = surfaces`). Dynamic sections are planes through the voxel layer used for visual analysis to inspect the voxel layer at specified positions. Sections are visible if the voxel layer shows surfaces. For example, you can define a cross section diagram using dynamic sections.",items:{$ref:"#/definitions/voxelDynamicSection_schema.json"}},exaggerationMode:{type:"string",description:"Determines how the vertical exaggeration is applied. `scale-height` scales from the voxel dataset origin only, for example, if a voxel layer has its minimum at sea level the layer will be exaggerated starting from sea level. `scale-height` is the default. `scale-position` also scales the space between voxel dataset origin in the coordinate system origin. This exaggeration mode is identical with exaggeration applied to other layers like feature layers (use the scale position option if you want to draw the voxel layer together with feature based data).",enum:["scale-height","scale-position"],default:"scale-height"},slices:{type:"array",description:"Array of slices applied to this volume. Slices reduce the voxel volume to an area of interest. Slices are applied to both voxel style renderModes (volume, surfaces).",items:{$ref:"#/definitions/voxelSlice_schema.json"},minItems:1},verticalExaggeration:{type:"number",description:"Vertical exaggeration factor.",default:1},verticalOffset:{type:"number",description:"Vertical offset value in vertical unit of the spatial reference of the voxel layer.",default:0},volumeId:{type:"integer",description:"Id of the volume in the volume array (a maximum of one style per volumeId).",default:0}},additionalProperties:!1},"edges_schema.json":{title:"Edges",type:"object",description:"Specifies an edge visualization style (only applies to MeshSymbol3D). Edges describe the style applied to visually important edges of 3D objects.",oneOf:[{$ref:"#/definitions/sketchEdges_schema.json"},{$ref:"#/definitions/solidEdges_schema.json"}]},"codedValue_schema.json":{title:"codedValue",type:"object",description:"A set of valid coded values with unique names.",properties:{code:{type:["number","string"],description:"The value stored in the feature attribute."},name:{type:"string",description:"User-friendly name for what the code means."}},required:["code","name"],additionalProperties:!1},"joinTableDataSource_schema.json":{title:"Join Table DataSource",type:"object",description:"Join Table data source is the result of a join operation. Nested joins are supported. To use nested joins, set either leftTableSource or rightTableSource to be a joinTable.",properties:{joinType:{type:"string",description:"The type of join (left outer or left inner).",enum:["esriLeftInnerJoin","esriLeftOuterJoin"]},leftTableKey:{type:"string",description:"Field name from the left table."},leftTableSource:{description:"The left source. If the leftTableSource is a table, the resulting joinTable is a table. If the leftTableSource is a layer, the resulting joinTable is a layer.",$ref:"#/definitions/source_schema.json"},rightTableKey:{type:"string",description:"Field name from the right table."},rightTableSource:{description:"The right table source.",$ref:"#/definitions/source_schema.json"},type:{type:"string",description:"String value indicating the type for the dataSource.",enum:["joinTable"]}},required:["type"],additionalProperties:!1},"queryTableDataSource_schema.json":{title:"Query Table DataSource",type:"object",description:"Query table data source is a layer/table that is defined by a SQL query.",properties:{geometryType:{type:"string",description:"The geometry type. When querying a table that does not have a geometry column, do not include geometryType.",enum:["esriGeometryMultipoint","esriGeometryPoint","esriGeometryPolygon","esriGeometryPolyline"]},oidFields:{type:"string",description:"Comma separated list of identifier fields. There are only certain field types that can be used as a unique identifier. These field types include integer, string, GUID, and date. If a single integer field is specified, map server uses the values in that field directly to uniquely identify all features and rows returned from a queryTable. However, if a single string field or a group of fields is used as the unique identifier, map server maps those unique values to an integer."},query:{type:"string",description:"The SQL query."},spatialReference:{description:"The spatial reference of the geometry column. When querying a table that does not have a geometry column, do not include spatialReference.",$ref:"#/definitions/spatialReference_schema.json"},type:{type:"string",description:"String value indicating the type for the dataSource.",enum:["queryTable"]},workspaceId:{type:"string",description:"The unique string value used to identify the datasource's workspace."}},required:["type"],additionalProperties:!1},"rasterDataSource_schema.json":{title:"Raster DataSource",type:"object",description:"Raster data source is a file-based raster that resides in a registered raster workspace.",properties:{dataSourceName:{type:"string",description:"The raster datasource's name."},type:{type:"string",description:"String value indicating the type for the dataSource.",enum:["raster"]},workspaceId:{type:"string",description:"The unique string value used to identify the datasource's workspace."}},required:["type"],additionalProperties:!1},"tableDataSource_schema.json":{title:"Table DataSource",type:"object",description:"Table data source is a table, feature class, or raster that resides in a registered workspace (either a folder or geodatabase). In the case of a geodatabase, if versioned, use version to switch to an alternate geodatabase version. If version is empty or missing, the registered geodatabase version will be used.",properties:{dataSourceName:{type:"string",description:"The fully-qualified string value used to specify where the dataSource is derived."},gdbVersion:{type:"string",description:"If applicable, the value indicating the version of the geodatabase."},type:{type:"string",description:"String value indicating the type for the dataSource. The value for a Table DataSource is `table`.",enum:["table"]},workspaceId:{type:"string",description:"The unique string value used to identify the datasource's workspace."}},required:["type"],additionalProperties:!1},"border_schema.json":{title:"border",type:"object",description:"Optional border on the line that is used to improve the contrast of the line color against various background colors.",properties:{color:{$ref:"#/definitions/color_schema.json"},transparency:{type:"integer",description:"A value between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100}},required:["color"],additionalProperties:!1},"textBackground_schema.json":{title:"text background",type:"object",description:"Text background definition.",properties:{color:{$ref:"#/definitions/color_schema.json"},transparency:{type:"integer",description:"A value between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100}},additionalProperties:!1},"font_schema.json":{title:"font",type:"object",description:"Font used for text symbols.",properties:{decoration:{type:"string",description:"The text decoration.",enum:["line-through","none","underline"],default:"none"},family:{type:"string",description:"The font family."},size:{type:"number",description:"The font size in points. Ignored when font is used on TextSymbol3DLayer."},style:{type:"string",description:"The text style.",enum:["italic","normal","oblique"],default:"normal"},weight:{type:"string",description:"The text weight.",enum:["bold","bolder","lighter","normal"],default:"normal"}},additionalProperties:!1},"halo_schema.json":{title:"halo",type:"object",description:"Halo definition.",properties:{color:{$ref:"#/definitions/color_schema.json"},size:{type:"number",description:"Width of the halo in points."},transparency:{type:"integer",description:"A value between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100}},additionalProperties:!1},"material_schema.json":{title:"Material",type:"object",description:"The material used to shade the geometry.",properties:{color:{$ref:"#/definitions/color_schema.json"},transparency:{type:"integer",description:"A value between `100` (full transparency) and `0` (full opacity). Ignored if no color is specified.",minimum:0,maximum:100}},additionalProperties:!1},"colorRamp_schema.json":{title:"Color Ramp",description:"A colorRamp object is used to specify a range of colors that are applied to a group of symbols.",oneOf:[{$ref:"#/definitions/colorRamp_algorithmic_schema.json"},{$ref:"#/definitions/colorRamp_multipart_schema.json"}]},"field_authoringInfo_schema.json":{title:"AuthoringInfo Field",type:"object",description:"Contains information about an attribute field relating to Relationship renderers.",properties:{classBreakInfos:{type:"array",items:{$ref:"#/definitions/classBreakInfo_authoringInfo_schema.json"}},field:{type:"string",description:"Attribute field used for renderer."},label:{type:"string",description:"The label used to describe the field or attribute in the legend."},normalizationField:{type:"string",description:"Attribute field used to normalize the data."}},additionalProperties:!1},"authoringInfo_visualVariable_schema.json":{title:"AuthoringInfo Visual Variable",type:"object",description:"This visual variable pertains specifically to [authoringInfo](authoringInfo.md) and is different from visual variables directly on the [renderer](renderer.md).",properties:{endTime:{type:["number","string"],description:"A Unix stamp. Both `startTime` or `endTime` can be fields. If this is the case, their names must be different."},field:{type:"string",description:"The attribute field the user chose in the Smart Mapping gallery. Must be the same as in either `startTime` or `endTime`."},maxSliderValue:{type:"number",description:"A numeric value indicating the maximum value displayed."},minSliderValue:{type:"number",description:"A numeric value indicating the minimum value displayed."},startTime:{type:["number","string"],description:"A Unix time stamp. Both `startTime` or `endTime` can be fields. If this is the case, their names must be different."},style:{type:"string",description:"(This property is used for comparison rendering). It is used to map the ratio between two numbers. It is possible to express that relationship as percentages, simple ratios, or an overall percentage.",enum:["percent","percentTotal","ratio"]},theme:{type:"string",description:"Theme to be used only when working with visual variables of type `colorInfo`. Default is `high-to-low`.",enum:["above-and-below","centered-on","extremes","high-to-low"]},type:{type:"string",description:"A string value specifying the type of renderer's visual variable.",enum:["colorInfo","rotationInfo","sizeInfo","transparencyInfo"]},units:{type:"string",description:"(This property is used only with age renderers.) Units for startTime and endTime.",enum:["days","hours","minutes","months","seconds","years"]}},additionalProperties:!1},"styleOrigin_schema.json":{title:"styleOrigin",type:"object",description:"The origin of the style from which the symbol was originally referenced. A reference to the style origin can be either by styleName or by styleUrl (but not both). It may be used to understand where a symbol was originally sourced from, but does not affect actual appearance or rendering of the symbol.",properties:{name:{type:"string",description:"Identifies a symbol in the style by name."},styleName:{type:"string",description:"A registered web style name, such as `EsriThematicShapesStyle`"},styleUrl:{description:"URL to a style definition.",oneOf:[{type:"string",description:"An absolute URL"},{type:"string",description:"A relative path starting with ./",pattern:"^\\./.+$"}]}},oneOf:[{required:["name","styleName"]},{required:["name","styleUrl"]}],additionalProperties:!1},"extrudeSymbol3DLayer_schema.json":{title:"ExtrudeSymbol3DLayer",type:"object",description:"ExtrudeSymbol3DLayer is used to render Polygon geometries by extruding them upward from the ground, creating a 3D volumetric object.",properties:{castShadows:{type:"boolean",description:"Boolean to control the shadow casting behaviour of the rendered geometries.",default:!0},edges:{description:"Specifies an edge visualization style.",$ref:"#/definitions/edges_schema.json"},enable:{type:"boolean"},material:{$ref:"#/definitions/material_schema.json"},size:{type:"number",description:"Extrusion height in meters."},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Extrude"]}},required:["size","type"],additionalProperties:!1},"fillSymbol3DLayer_schema.json":{title:"FillSymbol3DLayer",type:"object",description:"FillSymbol3DLayer is used to render the surfaces of flat 2D Polygon geometries and 3D volumetric meshes in a SceneView.",properties:{castShadows:{type:"boolean",description:"Boolean to control the shadow casting behaviour of the rendered geometries (only applies to MeshSymbol3D).",default:!0},edges:{$ref:"#/definitions/edges_schema.json"},enable:{type:"boolean"},material:{$ref:"#/definitions/materialColorMixMode_schema.json"},outline:{description:"The outline of the symbol layer (only applies to PolygonSymbol3D).",$ref:"#/definitions/outline_schema.json"},pattern:{$ref:"#/definitions/polygonPattern_schema.json"},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Fill"]}},required:["type"],additionalProperties:!1},"iconSymbol3DLayer_schema.json":{title:"IconSymbol3DLayer",type:"object",description:"IconSymbol3DLayer is used to render Point geometries using a flat 2D icon (e.g. a circle) with a PointSymbol3D in a SceneView.",properties:{anchor:{type:"string",enum:["bottom","bottomLeft","bottomRight","center","left","relative","right","top","topLeft","topRight"],default:"center"},anchorPosition:{type:"array",description:"When `anchor` equals `relative`, this property specifies the position within the icon that should coincide with the feature geometry. Otherwise it is ignored. The position is defined as a factor of the icon dimensions that is added to the icon center: `positionInIcon = (0.5 + anchorPosition) * size`, where `size` is the original size of the icon resource.",items:{type:"number"},minItems:2,maxItems:2},angle:{type:"number",description:"Rotation angle in degrees. The rotation is defined in screen space, with a rotation of 0 degrees (default value) pointing in the direction of the Y-axis. Positive values indicate clockwise rotation.",default:0},enable:{type:"boolean"},material:{$ref:"#/definitions/material_schema.json"},outline:{type:"object",description:"Sets properties of the outline of the IconSymbol3DLayer.",allOf:[{$ref:"#/definitions/outline_schema.json"},{properties:{color:{},size:{},transparency:{}},additionalProperties:!1}]},resource:{$ref:"#/definitions/iconSymbol3DLayer_resource_schema.json"},size:{type:"number",description:"Icon size in points, positive only",minimum:0},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Icon"]}},required:["size","type"],additionalProperties:!1},"lineSymbol3DLayer_schema.json":{title:"LineSymbol3DLayer",type:"object",description:"LineSymbol3DLayer renders Polyline geometries using a flat 2D line with a LineSymbol3D in a 3D SceneView.",properties:{cap:{$ref:"#/definitions/lineCap_schema.json"},enable:{type:"boolean"},join:{type:"string",description:"Shape of the intersection of two line segments.",enum:["bevel","miter","round"],default:"miter"},marker:{$ref:"#/definitions/lineMarker_schema.json"},material:{$ref:"#/definitions/material_schema.json"},pattern:{$ref:"#/definitions/linePattern_schema.json"},size:{type:"number",description:"Line width in points, positive only",minimum:0},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Line"]}},required:["size","type"],additionalProperties:!1},"objectSymbol3DLayer_schema.json":{title:"ObjectSymbol3DLayer",type:"object",description:"ObjectSymbol3DLayer is used to render Point geometries using a volumetric 3D shape (e.g., a sphere or cylinder) with a Symbol3D in a SceneView.",properties:{anchor:{type:"string",description:"The positioning of the object relative to the geometry.",enum:["bottom","center","origin","relative","top"],default:"origin"},anchorPosition:{type:"array",description:"When `anchor` equals `relative`, this property specifies the positioning of the object relative to the geometry as a fraction of the symbol layer's bounding box. Otherwise it is ignored.",items:{type:"number"},minItems:3,maxItems:3},castShadows:{type:"boolean",description:"Boolean to control the shadow casting behaviour of the rendered geometries.",default:!0},depth:{type:"number",description:"Object depth in meters, positive only",minimum:0},enable:{type:"boolean"},heading:{type:"number",description:"Rotation angle around Z axis in degrees. At 0 degrees, the model points in the direction of the Y-axis. Positive values indicate clockwise rotation (when looked at from the top). [Detailed description](static/objectSymbolLayerOrientation.md)."},height:{type:"number",description:"Object height in meters, positive only",minimum:0},material:{$ref:"#/definitions/material_schema.json"},resource:{$ref:"#/definitions/objectSymbol3DLayer_resource_schema.json"},roll:{type:"number",description:"Rotation angle around Y axis in degrees. At 0 degrees, the model is level. A positive value lifts the left part and lowers the right part of the model. [Detailed description](static/objectSymbolLayerOrientation.md)."},tilt:{type:"number",description:"Rotation angle around X axis in degrees. At 0 degrees, the model is level. A positive value lifts the front and lowers the back of the model. [Detailed description](static/objectSymbolLayerOrientation.md)."},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Object"]},width:{type:"number",description:"Object width in meters, positive only",minimum:0}},required:["type"],additionalProperties:!1},"waterSymbol3DLayer_schema.json":{title:"WaterSymbol3DLayer",type:"object",description:"Symbol Layer that describes a water appearance on surfaces in a SceneView.",properties:{color:{description:"The dominant water color.",$ref:"#/definitions/color_schema.json",default:[0,119,190]},enable:{type:"boolean"},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Water"]},waterbodySize:{type:"string",description:"Size of the waterbody the symbol layer represents. Applications will display waves that are appropriate for the chosen body of water, for example ocean versus marina versus swimming pool.",enum:["large","medium","small"],default:"medium"},waveDirection:{type:"number",description:"Azimuthal bearing for direction of the waves. If ommitted, waves appear directionless. The value is interpreted as 'geographic' rotation, i.e. clockwise starting from north.",minimum:0,maximum:360},waveStrength:{type:"string",description:"The magnitude of the waves displayed on the waterbody. Strings roughly follow the [Douglas sea scale](https://en.wikipedia.org/wiki/Douglas_sea_scale), currently limited to lower degrees.",enum:["calm","moderate","rippled","slight"],default:"moderate"}},required:["type"],additionalProperties:!1},"cimSymbolReference_schema.json":{title:"CIMSymbolReference",type:"object",description:"Represents a symbol reference that contains a CIM symbol. In addition to `type` listed below, a symbol reference will contain additional properties. More details on CIM symbol reference and CIM symbols are available on github: [Symbols Overview](https://github.com/Esri/cim-spec/blob/main/docs/v2/Overview-Symbols.md).",properties:{type:{type:"string",description:"Specifies the type of symbol used.",enum:["CIMSymbolReference"]}},required:["type"],additionalProperties:!0},"lineSymbol3D_schema.json":{title:"LineSymbol3D",type:"object",description:"LineSymbol3D is used to render features with Polyline geometry in a 3D SceneView.",properties:{styleOrigin:{$ref:"#/definitions/styleOrigin_schema.json"},symbolLayers:{type:"array",description:"A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",items:{oneOf:[{$ref:"#/definitions/lineSymbol3DLayer_schema.json"},{$ref:"#/definitions/pathSymbol3DLayer_schema.json"}]}},type:{type:"string",description:"Specifies the type of symbol used.",enum:["LineSymbol3D"]}},required:["symbolLayers","type"],additionalProperties:!1},"meshSymbol3D_schema.json":{title:"MeshSymbol3D",type:"object",description:"MeshSymbol3D is used to render 3D mesh features in a SceneLayer in a 3D SceneView.",properties:{styleOrigin:{$ref:"#/definitions/styleOrigin_schema.json"},symbolLayers:{type:"array",description:"A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",items:{oneOf:[{$ref:"#/definitions/fillSymbol3DLayer_schema.json"}]}},type:{type:"string",description:"Specifies the type of symbol used",enum:["MeshSymbol3D"]}},required:["symbolLayers","type"],additionalProperties:!1},"pointSymbol3D_schema.json":{title:"PointSymbol3D",type:"object",description:"PointSymbol3D is used to render features with Point geometry in a 3D SceneView.",properties:{callout:{$ref:"#/definitions/callout_schema.json"},styleOrigin:{$ref:"#/definitions/styleOrigin_schema.json"},symbolLayers:{type:"array",description:"A Collection of Symbol3DLayer objects used to visualize the graphic or feature.",items:{oneOf:[{$ref:"#/definitions/iconSymbol3DLayer_schema.json"},{$ref:"#/definitions/objectSymbol3DLayer_schema.json"},{$ref:"#/definitions/textSymbol3DLayer_schema.json"}]}},type:{type:"string",description:"Specifies the type of symbol used",enum:["PointSymbol3D"]},verticalOffset:{$ref:"#/definitions/verticalOffset_schema.json"}},required:["symbolLayers","type"],additionalProperties:!1},"styleSymbolReference_schema.json":{title:"StyleSymbolReference",type:"object",description:"The StyleSymbolReference is used to reference a symbol from a portal styleItem",properties:{name:{type:"string",description:"Identifies a symbol in the style by name."},styleName:{type:"string",description:"A registered web style name, such as `EsriThematicTreesStyle`"},styleUrl:{type:"string",description:"URL to a style definition."},type:{type:"string",description:"The type of the symbol",enum:["styleSymbolReference"]}},oneOf:[{required:["name","styleUrl","type"]},{required:["name","styleName","type"]}],additionalProperties:!1},"colorInfo_visualVariable_schema.json":{title:"ColorInfo Visual Variable",type:"object",description:"The colorInfo visual variable defines how a continuous color ramp is applied to features based on the values of a numeric field attribute.",properties:{field:{type:"string",description:"Attribute field used for color rendering if no valueExpression is provided."},legendOptions:{type:"object",allOf:[{$ref:"#/definitions/visualVariableLegendOptions_schema.json"},{properties:{showLegend:{},title:{}},additionalProperties:!1}]},normalizationField:{type:"string",description:"Attribute field used to normalize the data."},stops:{type:"array",description:"An array of stop objects.",items:{$ref:"#/definitions/colorStop_schema.json"}},type:{type:"string",description:"Specifies the type of visual variable.",enum:["colorInfo"]},valueExpression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) that computes a value in lieu of a value provided by an attribute `field`."},valueExpressionTitle:{type:"string",description:"The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property."}},required:["type"],additionalProperties:!1},"rotationInfo_visualVariable_schema.json":{title:"RotationInfo Visual Variable",type:"object",description:"The rotation visual variable defines how features rendered with marker symbols are rotated. The rotation value is determined by a value in a field or an Arcade expression calculating a value. Use either the `field` property or `valueExpression` when specifying rotation values.",properties:{axis:{type:"string",description:"Defines the rotation axis the visual variable should be applied to when rendering features with an ObjectSymbol3DLayer. [Detailed description](static/objectSymbolLayerOrientation.md).",enum:["heading","roll","tilt"],default:"heading"},field:{type:"string",description:"Attribute field used for setting the rotation of a symbol if no `valueExpression` is provided."},legendOptions:{type:"object",allOf:[{$ref:"#/definitions/visualVariableLegendOptions_schema.json"},{properties:{showLegend:{},title:{}},additionalProperties:!1}]},rotationType:{type:"string",description:"Defines the origin and direction of rotation depending on how the angle of rotation was measured. Possible values are `geographic` which rotates the symbol from the north in a clockwise direction and `arithmetic` which rotates the symbol from the east in a counter-clockwise direction.",enum:["arithmetic","geographic"],default:"geographic"},type:{type:"string",description:"A string value indicating the type of visual variable used for the renderer.",enum:["rotationInfo"]},valueExpression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number."},valueExpressionTitle:{type:"string",description:"The title identifying and describing the associated [Arcade expression] (https://developers.arcgis.com/arcade/) as defined in the `valueExpression` property."}},required:["type"],additionalProperties:!1},"sizeInfo_visualVariable_schema.json":{title:"SizeInfo Visual Variable",type:"object",description:"The sizeInfo visual variable defines how size is applied to features based on the values of a numeric field attribute. The minimum and maximum values of the data should be indicated along with their respective size values. You must specify either `minSize` and `maxSize`, or `stops`, or `valueUnit` to construct the size ramp. All features with values falling in between the specified min and max data values (or stops) will be scaled proportionally between the provided min and max sizes.",properties:{axis:{type:"string",description:"Defines the axis the size visual variable should be applied to when rendering features with an ObjectSymbol3DLayer.",enum:["all","depth","height","width","widthAndDepth"],default:"all"},field:{type:"string",description:"Attribute field used for size rendering if no valueExpression is provided."},legendOptions:{type:"object",allOf:[{$ref:"#/definitions/visualVariableLegendOptions_schema.json"},{properties:{customValues:{},showLegend:{},title:{}},additionalProperties:!1}]},maxDataValue:{type:"number",description:"The maximum data value."},maxSize:{type:"number",description:"Specifies the maximum size to be applied to the symbol. This is required if valueUnit is set to `unknown`."},minDataValue:{type:"number",description:"The minimum data value."},minSize:{type:"number",description:"Specifies the minimum size to be applied to the symbol. This is required if valueUnit is set to `unknown`."},normalizationField:{type:"string",description:"Attribute field used to normalize the data."},stops:{type:"array",description:"An array of objects that defines the thematic size ramp in a sequence of data or expression stops. At least two stops are required. The stops must be listed in ascending order based on the value of the `value` property in each stop. This property is required if `minDataValue`, `maxDataValue`, `minSize`, and `maxSize` are not defined.",items:{$ref:"#/definitions/sizeStop_schema.json"}},target:{type:"string",description:"Only used when sizeInfo is used for polygon outlines.",enum:["outline"]},type:{type:"string",description:"Specifies the type of visual variable.",enum:["sizeInfo"]},useSymbolValue:{type:"boolean",description:"When setting a size visual variable on a renderer using an ObjectSymbol3DLayer, this property indicates whether to apply the value defined by the height, width, or depth properties to the corresponding axis of this visual variable instead of proportionally scaling this axis' value after other axes."},valueExpression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number."},valueExpressionTitle:{type:"string",description:"The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property."},valueRepresentation:{type:"string",description:"Specifies how to apply the data value when mapping real-world sizes. See table below for supported values.",enum:["area","diameter","distance","radius","width"]},valueUnit:{type:"string",description:"A string value indicating the unit of measurement. Defaults to `meters` if not set.",enum:["centimeters","decimeters","feet","inches","kilometers","meters","miles","millimeters","nautical-miles","unknown","yards"],default:"meters"}},required:["type"],additionalProperties:!1},"transparencyInfo_visualVariable_schema.json":{title:"TransparencyInfo Visual Variable",type:"object",description:"The transparencyInfo visual variable defines the transparency, or opacity, of each feature's symbol based on a numeric attribute field value.",properties:{field:{type:"string",description:"Attribute field used for setting the transparency of a feature if no `valueExpression` is provided."},legendOptions:{type:"object",allOf:[{$ref:"#/definitions/visualVariableLegendOptions_schema.json"},{properties:{showLegend:{},title:{}},additionalProperties:!1}]},normalizationField:{type:"string",description:"Attribute field used to normalize the data."},stops:{type:"array",description:"An array of transparencyStop objects.",items:{$ref:"#/definitions/transparencyStop_schema.json"}},type:{type:"string",description:"Specifies the type of visual variable.",enum:["transparencyInfo"]},valueExpression:{type:"string",description:"An [Arcade expression](https://developers.arcgis.com/arcade/) evaluating to a number."},valueExpressionTitle:{type:"string",description:"The title identifying and describing the associated [Arcade](https://developers.arcgis.com/arcade/) expression as defined in the `valueExpression` property."}},required:["type"],additionalProperties:!1},"colorClassBreakInfo_schema.json":{title:"colorClassBreakInfo",type:"object",description:"The classBreaksInfo object provides information about the class breaks associated with the renderer.",properties:{classMaxValue:{type:"number",description:"A numeric value used to specify the maximum value for a break."},classMinValue:{type:"number",description:"A numeric value used to specify discontinuous class breaks. If this value is null or is missing, the map server will calculate the minimum value based on the preceding class' maximum value."},color:{$ref:"#/definitions/color_schema.json"},description:{type:"string",description:"String value used to describe the class."},label:{type:"string",description:"String value used to label the class."}},required:["classMaxValue","classMinValue","color"],additionalProperties:!1},"colorModulationInfo_schema.json":{title:"colorModulationInfo",type:"object",description:"Indicates whether modulation should be used to render the point.",properties:{field:{type:"string",description:"the attribute to use as a source for the modulation amplitude"},maxValue:{type:"number",description:"maximum value to compute modulation linear mapping"},minValue:{type:"number",description:"minimum value to compute modulation linear mapping"}},required:["field","maxValue","minValue"],additionalProperties:!1},"pointSizeAlgorithm_schema.json":{title:"PointCloud Size Algorithm",type:"object",description:"Size algorithms for point cloud performance.",oneOf:[{$ref:"#/definitions/pointCloudFixedSizeAlgorithm_schema.json"},{$ref:"#/definitions/pointCloudSplatAlgorithm_schema.json"}]},"colorStop_schema.json":{title:"colorStop",description:"A colorStop object describes the renderer's color ramp with more specificity than just colors.",properties:{color:{description:"A CSS color string or an array of rbga values. The color to place at the stop indicated by either a ratio or value.",$ref:"#/definitions/color_schema.json"},label:{type:"string",description:"Value if a label is needed on the legend for a stop."},value:{type:"number",description:"The pixel intensity value. Describes the pixel intensity value that the color should be associated with. Just like in colorInfo, using value will ignore `maxPixelIntensity` and `minPixelIntensity` properties. It will actually set those properties to maximum and minimum values you set in the colorStops array. The hard values are converted to ratios to create the color gradient that is used in the heatmap calculations. Setting `minPixelIntensity` or `maxPixelIntensity`, after setting colorStops with values, removes the hard link between the color ramp and the pixel intensity values that were used to create it."}},required:["color"]},"colorUniqueValueInfo_schema.json":{title:"colorUniqueValueInfo",type:"object",description:"The colorUniqueValueInfo object matches a unique value with a specific color.",properties:{color:{$ref:"#/definitions/color_schema.json"},description:{type:"string",description:"String value used to describe the class."},label:{type:"string",description:"String value used to label the class."},values:{type:"array",description:"List of string values used to classify points.",items:{oneOf:[{type:"string"}]}}},required:["color","values"],additionalProperties:!1},"colormapInfo_schema.json":{title:"ColormapInfo",type:"object",description:"The colormap information used when rendering raster data.",properties:{color:{description:"The pixel color.",$ref:"#/definitions/color_schema.json"},label:{type:"string",description:"The label used for the pixel value."},value:{type:"number",description:"The raster pixel value."}},required:["color","value"],additionalProperties:!1},"uniqueValueClass_schema.json":{title:"Unique Value Class",type:"object",description:"The following is a list of properties found on the uniqueValueClass object. The uniqueValueClass object contains the symbology for grouped unique values in the renderer.",properties:{description:{type:"string",description:"String value used to describe the drawn symbol."},label:{type:"string",description:"String value used to label the drawn symbol."},symbol:{description:"An object used to display the value.",$ref:"#/definitions/symbol3D_schema.json"},values:{type:"array",description:"A list of unique values that should be rendered with the same symbol. Each item in the list represents a set of value combinations represented by the given symbol. The inner array must contain only one value if only field1 is specified, two values if field1 and field2 are specified, or three values if field1, field2, and field3 are specified. The inner arrays must not contain more than three values.",items:{type:"array",items:{type:"string"}}}},required:["symbol","values"],additionalProperties:!1},"voxelIsosurfaceStyle_schema.json":{title:"Voxel Isosurface Style",type:"object",description:"The isosurface style describes the value and coloring of the isosurface. For example, for a given variable temperature you can define up to four isosurfaces based on a temperature value.",properties:{color:{$ref:"#/definitions/color_schema.json"},colorLocked:{type:"boolean",description:"If false the isosurface color is automatically updated when the variable's colorStops are modified.",default:!1},enabled:{type:"boolean",description:"Determines if the isosurface should be shown or hidden.",default:!0},label:{type:"string",description:"Label for the isosurface."},value:{type:"number",description:"Value of the variable."}},required:["color","value"],additionalProperties:!1},"voxelTransferFunctionStyle_schema.json":{title:"Voxel Transfer Function Style",type:"object",description:"Defines the mapping between voxel values and color and transparency.",properties:{alphaStops:{type:"array",description:"Describes the transparency stops (transparency mapping).",items:{$ref:"#/definitions/voxelAlphaStop_schema.json"},minItems:2},colorStops:{type:"array",description:"Describes the color stops defining the color ramp.",items:{$ref:"#/definitions/voxelColorStop_schema.json"},minItems:2},interpolation:{type:"string",description:"Interpolation mode",enum:["linear","nearest"]},rangeFilter:{description:"Defines the range of voxels displayed. Voxels with values outside of this range will be discarded.",$ref:"#/definitions/voxelRangeFilter_schema.json"},stretchRange:{type:"array",description:"Describes the low and high point for value-to-color mapping.",items:{type:"number"},minItems:2,maxItems:2}},required:["stretchRange"],additionalProperties:!1},"voxelUniqueValue_schema.json":{title:"Voxel Unique Value",type:"object",description:"Describes the unique value of a variable with a discrete data type.",properties:{color:{$ref:"#/definitions/color_schema.json"},enabled:{type:"boolean",description:"Show or hide all voxels equal to this value.",default:!0},label:{type:"string",description:"Label for the unique value."},value:{type:"integer",description:"Unique value of the variable."}},required:["color","value"],additionalProperties:!1},"voxelDynamicSection_schema.json":{title:"Voxel Dynamic Section",type:"object",description:"A section is a plane through the voxel layer. A section can be moved by changing the position and orientation.",properties:{enabled:{type:"boolean",description:"Boolean defining section visibility.",default:!0},label:{type:"string",description:"The label for the dynamic section."},normal:{type:"array",description:"Normal vector to the plane in voxel space. Defining the orientation of the dynamic section.",items:{type:"number"},minItems:3,maxItems:3},point:{type:"array",description:"Point belonging to the section plane in voxel space. Defining the position of the dynamic section.",items:{type:"number"},minItems:3,maxItems:3}},required:["normal","point"],additionalProperties:!1},"sketchEdges_schema.json":{title:"Sketch Edges",type:"object",description:"The sketch edge rendering configuration of a symbol layer. Edges of type `sketch` are rendered with a hand-drawn look in mind.",properties:{color:{$ref:"#/definitions/color_schema.json"},extensionLength:{type:"number",description:"A size in points by which to extend edges beyond their original end points."},size:{type:"number",description:"Edge size in points, positive only",minimum:0},transparency:{type:"integer",description:"The value has to lie between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100},type:{type:"string",description:"The type of edge visualization.",enum:["sketch"]}},required:["color","type"],additionalProperties:!1},"solidEdges_schema.json":{title:"Solid Edges",type:"object",description:"The solid edge rendering configuration of a symbol layer. Edges of type `solid` are rendered in a single color, unaffected by scene lighting.",properties:{color:{$ref:"#/definitions/color_schema.json"},extensionLength:{type:"number",description:"A size in points by which to extend edges beyond their original end points."},size:{type:"number",description:"Edge size in points, positive only",minimum:0},transparency:{type:"integer",description:"The value has to lie between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100},type:{type:"string",description:"The type of edge visualization.",enum:["solid"]}},required:["color","type"],additionalProperties:!1},"colorRamp_algorithmic_schema.json":{title:"Color Ramp Algorithmic",description:"A colorRamp object is used to specify a range of colors that are applied to a group of symbols.",properties:{algorithm:{type:"string",description:"Algorithm used for calculating the ramp.",enum:["esriCIELabAlgorithm","esriHSVAlgorithm","esriLabLChAlgorithm"]},fromColor:{description:"Array representing the initial color to start the ramp from.",$ref:"#/definitions/color_schema.json"},toColor:{description:"Array representing the final color to end the ramp with.",$ref:"#/definitions/color_schema.json"},type:{type:"string",description:"Value indicating the type of colorRamp.",enum:["algorithmic"]}},additionalProperties:!1},"colorRamp_multipart_schema.json":{title:"Color Ramp Multipart",description:"A colorRamp object is used to specify a range of colors that are applied to a group of symbols.",properties:{colorRamps:{type:"array",description:"A multipart color ramp is defined by a list of constituent color ramps.",items:{$ref:"#/definitions/colorRamp_algorithmic_schema.json"}},type:{type:"string",description:"Value indicating the type of colorRamp.",enum:["multipart"]}},additionalProperties:!1},"classBreakInfo_authoringInfo_schema.json":{title:"AuthoringInfo classBreakInfo",type:"object",description:"The classBreaksInfo object provides information about the class breaks associated with the Relationship renderer.",properties:{maxValue:{type:"number",description:"A numeric value used to specify the maximum value for a break."},minValue:{type:"number",description:"A numeric value used to specify the minimum value for a break."}},additionalProperties:!1},"materialColorMixMode_schema.json":{title:"Material",type:"object",description:"The material used to shade the geometry, including colorMixMode options.",properties:{color:{$ref:"#/definitions/color_schema.json"},colorMixMode:{type:"string",description:"Controls how symbolLayer or visualVariable color is applied onto the underlying geometry color/texture. This property only applies to FillSymbol3DLayer within MeshSymbol3D.<br><ul><li>`tint`: the feature's appearance (in terms of color) should be altered to match the symbol / visual variable color.</li><li>`replace`: the feature's color is replaced with the symbol / visual variable color.</li><li>`multiply`: the feature's color is multiplied with the symbol color.</li></ul>",enum:["multiply","replace","tint"]},transparency:{type:"integer",description:"A value between `100` (full transparency) and `0` (full opacity). Ignored if no color is specified.",minimum:0,maximum:100}},additionalProperties:!1},"outline_schema.json":{title:"Outline",type:"object",description:"The outline of the symbol layer.",properties:{color:{$ref:"#/definitions/color_schema.json"},pattern:{$ref:"#/definitions/linePattern_schema.json"},patternCap:{type:"string",$ref:"#/definitions/lineCap_schema.json"},size:{type:"number",description:"Outline size in points, positive only",minimum:0},transparency:{type:"integer",description:"The value has to lie between `100` (full transparency) and `0` (full opacity).",minimum:0,maximum:100}},required:["color","size"],additionalProperties:!1},"polygonPattern_schema.json":{title:"Pattern",type:"object",description:"The pattern used to render the fill of the polygon (only applies to PolygonSymbol3D).",properties:{style:{type:"string",description:"String value representing predefined styles that can be set as polygon fills.",enum:["backward-diagonal","cross","diagonal-cross","forward-diagonal","horizontal","none","solid","vertical"]},type:{type:"string",description:"The type of pattern applied to the polygon fill.",enum:["style"]}},required:["style","type"],additionalProperties:!1},"iconSymbol3DLayer_resource_schema.json":{title:"IconSymbol3DLayer Resource",type:"object",description:"The shape (primitive) or image URL (href) used to visualize the features.",properties:{dataURI:{type:"string",description:"an image encoded as base64 string, starting with `data:image/`",pattern:"^data:image/(.|\\n|\\r)+$"},href:{description:"URL to the returned image.",oneOf:[{type:"string",description:"An absolute URL",pattern:"^https?://.+$"},{type:"string",description:"A relative path starting with ./",pattern:"^\\./.+$"}]},primitive:{type:"string",description:"Specifies the type of symbol used.",enum:["circle","cross","kite","square","triangle","x"]}},oneOf:[{required:["primitive"]},{required:["href"]},{required:["dataURI"]}],additionalProperties:!1},"lineCap_schema.json":{title:"Line cap",type:"string",description:"Shape of the tips at the start and end of each line geometry. This also applies to the tips of each pattern segment along the line.",enum:["butt","round","square"],default:"butt"},"lineMarker_schema.json":{title:"Line marker",type:"object",description:"Represents markers placed at the start and end of each line geometry, or both. Markers size is proportional to the width of the line.",properties:{color:{description:"An option to color the markers differently from the line. By default the markers inherit the line's color.",$ref:"#/definitions/color_schema.json"},placement:{type:"string",description:"Indicates where the marker is placed.",enum:["begin","begin-end","end"]},style:{type:"string",description:"Style of the marker.",enum:["arrow","circle","cross","diamond","square","x"]},type:{type:"string",description:"The type of marker applied to a line.",enum:["style"]}},required:["type"],additionalProperties:!1},"linePattern_schema.json":{title:"Line pattern",type:"object",description:"A pattern used to render a line.",properties:{style:{type:"string",description:"String value representing the pattern used to render a line.",enum:["dash","dash-dot","dash-dot-dot","dot","long-dash","long-dash-dot","null","short-dash","short-dash-dot","short-dash-dot-dot","short-dot","solid"]},type:{type:"string",description:"The type of pattern applied to a line.",enum:["style"]}},required:["style","type"],additionalProperties:!1},"objectSymbol3DLayer_resource_schema.json":{title:"ObjectSymbol3DLayer Resource",type:"object",description:"The primitive shape (primitive) or external 3D model (href) used to visualize the points.",properties:{href:{oneOf:[{type:"string",description:"An absolute URL",pattern:"^https?://.+$"},{type:"string",description:"A relative path starting with ./",pattern:"^\\./.+$"}]},primitive:{type:"string",enum:["cone","cube","cylinder","diamond","invertedCone","sphere","tetrahedron"]}},oneOf:[{required:["primitive"]},{required:["href"]}],additionalProperties:!1},"pathSymbol3DLayer_schema.json":{title:"PathSymbol3DLayer",type:"object",description:"PathSymbol3DLayer renders polyline geometries by extruding a 2D profile along the line, resulting in visualizations like tubes, walls, etc.",properties:{anchor:{type:"string",description:"The position of the extrusion profile with respect to the polyline geometry.",enum:["bottom","center","top"],default:"center"},cap:{$ref:"#/definitions/pathCap_schema.json"},castShadows:{type:"boolean",description:"Boolean to control the shadow casting behaviour of the rendered geometries.",default:!0},enable:{type:"boolean"},height:{type:"number",description:"Path height in meters. If unspecified, it is equal to `width`.",minimum:0},join:{type:"string",description:"Shape of the intersection of two line segments.",enum:["bevel","miter","round"],default:"miter"},material:{$ref:"#/definitions/material_schema.json"},profile:{type:"string",description:"The shape which is extruded along the line.",enum:["circle","quad"],default:"circle"},profileRotation:{type:"string",description:"Specifies the axes about which the profile may be rotated at the joins. Constraining the rotation axes leads to a fixed orientation of the profile for the specified directions.",enum:["all","heading"],default:"all"},size:{type:"number",description:"Path size (diameter) in meters. Ignored if either `width` or `height` are present.",minimum:0},type:{type:"string",description:"Specifies the type of symbol used.",enum:["Path"]},width:{type:"number",description:"Path width in meters. If unspecified, it is equal to `height`.",minimum:0}},anyOf:[{required:["size","type"]},{required:["type","width"]},{required:["height","type"]}],additionalProperties:!1},"visualVariableLegendOptions_schema.json":{title:"Visual Variable Legend Options",type:"object",description:"Options available for the legend for visual variables.",properties:{customValues:{type:"array",items:{type:"number"}},showLegend:{type:"boolean",description:"Indicates whether to show the color/size/opacity ramp in the legend."},title:{type:"string",description:"The title of the legend."}}},"sizeStop_schema.json":{title:"sizeStop",description:"A `sizeStop` object describes the size of the symbol at various values of the expression.",properties:{label:{type:"string",description:"Value if a label is needed on the legend for a stop."},size:{type:"number",description:"Specifies the marker size to use for the specified value."},value:{type:"number",description:"The value to be mapped to a size."}},required:["size","value"],additionalProperties:!1},"transparencyStop_schema.json":{title:"transparencyStop",description:"The transparencyStop object defines the thematic opacity ramp in a sequence of stops. At least two stops are required. The stops must be listed in ascending order based on the value of the `value` property in each stop.",properties:{label:{type:"string",description:"A string value used to label the stop in the legend."},transparency:{type:"integer",description:"A numeric transparancy value for a stop ranging from 0-100, where 0 is opaque and 100 is 100% transparent.",minimum:0,maximum:100},value:{type:"number",description:"The pixel intensity value. Describes the pixel intensity value that the color should be associated with."}},required:["transparency","value"],additionalProperties:!1},"pointCloudFixedSizeAlgorithm_schema.json":{title:"PointCloud FixedSize Algorithm",type:"object",description:"Render points with fixed real world or screen space size.",properties:{size:{type:"number",description:"Symbol size in real world units or display unit"},type:{type:"string",enum:["pointCloudFixedSizeAlgorithm"]},useRealWorldSymbolSizes:{type:"boolean",description:" If true symbol size is in meters, display unit (pt) otherwise"}},required:["size","type"],additionalProperties:!1},"pointCloudSplatAlgorithm_schema.json":{title:"PointCloud Splat Algorithm",type:"object",description:"Render points using sizes depending on point density.",properties:{scaleFactor:{type:"number",description:"Scale factor to applied to the computed point size (in real world)"},type:{type:"string",enum:["pointCloudSplatAlgorithm"]}},required:["scaleFactor","type"],additionalProperties:!1},"voxelAlphaStop_schema.json":{title:"Voxel Alpha Stop",type:"object",description:"Defines transparency stop for a transfer function style.",properties:{alpha:{type:"number",description:"Opacity of the stop in [0,1]. 1 is fully opaque.",minimum:0,maximum:1},position:{type:"number",description:"Normalized position of the stop in [0,1].",minimum:0,maximum:1}},required:["alpha","position"],additionalProperties:!1},"voxelColorStop_schema.json":{title:"Voxel Color Stop",type:"object",description:"Defines color stop for a transfer function style.",properties:{color:{$ref:"#/definitions/color_schema.json"},position:{type:"number",description:"Normalized position of the stop in [0,1].",minimum:0,maximum:1}},required:["color","position"],additionalProperties:!1},"voxelRangeFilter_schema.json":{title:"Voxel Range Filter",type:"object",description:"Defines the range to filter values from. Voxel Values outside this range will be discarded.",properties:{enabled:{type:"boolean",description:"Determines if the range filter is enabled.",default:!1},range:{type:"array",description:"Defines the minimum and maximum values of the range. Data outside of the range will be discarded.",items:{type:"number"},minItems:2,maxItems:2}},required:["range"],additionalProperties:!1},"pathCap_schema.json":{title:"Path cap",type:"string",description:"Shape of the tips at the start and end of each path geometry.",enum:["butt","none","round","square"],default:"butt"}};e.schema_definitions=i,Object.defineProperty(e,Symbol.toStringTag,{value:"Module"})}));
